#!/usr/bin/ruby -w

# Ruby beautifier, version 2.1, 09/11/2006
# Copyright (c) 2006, P. Lutus
# Released under the GPL

# Changes: 2007/07/12, Mario Pehle: optional parameter withbackup for beautifyRuby
# Changes: 2008/01/21, Mario Pehle: it's now a class

class RubyBeautifyer

  TABSIZE = 2
  TABSTR = " "

  # indent regexp tests

  INDENTEXP = [
     /^module\b/,
     /^if\b/,
     /(=\s*|^)until\b/,
     /(=\s*|^)for\b/,
     /^unless\b/,
     /(=\s*|^)while\b/,
     /(=\s*|^)begin\b/,
     /(^| )case\b/,
     /\bthen\b/,
     /^class\b/,
     /^rescue\b/,
     /^def\b/,
     /\bdo\b/,
     /^else\b/,
     /^elsif\b/,
     /^ensure\b/,
     /\bwhen\b/,
     /\{[^\}]*$/,
     /\[[^\]]*$/
  ]

  # outdent regexp tests

  OUTDENTEXP = [
     /^rescue\b/,
     /^ensure\b/,
     /^elsif\b/,
     /^end\b/,
     /^else\b/,
     /\bwhen\b/,
     /^[^\{]*\}/,
     /^[^\[]*\]/
  ]

  def makeTab(tab)
     return (tab < 0) ? "": TABSTR * TABSIZE * tab
  end

  def addLine(line,tab)
     line.strip!
     line = makeTab(tab)+line if line.length > 0
     return line + "\n"
  end
  
  def beautifyRuby(path, withbackup=true, rewrite=true)
    beautifyRubyString(File.read(path), withbackup, rewrite)
  end

  def beautifyRubyString(string, withbackup=true, rewrite=true)
     commentBlock = false
     programEnd = false
     multiLineArray = Array.new
     multiLineStr = ""
     tab = 0
     source = string
     dest = ""
     source.split("\n").each do |line|
        if(!programEnd)
           # detect program end mark
           if(line =~ /^__END__$/)
              programEnd = true
           else
              # combine continuing lines
              if(!(line =~ /^\s*#/) && line =~ /[^\\]\\\s*$/)
                 multiLineArray.push line
                 multiLineStr += line.sub(/^(.*)\\\s*$/,"\\1")
                 next
              end

              # add final line
              if(multiLineStr.length > 0)
                 multiLineArray.push line
                 multiLineStr += line.sub(/^(.*)\\\s*$/,"\\1")
              end

              tline = ((multiLineStr.length > 0)?multiLineStr:line).strip
              if(tline =~ /^=begin/)
                 commentBlock = true
              end
           end
        end
        if(commentBlock || programEnd)
           # add the line unchanged
           dest += line + "\n"
        else
           commentLine = (tline =~ /^#/)
           if(!commentLine)
              # throw out sequences that will
              # only sow confusion
              while tline.gsub!(/'.*?'/,"")
              end
              while tline.gsub!(/".*?"/,"")
              end
              while tline.gsub!(/\`.*?\`/,"")
              end
              while tline.gsub!(/\{[^\{]*?\}/,"")
              end
              while tline.gsub!(/\([^\(]*?\)/,"")
              end
              while tline.gsub!(/\/.*?\//,"")
              end
              while tline.gsub!(/%r(.).*?\1/,"")
              end
              tline.gsub!(/\\\"/,"'")
              OUTDENTEXP.each do |re|
                 if(tline =~ re)
                    tab -= 1
                    break
                 end
              end
           end
           if (multiLineArray.length > 0)
              multiLineArray.each do |ml|
                 dest += addLine(ml,tab)
              end
              multiLineArray.clear
              multiLineStr = ""
           else
              dest += addLine(line,tab)
           end
           if(!commentLine)
              INDENTEXP.each do |re|
                 if(tline =~ re && !(tline =~ /\s+end\s*$/))
                    tab += 1
                    break
                 end
              end
           end
        end
        if(tline =~ /^=end/)
           commentBlock = false
        end
     end
     if(source != dest)
       if rewrite
        # make a backup copy
        File.open(path + "~","w") { |f| f.write(source) } if withbackup
        # overwrite the original
        File.open(path,"w") { |f| f.write(dest) }
       end
     end
     if(tab != 0)
        STDERR.puts "Indentation error: #{tab}"
     end
     dest
  end
  
end


if $0 == __FILE__
  
  if !ARGV[0]
    STDERR.puts "usage: Ruby filenames to beautify."
    exit 0
  end
  ARGV.each { |path| RubyBeautifyer.new.beautifyRuby(path, false, false) }
  
end
