###---------------------------------------------------------------------
#	 Name : XrefManager.rb
#
# Description : Tools - 'Add Xref Component', 'Add Xref DWG/DXF'****, 
#		"Make Into Xref, 'Xref Settings'**** and 'Manage Xrefs' 
#		added to Plugins menu under 'Xrefs Tools...'
#		**** = Currently on PCs only.
#		On PCs it requires 'win32ole.so' in the Plugins folder (v3.0+).
#
#   Author : TIG (c) 11/2005
#
#   Usage : 
#		Add Xref Component:
#
#		A file-browser dialog asks you to select a '.skp' file to 
#		insert into the model as an "Xref Component".***  If you Cancel 
#		the browser or the file is not a '.skp' or you try to insert 
#		the model back into itself you will get an error dialog and it 
#		exits without adding anything.  You pick one and OK and it is 
#		added and is placed where you want in the model.
#		The added Xref Component is also given two new "XrefManager" 
#		entity attributes, which also identify it as an Xref Component 
#		for future Xref Manager use, named:
#		"FilePath"	i.e. path to the original skp file.
#		"FileAge"	i.e. date the original skp file was last modified.
#
#		The Add is easily Undo-able.
#
#		***An 'External (Xref) Reference' means that if you insert a 
#		component from a file and then change the original file, you 
#		can then use the Xref Manager or Xref Settings to Refresh it.
#
#
#		Add Xref DWG/DXF:
#
#		This Tool is available on 'Windows Platform' only 
#		- sorry 'Apple Mac' users !
#		A dialog asks you to confirm the DWG/DXF Import settings:
#		- Merge co-planar Faces 	- default = No
#		- Orient Faces Consistently	- default = Yes
#		- Units - Inches, Feet, Millimeters, Centimeters and Meters 
#		  [the default is your model's units, BUT you might be working 
#		   in millimeters but importing a dwg that's in meters, so 
#		   think about this bit...]
#		- Preserve Drawing Origin 	- fixed at = Yes 
#		(but you can Move it later...)
#		Usually taking the defaults is recommended...
#		Additionally at the end is a setting called "Timer" 
#		- default is 5 - see below 	for more details****...
#		A file-browser dialog then asks you to select a '.dwg/dxf' file 
#		to insert into the model as an "Xref DWG/DXF Component".  
#		If you Cancel or it's not a '.dwg/dxf' then you will get an 
#		error dialog and it exits without adding anything.  Normally 
#		you'll pick a suitable file and select OK.  It then briefly and 
#		automatically opens the Import dialog and sets the dwg/dxf's 
#		path/name and it also opens the Options dialog and sets it as 
#		you have selected...  NOTE: DO NOTHING WHILE THIS RUNS 
#		- no key strokes, mouse clicks etc or else you might affect the 
#		processing.  The Import Progress bar displays whilst the file 
#		is processed.  There is then a DWG/DXF Import Report dialog 
#		displayed.  It will auto-close after a few seconds, or you OK 
#		it to continue...
#		The selected file is added to the model as an Imported DWG/DXF 
#		Xref Component named as the dwg/dxf file 
#		- e.g. "SitePlan.dwg".  It's placed at the equivalent to the 
#		dwg/dxf's X,Y,Z in the model; you can always Move it later, BUT 
#		keeping Xrefs aligned is often critical.
#		****"Timer" option setting - an explantion:
#		There has to be a built in delay to allow the Import to finish 
#		completely before trying to add attributes etc, this is usually  
#		a matter of seconds - the time before the Import Report dialog 
#		is auto-closed and the special Xref Attributes are added to the 
#		component.  A "Timer" default of 5 should suit most PCs and 
#		dwg/dxf imports: however if you have a fast PC or if you use 
#		simple, but large, dwg/dxf files that Import relatively 
#		quickly, i.e they are perhaps taking perhaps too long for the 
#		Report to auto-close - then you can always reduce the Timer by 
#		a little to spred this this up.  Conversely if your PC is slow 
#		or if you use dwg/dxf files that are very complex and that take 
#		longer to Import you might get errors messages - such as, 'Do 
#		you really want to stop the Import...' and Xref components that 
#		don't have the "Xref[...]" prefix in their description and that 
#		don't get listed in the Xref Manager dialog because their 
#		Attributes are not set up: in that case increase the Timer 
#		setting a little and see if that helps, and then 'Refresh' any 
#		'faulty' Xref DWG/DXF with the new setting...
#		If you consistently get problems needing either excessively 
#		long or short Timer settings then please give feed-back as it 
#		might be fixable in the code itself...
#		Remember a 6Mb DWG might take 6 minutes to import - be patient,
#		Assuming the Timer is just right then the added Xref DWG/DXF 
#		Component will be given the new "XrefManager" entity 
#		Attributes, named:
#		"FilePath"	i.e. path to the original skp file.
#		"FileAge"	i.e. date the original skp file was last modified.
#		Additionally a Xref DWG/DXF Component gets attributes to cover 
#		the earlier selected Import settings: 
#		"Merge"(yes/no), "Orient"(yes/no), and "Units"(ins/ft/mm/cm/m).
#		Remember that it's initial origin is always set as the 
#		dwg/dxf's origin by default.  These additional attributes are 
#		'fixed' for that Xref Component - if you want to change
#		(e.g. you got the units wrong) then you'll have to Refresh it 
#		with the Xref Manager and re-do the settings...
#
#		The Add is easily Undo-able.
#
#
#		Make Into Xref:
#
#		Select any component in the model.  Select the 'Make Into Xref' 
#		Tool.  That component is made into an Xref Component.  If there 
#		is no selection or the selected item is not a component then 
#		there are error dialogs.  If the component is 'internal'** - 
#		then a dialog suggests you do the following - select the 
#		component in the Component Browser, Right-click and pick 
#		Save-As.  It is now saved externally and it can then made into 
#		an Xref with this Tool...  Using this Tool on an existing Xref 
#		Component will update its File-path and Last-modified 
#		Attributes to the external file's current settings*.
#		This Tool is also useful for fixing Xrefs that have problems 
#		Refreshing etc because the Component Browser's 'Delete' 
#		function has been used and/or it's been manually 'Reloaded' 
#		from the Browser - loosing some Xref Attributes in the process, 
#		or ones that have been made into ordinary components with 
#		'Bind' and you'd like them back as Xrefs - *depending on its 
#		state the confirming dialog will say ..XXX 'made into'.. or 
#		're-created as an'.. Xref Component...
#		**NOTE if you run this on an Xref DWG/DXF Component it is also 
#		'internal', so you are warned what it is, and it than exits 
#		without doing anything - if you really want to save it as an 
#		external component you can still do so manually...
#
#
#		Xref Setting:
#
#		This Tool is available on 'Windows Platform' only 
#		- sorry 'Apple Mac' users !
#		It opens a dialog which lets you set the type of Xref Checking 
#		that runs when you first open a model containing Xref 
#		Components - you can choose two options if the chosen setting 
#		is 'Open Xref Manager If Needed' and it finds anything that 
#		needs fixing then it opens 'Xref Manager' (see below).  
#		Note than changes done initially at startup are NOT 
#		subsequently undo-able.  If it is set to 'Do Nothing' then 
#		there is no initial checking.
#
#
#		Xref Manager:
#
#		This allows you to manage Xref Components.  If there are no 
#		Xref Components to manage then an error dialog tells you and it 
#		exits.  Otherwise a dialog lists each Xref Component by its 
#		type (.SKP:/.DWG:/.DXF:), name and its Status... e.g.
#		.SKP: Building-1 		  		i.e. it's all OK - path confirmed, 
#										loaded and up-to-date.
#		.SKP: Building-2 [ Un-Pathed ]  i.e. remembered file path no longer 
#										exists.
#		.SKP: Building-3 [ Un-Loaded ]  i.e. component not in the model but 
#										still in comp-browser.
#		.SKP: Building-4 [ Superseded ] i.e. original skp file has been 
#										changed recently.
#		Any DWG/DXF Xrefs are also included in the Manager list at the 
#		end and are prefixed to suit, e.g.
#		.DWG: SitePlan   [ ... ]	  i.e. a dwg of the site...
#		.DXF: Building-9 [ ... ]	  i.e. a dxf of building-9...
#
# 		They can be 'OK'd' (Un-Chnaged), 'Deleted' or 'Bound', note 
#		that on PCs only they can also be 'Refreshed' and 'Re-pathed' 
#		in a similar way as SKPs - however on non-PCs they will be auto-
#		re-set to be 'Un-Changed [PC ONLY]'... 
#
#		Note: any Xref Component can also have a combined Status 
#		- e.g. Un-Loaded + Superseded.
#
#		Each Xref Component listed has a drop-down to its right giving 
#		a choice of Actions - these are defaulted to suit the present 
#		Status and the likely Fix (1 of first 3) thus:
#
#		"OK..."	  	i.e. leaves Xref Component unchanged.
#
#		"Refresh"	i.e. refreshes Xref Component from the original 
#						file - you do not need to refresh unless its 
#						Status is shown as 'Superseded'.
#		
#		"Re-Path"	i.e. resets the path of the original file - you do 
#						not normally need to Re-Path, unless you have 
#						moved or renamed the original file and its 
#						Status is shown as 'Un-Pathed', however you 
#						might want to change a placed Xref for another 
#						file.
#
#		"Bind"		i.e. removes the "XrefManager" attributes and makes 
#						the Xref Component into an ordinary component.  
#						You can undo this with 'Make Into Xref' which 
#						will convert the component back into an Xref 
#						one.
#
#		"Delete"	i.e. deletes any Xref Component instances in the 
#						model - please use 'Delete' with care !
#
#		On picking the OK button each listed Xref Component is 
#		processed with the selected Action thus:
#
#		'Refresh' means the Xref Component is updated from a file.  The 
#		old version in the comp-browser is renamed as 
#		'#Xref-Scrap[#nn]' and becomes an ordinary unused component 
#		that will be removed from the model data-base if you select the 
#		Auto-Purge-Unused option (PC only), or when you next pick 
#		'Purge Unused' on the compo-browser dialog fly-out (a dialog 
#		asks / reminds you of this).  If you picked 'Refresh' but the 
#		file is shown as Un-Pathed then 'Re-Path' will run as set out 
#		in the paragraph below, asking you to re-define the file-path.
#
#		'Re-Path' runs a file-browser dialog that asks you to select a 
#		'.skp' file to reinsert into the model as the Xref Component.  
#		If you Cancel the browser or the file is not a '.skp' or you 
#		try to insert the model back into itself you will get an error 
#		dialog and it exits without processing that Xref Component - 
#		also any remaining on the list are also NOT processed
#		- you will need to re-run the tool.  Otherwise Re-path is 
#		similar to Refresh in the way '#Xref-Scrap[#nn]' is made...
#
#		'Bind' removes the "XrefManager" attributes and make the Xref 
#		Component into an ordinary loaded component.  You can undo 
#		'Bind' by using the 'Make Into Xref' Tool...
#
#		'Delete' erases the Xref Component Instances in the model.  
#		The version in the compo-browser is renamed as 
#		'#Xref-Scrap[#nn]' and becomes an unloaded ordinary component; 
#		it will be removed from the model data-base if you select the 
#		Auto-Purge-Unused option (PC only), or when you next pick 
#		'Purge Unused' on the compo-browser dialog fly-out (a dialog 
#		asks / reminds you of this).  Use Xref 'Delete' with care !
#
#		Note:  DWG/DXF Xrefs can only be Added or Refreshed/Repathed on 
#		a PC, so if the model is subsequently opened on a Mac system 
#		and Xref Manager is invoked then the  options are limited for 
#		that type of Xref - see above...
#
#		It is recommended that you Refresh only ONE DWG/DXF at a time, 
#		otherwise it can take a long time and sometimes errors might 
#		occur with multiple refreshings (multiple SKP's are OK).
#
#		If you cancel out of a Re-Path during an Xref Manager session 
#		it might not all complete successfully and an error dialog 
#		warns you.
#		At the end of processing the Xrefs that were listed then a 
#		report dialog is displayed showing what was done.
#		(Note: A currently known, but unidentified, bug sometimes means 
#		that to close this dialog more than one click on 'OK' might be 
#		required ! )
#
#		All Actions Undo - depending on the list and action then more 
#		than one Undo might be needed.
#
#		Notes:
#
#		Do NOT manually 'Reload' an Xref Component from the Component 
#		Browser or Context-Menu  - it'll lose its Xref Attributes (the 
#		same result as 'Bind').  But you can use the 'Make Into Xref' 
#		tool to convert it back into an Xref Component...
#
#		From v2.3 on PCs only AND Mac using SUp v6
#   'old' Xref Components will be removed 
#		from the model data-base if you select the Auto-Purge-Unused 
#		option, or when you next pick 'Purge Unused' on the compo-
#		browser dialog fly-out (a closing dialog asks / reminds you of 
#		this).  Do NOT user the Component Browser 'Delete' option to 
#		remove former Xrefs - this might cause the current Xref to fail 
#		to Refresh properly.  You will need to Reload it from the 
#		Component Browser and use the 'Make Into Xref' tool to convert 
#		it back into an Xref Component...
#
#		Type : Tool
#
#     	Version :
#		1.0	 26/11/05	First issue.
#		1.1  27/11/05	Error with mpath = "" if unsaved now trapped.
#		1.2  28/11/05	Error with menu call typo fixed.
#		1.3  03/12/05	Xref Class added ( for TBD ! ).
#		1.4  10/12/05	'Add Xref' dialog now has [0,0,0] placement 
#						clue in title-bar.  'Refresh/Repath/Delete' 
#						reminders now only one per dialog session.
#						'Xref Settings' Tool added to allow settings 
#						for Xref.check which runs at model open, if 
#						needed suggests use 'Xref Manager'.  'Make Into 
#						Xref' Tool added.  Xref Component description 
#						now prefixed with 'XREF: '.
#		1.5  12/12/05	Name of script corrected to 'XrefManager.rb' 
#						- doh ;-}
#		1.6  20/12/05	Start_operation glitch trapped for Mac users.
#		1.7  21/12/05	Mac friendliness improved ?  Undos improved.
#		1.8  21/12/05pm	Mac friendliness re-improved ?
#		1.8a 21/12/05pm Mac version = cut-down - no setting/check 
#						(limited issue)...
#		1.9  21/12/05pm Forced refresh/repath improved.  Setting/Check 
#						reinstated (at end).  Xref.make improved.
#		2.0  22/12/05	Xref.check waits for model info to load then 
#						runs Xref.manager.  Report after Xref.manager 
#						layout improved.
#		2.1  22/12/05	Mac cannot now use Xref.setting/check option 
#						- saves failures.
#		2.2  22/12/05 	Platform checking improved.
#		2.3  25/01/06	Automatic Purge-Unused (PC only) added.
#		3.0  06/02/06	Add Xref id split into Component(SKP) & DWG/DXF  
#						(PC only).  Xref Manager now identifies 
#						SKP/DWG/DXF Xrefs separately, currently some 
#						actions for these are only available on PC.
#		3.1  08/02/06	Timer default etc and dwg management adjusted 
#						following beta testing.
#		3.2  27/02/06	Timer issues resolved.
#		3.3  10/05/06	Version fix for extra import types, Naming of  
#						dxf and DXF etc allowed.
#		3.4  12/05/06	Typos corrected etc.
#		3.5  10/06/06	Manager Message glitch fixed if only skp's.
#		3.6  11/06/06	More Manager glitches fixed.
#   4.0  13/01/07 Updated to work with SUp v6.
#   4.1  14.01.07 Trapped for empty model not making a component of its 
#                 first imported DWG/DXF.
#   4.2  15/01/07 PC AND Mac Auto-Purge All now available if SUp 6.
#                 Glitch with multiple Manager report dialogs fixed.
#   5.0  05/03/08 Glitch with Make Xref fixed.
#	5.1	 07/03/08 Menu error fixed (sorry !).
###---------------------------------------------------------------------
require 'sketchup.rb'
###---------------------------------------------------------------------
class Xref ### v1.3
###
def Xref::add_component
   model = Sketchup.active_model
   mpath = model.path
   entities = model.entities
   definitions = model.definitions
   view = model.active_view
   model.start_operation "Add Xref"
### show VCB and status info
   Sketchup::set_status_text(("ADD XREF COMPONENT..." ), SB_PROMPT)
   Sketchup::set_status_text(" ", SB_VCB_LABEL)
   Sketchup::set_status_text(" ", SB_VCB_VALUE)
### dialog
   if mpath == "" ### v1.1 / v1.2
      UI.beep
      UI.messagebox("You must save the 'Untitled' new model before Adding Xrefs !\nExiting... ")
      view.invalidate
      return nil
   end
   xpath = UI.openpanel "Select an Xref '.skp' File to Insert... Insert at [0,0,0] to keep origin."###v1.4
   if not xpath
      UI.beep
      UI.messagebox("No Xref Added !\nExiting... ")
      view.invalidate
      return nil
   end
   if xpath[-4,4] != ".skp"
      UI.beep
      UI.messagebox("That's NOT a '.skp' file !\nNo Xref Added !\nExiting... ")
      view.invalidate
      return nil
   end
   if File.stat(xpath) == File.stat(mpath)
      UI.beep
      UI.messagebox("You CANNOT insert a file into itself !\nExiting... ")
      view.invalidate
      return nil
   end
### load as component
   xdef = definitions.load xpath
### add XREF: to description ### v1.4
   xdef.description= "XREF: "+xdef.description ### v1.4
### add attributes
   xdef.set_attribute "XrefManager", "FilePath", xpath
   xdef.set_attribute "XrefManager", "FileAge", File.mtime(xpath.to_s).to_f
### place componnet
   model.place_component xdef,false
### commit undo
   model.commit_operation
### update view
   view.invalidate
end #def add_component
   
### ####################################################################
def Xref::add_dwg(xpath,xname) ### only runs on PCs ### v3.0
   model = Sketchup.active_model
   mpath = model.path
   entities = model.active_entities
   definitions = model.definitions
   view = model.active_view
   model.start_operation "Add DWG\\DXF Xref"
   if entities[0] ###v4.1
    temp = nil
   else
    temp = entities.add_line([0,0,0],[1.mm,1.mm,1.mm])
    entities = model.active_entities
   end#if ###v4.1
### get array of initial entities
   @ents=[]
   for e in entities
      @ents=@ents.push(e)
   end#for e
### show VCB and status info
   Sketchup::set_status_text(("ADD XREF DWG/DXF COMPONENT..." ), SB_PROMPT)
   Sketchup::set_status_text(" ", SB_VCB_LABEL)
   Sketchup::set_status_text(" ", SB_VCB_VALUE)
### dialog
   if mpath == ""
      UI.beep
      UI.messagebox("You must save the 'Untitled' new model before Adding Xrefs !\nExiting... ")
      temp.erase! if temp### v4.1
      view.invalidate
      return nil
   end
   xmsg="Select a '.dwg' or '.dxf' File to Insert as an Xref..."
   xmsg="Select a '.dwg' or '.dxf' File to Insert as Xref ["+xname+"]..." if xname != ""
   if xpath == "?"
      @xpath = UI.openpanel xmsg
   else
      @xpath = xpath ###dwg-path
    end#if

   if not @xpath or @xpath=="?" ###cancelled
      UI.beep
      if xname == ""
         UI.messagebox("No Xref DWG/DXF Added !\nExiting... ")
      else
         UI.messagebox("Xref DWG/DXF NOT Re-Pathed !\nALSO Some Other Items Might NOT Have Been Completed !\nExiting... ")
      end#if xname
      temp.erase! if temp### v4.1
      view.invalidate
      return nil
   end
   if @xpath[-4,4] != (".dwg" or ".DWG")###v3.3
      if @xpath[-4,4] != (".dxf" or ".DXF")###v3.3
         UI.beep
         UI.messagebox("That's NOT a '.dwg' or '.dxf' file !\nNo DWG/DXF Xref Added !\nExiting... ")
         temp.erase! if temp### v4.1
         view.invalidate
         return nil
      end
   end
### get options - dialog...
   @merge = "No" if not @merge
   @orient="Yes" if not @orient
   @origin="Yes" if not @origin
### get current units
currentunits1=Sketchup.format_length 120000
units="Inches" ### default
if currentunits1.to_s.split(".")[0].split(",")[0].split("'")[0].split("\"")[0].split("c")[0].split("m")[0]=="120000"
   units="Inches"
end
if currentunits1.to_s.split(".")[0].split(",")[0].split("'")[0].split("\"")[0].split("c")[0].split("m")[0]=="10000"
   units="Feet"
end
if currentunits1.to_s.split(".")[0].split(",")[0].split("'")[0].split("\"")[0].split("c")[0].split("m")[0]=="3048000"
   units="Millimeters"
end
if currentunits1.to_s.split(".")[0].split(",")[0].split("'")[0].split("\"")[0].split("c")[0].split("m")[0]=="304800"
   units="Centimeters"
end
if currentunits1.to_s.split(".")[0].split(",")[0].split("'")[0].split("\"")[0].split("c")[0].split("m")[0]=="3048"
   units="Meters"
end
@units=units if not @units
@timer=5 if not @timer
###
   prompts= ["Merge Co-planar Faces: ","Orient Faces Consistently: ","Units: ","Preserve Drawing Origin: ","'Timer': "]
   title= "Import AutoCAD DWG/DXF Options"
   values= [@merge,@orient,@units,@origin,@timer]
   popups= ["Yes|No","Yes|No","Inches|Feet|Millimeters|Centimeters|Meters","Yes",""]
   results= inputbox( prompts, values, popups, title )
   if not results
     temp.erase! if temp && (not results)### v4.1
     return nil
   end#if
### do processing of results
   @merge =results[0]
   @orient=results[1]
   @units =results[2]
   @origin=results[3]
   @timer =results[4].to_i
   @timer = 1 if @timer < 1
### import
   #~ if not Sketchup.find_support_file "win32ole.so"
      #~ UI.messagebox("This requires 'win32ole.so' to be in the Plugins folder ! \nExiting... ")
      #~ return nil
   #~ end #if

   model.import "#{@xpath}",Sketchup.version_number>=6004112 ? false :     true
   Xref.add_dwg_attrib
=begin   
   require 'win32ole.so'
   Sketchup.send_action 21933 ###open Import dialog
   s=WIN32OLE.new("WScript.Shell")
   s.SendKeys @xpath
   s.SendKeys "{TAB}"
   s.SendKeys "{DOWN}"
   s.SendKeys "{UP 8}" if Sketchup.version_number < 5000260 ###v3.3 fix for new import types 
   s.SendKeys "{DOWN 2}" if Sketchup.version_number < 5000260 ###v3.3 fix for new import types 
   s.SendKeys "{UP 10}" if Sketchup.version_number >= 5000260 ###v3.3 fix for new import types 
   s.SendKeys "{DOWN 3}" if Sketchup.version_number >= 5000260 ###v3.3 fix for new import types 
   s.SendKeys "{UP 2}" if Sketchup.version_number >= 6000277 ###### v4.0 fix for SUp6 dialog list changes
   s.SendKeys "{ENTER}"
   s.SendKeys "{TAB 3}"
   s.SendKeys "%{p}"### options
   ### merge
   if @merge=="Yes"
      s.SendKeys "{+}"
   else
      s.SendKeys "{-}"
   end#if
   s.SendKeys "{TAB}"### orient
   if @orient=="Yes"
      s.SendKeys "{+}"
   else
      s.SendKeys "{-}"
   end#if
   s.SendKeys "{TAB}"### units
   s.SendKeys "{DOWN 4}"
   s.SendKeys "{UP 4}" if @units=="Inches"
   s.SendKeys "{UP 3}" if @units=="Feet"
   s.SendKeys "{UP 2}" if @units=="Millimeters"
   s.SendKeys "{UP 1}" if @units=="Centimeters"
   ### otherwise it's Meters...
   s.SendKeys "{TAB}"### origin always on
   s.SendKeys "{+}"
   s.SendKeys "{ENTER}"
   s.SendKeys "{TAB}"
   s.SendKeys "{ENTER}"
### set delay to suit size of dwg...
   fsize=File.size(@xpath)###in bytes
   div=375000/@timer
   @delay=(fsize/div).to_i 
   ### ~1 second per 1kb - BUT allow for v.small and big files
   ### adjust delay for dxf file's larger size, 
   ### but processing time is about the same...
   @delay=(@delay/6).to_i if @xpath[-4,4] == ".dxf"
   ###
   @delay=(@delay*4).to_i if @delay > 90
   @delay=(@delay*3).to_i if @delay > 80
   @delay=(@delay*2).to_i if @delay > 70
   @delay=(@delay*1.5).to_i if @delay > 60
   @delay=(@delay*1.30).to_i if @delay > 50
   @delay=(@delay*1.15).to_i if @delay > 40
   @delay=(@delay*1.05).to_i if @delay > 30
   @delay=3 if @delay < 3 ### minimum
   ###puts @delay ###
   ###
   UI.start_timer(@delay,false){Xref.add_dwg_attrib}
   ### TBD tip - thanks !
   ###@delay=0 ###zero delay
   ### commit undo
   UI.start_timer(@delay+0.1,false){temp.erase! if temp}### v4.1
   s.SendKeys "{ENTER}"
=end   
   model.commit_operation
   
   return true
end #def add_dwg
   
### ####################################################################
def Xref::add_dwg_attrib ### v3.0
   #~ s=WIN32OLE.new("WScript.Shell")
   #~ s.SendKeys "{ENTER}"### close report dialog
   model = Sketchup.active_model
   entities = model.entities
   model.start_operation "Add Xref Attributes"
### sort attribs
cents=[]
for e in entities
   cents=cents.push(e)
end#for e
@xdwg=(cents-@ents)
@xdwg=@xdwg[0]
###
   @xdef=@xdwg.definition
### add XREF: to description etc 
   @xdef.description= "XREF[DWG/DXF]: "+@xdef.description
   ### 9 more chars than plain xref 
### add attributes
   @xdef.set_attribute "XrefManager", "FilePath", @xpath
   @xdef.set_attribute "XrefManager", "FileAge", File.mtime(@xpath.to_s).to_f
   @xdef.set_attribute "XrefManager", "Merge", @merge
   @xdef.set_attribute "XrefManager", "Orient", @orient
   @xdef.set_attribute "XrefManager", "Units", @units
   ### wait till done...
   ###UI.messagebox("Xref DWG/DXF Imported. ")
   ### commit undo
   model.commit_operation
end#def add_dwg_attrib
   
### ####################################################################
def Xref::make ### v1.4
   model = Sketchup.active_model
### start undo
   model.start_operation "Make into Xref"
   view = model.active_view
   ss = model.selection
###
   if ss.empty?
      Sketchup::set_status_text("Make into Xref... NO SELECTION ! ", SB_PROMPT)
      UI.messagebox("No Selection ! ")
      return nil
   end
###
   if ss.length > 1
      Sketchup::set_status_text("Make into Xref... MULTIPLE SELECTION ! ", SB_PROMPT)
      UI.messagebox("Select only ONE Component ! ")
      return nil
   end
###
   if ss[0].typename != "ComponentInstance"
      Sketchup::set_status_text("Make into Xref... SELECTION IS NOT A COMPONENT ! ", SB_PROMPT)
      UI.messagebox("Select ONE COMPONENT Instance ! ")
      return nil
   end
### dwg v3.0
   if (ss[0].get_attribute "XrefManager", "FilePath","")[-4,4]==".dwg" or (ss[0].get_attribute "XrefManager", "FilePath","")[-4,4]==".dxf"
      Sketchup::set_status_text("Make into Xref... SELECTED COMPONENT MADE FROM AN EXTERNAL DWG/DXF - SO IT CAN'T BE CHANGED ! ", SB_PROMPT)
      UI.messagebox("Selected Component made from an External DWG/DXF - so it CAN'T be changed ! ")
      return nil
   end
###
   if ss[0].definition.internal?
      Sketchup::set_status_text("Make into Xref... SELECTED COMPONENT IS NOT IN AN EXTERNAL FILE ! ", SB_PROMPT)
      UI.messagebox("Selected Component is NOT in an External File !\nSelect it [in the Component Browser], Right-click and pick 'Save-As'. \nThen re-run this Tool on it... ")
      return nil
   end
###
### So - we have 1 external component...
   xdef = ss[0].definition
   xname = xdef.name
   xpath = xdef.path
### add XREF: to description ### v1.4 / v1.9
   rep="re-created as an" if xdef.get_attribute "XrefManager", "FilePath"
   rep="made into" if not xdef.get_attribute "XrefManager", "FilePath"
   xdef.description= "XREF: "+xdef.description if not xdef.get_attribute "XrefManager", "FilePath"
###
   xdef.set_attribute "XrefManager", "FilePath", xpath
   xdef.set_attribute "XrefManager", "FileAge", File.mtime(xpath.to_s).to_f
###
   UI.messagebox("'#{xname}' has been #{rep} an Xref Component. ")
### commit undo
   model.commit_operation
### update view
   view.invalidate
end #def make ### v1.4
   
### ####################################################################
### v1.8a/v1.9 ### reinstated ###
def Xref::setting ### v1.4
   model = Sketchup.active_model
### dialog
   prompts = ["Pick Type of Xref Checking on Opening: "]
   types = ["Open Xref Manager If Needed", "Do Nothing"]
   title = "Xref Settings"
   values = [model.get_attribute("XrefManager", "Settings", types[0])]
   popups = [types.join("|")]
   results = inputbox( prompts, values, popups, title )
   return nil if not results
### do processing of results
   settings = results[0]
   model.set_attribute "XrefManager", "Settings", settings
end #def setting ### v1.4


### ####################################################################
def Xref::manager
   @ended=false
   pc = false
   pc = true if [PLATFORM].grep(/mswin/) == [PLATFORM] ### v3.0
   model = Sketchup.active_model
   ### start undo
   model.start_operation "Xref Manager"
   mpath = model.path
   entities = model.entities
   definitions = model.definitions
   view = model.active_view
### look in model for all xref compos  - that are currently inserted
   xrefs = []
   xrefs_unloaded = []
   xrefs_dwgs = []
   xrefs_dwgs_unloaded = []
   for xdef in definitions
      if not (xdef.group?) and (xdef.get_attribute "XrefManager","FilePath")###compo xref v3.6 
       if not (xdef.get_attribute "XrefManager","Units")###not dwg ###v3.6
         if (xdef.count_instances > 0)
            pathx = (xdef.get_attribute "XrefManager", "FilePath", "???")
            age = (xdef.get_attribute "XrefManager", "FileAge", 0)
            if FileTest.exist?(pathx.to_s)
               pathed = ""
               if File.mtime(pathx.to_s).to_f > age
                  aged = " [Superseded]"
               else
                  aged = ""
               end
            else
               pathed = " [Un-Pathed] "
               aged = ""
            end
            loaded = ""
            xrefs = xrefs.push( [ xdef, pathx, pathed, loaded, aged ] )
         else ### the unloaded xref compos
            if FileTest.exist?(pathx.to_s)
               pathed = ""
               if File.mtime(pathx.to_s).to_f > age
                  aged = " [Superseded]"
               else
                  aged = ""
               end
            else
               pathed = " [Un-Pathed] "
               aged = ""
            end
            loaded = " [Un-Loaded] "
            xrefs_unloaded = xrefs_unloaded.push( [ xdef, pathx, pathed, loaded, aged ])
         end #if instances
         ###
      else ### so then it's a dwg v3.0 ---------------------------------
         ###
         if (xdef.count_instances > 0)
            pathx = (xdef.get_attribute "XrefManager", "FilePath", "???")
            age = (xdef.get_attribute "XrefManager", "FileAge", 0)
            if FileTest.exist?(pathx.to_s)
               pathed = ""
               if File.mtime(pathx.to_s).to_f > age
                  aged = " [Superseded]"
               else
                  aged = ""
               end
            else
               pathed = " [Un-Pathed] "
               aged = ""
            end
            loaded = ""
            xrefs_dwgs = xrefs_dwgs.push( [ xdef, pathx, pathed, loaded, aged ] )
         else ### unloaded dwg compo
            if FileTest.exist?(pathx.to_s)
               pathed = ""
               if File.mtime(pathx.to_s).to_f > age
                  aged = " [Superseded]"
               else
                  aged = ""
               end
            else
               pathed = " [Un-Pathed] "
               aged = ""
            end
            loaded = " [Un-Loaded] "
            xrefs_dwgs_unloaded = xrefs_dwgs_unloaded.push( [ xdef, pathx, pathed, loaded, aged ])
         end #if instances
       end #if compo
     end #if not group###v3.6
   end #for xdef
###
   allxrefs_skps = (xrefs.sort! + xrefs_unloaded.sort!).compact
   allxrefs_dwgs = (xrefs_dwgs.sort! + xrefs_dwgs_unloaded.sort!).compact
   allxrefs = allxrefs_skps + allxrefs_dwgs ### v3.0
###
   if not allxrefs[0]
      UI.messagebox("There are NO Xrefs to Manage !\nExiting... ")
      return nil
   end
### show VCB and status info
   Sketchup::set_status_text(("XREF MANAGER..." ), SB_PROMPT)
   Sketchup::set_status_text(" ", SB_VCB_LABEL)
   Sketchup::set_status_text(" ", SB_VCB_VALUE)
### dialog
   prompts = []
   for x in allxrefs ### v3.0
      typ=".SKP: " if (x[0].get_attribute "XrefManager", "FilePath")[-4,4] == ".skp"
      typ=".DWG: " if (x[0].get_attribute "XrefManager", "FilePath")[-4,4] == ".dwg"
      typ=".DXF: " if (x[0].get_attribute "XrefManager", "FilePath")[-4,4] == ".dxf"
      prompts = prompts.push(typ+x[0].name + "  : " + x[2] + x[3] + x[4] + "  ")
   end #for x
   types = [ "OK...", "Refresh", "Re-Path", "Bind", "Delete" ]
   title = "Xref Manager"
   xlen = allxrefs.length
   values = []
   0.upto(xlen - 1) do |i|
      if allxrefs[i][2] == " [Un-Pathed] "
         values = values.push(types[2]) 
      elsif allxrefs[i][4] != ""
         values = values.push(types[1])
      else
         values = values.push(types[0])
      end #if aged
   end # upto
   popups = []
   1.upto(xlen) do |i|
      popups = popups.push(types.join("|"))
   end # upto
   results = inputbox( prompts, values, popups, title )
   return nil if not results
### do processing of results ###
   msg="Xref Manager Report:\n" ###v2.0
   ###
   0.upto(results.length - 1) do |r|
      ### get type  skp or dwg
      type=(allxrefs[r][0].get_attribute "XrefManager", "FilePath")[-4,4]
      ### ".skp" or ".dwg or .dxf"
      ### dump Mac dwg out
      mbit=""
      if not pc and type != ".skp"
         if results[r] == types[1] or results[r] == types[2]
            UI.messagebox("SORRY, BUT you CAN'T Refresh/Re-Path an Xref based on a DWG/DXF,\nUNLESS you are using a PC !\nTherefore it will remain 'Unchanged'...  ")
            results[r]=types[0]
            mbit=" [PC-ONLY]"
         end# refresh/repath
      end# if type skp
      ###---------------------------------------------------------------
      if results[r] == types[0] ###v2.0
         xold = allxrefs[r]
         xodef = xold[0]
         xn = xodef.name
         msg = msg+"\n"+xn+" : unchanged"+mbit+"."
      end#if------------------------------------------------------------
      
      if results[r] == types[1] or results[r] == types[2]
         xold = allxrefs[r]
         xodef = xold[0]
         xpath = xold[1]
         xpathed = xold[2]
         xname = xodef.name
         rp=0###v4.0
        if type == ".skp"
         ### Refresh OR Repath
         rp=0###v3.0
         if xpathed != "" or results[r] == types[2] ### RePath anyway
            rp=1###v2.0
            xpath = UI.openpanel ("Re-Path... Select an Xref '.skp' File [ for '" + xname + "' ] ")###v1.4
            if not xpath
    	       UI.beep
               UI.messagebox("No Xref Re-Pathed for '" + xname + "' !\nExiting... ")
               return nil
            end
            if xpath[-4,4] != ".skp"
               UI.beep
               UI.messagebox("That's NOT a '.skp' file !\nNo Xref Re-Pathed for '" + xname + "' !\nExiting... ")
               return nil
            end
            if File.stat(xpath) == File.stat(mpath)
               UI.beep
               UI.messagebox("You CANNOT insert a file into itself !\nNo Xref Re-Pathed for '" + xname + "' !\nExiting... ")
               return nil
            end
          end #if
          ### v1.9 ### ensure it is loaded by using a while test ###
          if rp==0 or rp==1
             dcount=definitions.count
             xcount=dcount
             while dcount == xcount ###
                xdef=definitions.load xpath
                definitions.unique_name xdef.name
                xcount=definitions.count
                ### swap names around
                xodef.name= "#Xref-Scrap"
                instances = xodef.instances
                for i in instances ### exchange
                   i.definition= xdef
                end #for
                ### delete old's attribues
                xodef.delete_attribute "XrefManager"
                ### add new's attribues
                xdef.set_attribute "XrefManager", "FilePath", xpath
                xdef.set_attribute "XrefManager", "FileAge", File.mtime(xpath.to_s).to_f
                ### add XREF: to description ### v1.4
                xdef.description= "XREF: "+xdef.description ### v1.4
                ### add FORMER XREF: to description ### v1.4
                xodef.description= "FORMER "+xodef.description ### v1.4
                xdef.name= xname
             end#while ###
             msg = msg+"\n"+xname+" : .skp refreshed." if rp==0
             msg = msg+"\n"+xname+" : .skp re-pathed." if rp==1
          end#if
        else ### a dwg
           @merge=xodef.get_attribute "XrefManager", "Merge"
           @orient=xodef.get_attribute "XrefManager", "Orient"
           @units=xodef.get_attribute "XrefManager", "Units"
           @xodef=xodef
           (return nil if not Xref.add_dwg(xpath,xname))if results[r]== types[1]
           (return nil if not Xref.add_dwg("?",xname)) if results[r] == types[2]
           UI.start_timer(1.4*@delay,false){###start block###v3.1
            @xdwg.erase!
            ### renaming and swapping
            @xodef.name= "#Xref-Scrap"
            instances = @xodef.instances
            for i in instances ### exchange
                i.definition= @xdef
            end #for
            ### delete old's attribues
            @xodef.delete_attribute "XrefManager"
            ### add FORMER XREF: to description ### v1.4
            @xodef.description= "FORMER "+@xodef.description
            @xdef.name= xname
            ###### then make msg
            msg = msg+"\n"+xname+" : .dwg/.dwf refreshed."
           }###end block
        end#if
      end #if refresh/repath--------------------------------------------
      
      if results[r] == types[3] ### Bind
         xold = allxrefs[r]
         xodef = xold[0]
         xn = xodef.name
         ### delete old's attribues
         xodef.delete_attribute "XrefManager"
         ### remove 'XREF: ' from description ### v1.4
         xodef.description= xodef.description.slice(6..-1)+""### v1.4
         xodef.description= xodef.description.slice(9..-1)+"" if type!=".skp"###v3.0
         msg = msg+"\n"+xn+" : bound."
      end #if bind------------------------------------------------------
      
      if results[r] == types[4] ### Delete
         xold = allxrefs[r]
         xodef = xold[0]
         xn = xodef.name
         ### delete old's attribues
         xodef.delete_attribute "XrefManager"
         ### erase all instances
         ### rename scrap
         xodef.name= "#Xref-Scrap"
         instances = xodef.instances
         for i in instances ### erase
           i.erase!
         end #for
         ### add FORMER XREF: to description ### v1.4
         xodef.description= "FORMER "+xodef.description ### v1.4
         msg = msg+"\n"+xn+" : deleted."
         ### purge old defn - BUT HOW ?
      end #if delete----------------------------------------------------
      
   ### commit undo
   ###model.commit_operation
   end #upto r----------------------------------------------------------

   if msg != "Xref Manager Report:\n" ### v4.0
      delay=1 if not @delay
      delay=@delay+1 if @delay
      delay=delay*results.length
      @once=true ###v4.2
      UI.start_timer(delay,false){###start block
         if @ended && @once
           @once=false
           Xref.manager_message(pc,msg)
         end#if ###v4.2
      }###end block
   end #if msg ### v4.0
###
   @ended=true   
### commit undo
   model.commit_operation
### update view
   view.invalidate
   
end #def manager

###------###
def Xref::manager_message (xpc,xmsg)### v4.0 separated out...
     if xpc || Sketchup.version_number >= 6000277 ###pc OR pc/mac SUp v6### v4.2
      yn = UI.messagebox(xmsg+"\n\nChoose 'YES' to run 'Purge-Unused' Automatically on ALL 'Unused' Components NOW...\n\nIF 'NO', then remember to use the Component Browser's 'Purge-Unused' fly-out\nto remove any 'Unused' Xref Components later.  Do NOT 'Delete' them in the Browser...   ",MB_YESNO)
      Sketchup.active_model.definitions.purge_unused if yn == 6  ### v4.2
      ###Sketchup.send_action 21086 if yn == 6 
    else ### =MAC SUp <=v5
      UI.messagebox(xmsg+"\n\nRemember to use the Component Browser's 'Purge-Unused' fly-out\nto remove 'Unused' Xref Components later.  Do NOT 'Delete' them in the Browser...   ")
    end #xmsg xpc
end #def Xref.manager_message
###------##

### ####################################################################
### v1.4 / v1.8a / v1.9 / v2.0 / v3.0
def Xref::check
   model = Sketchup.active_model
   definitions = model.definitions
### look in model for all xref comps
### - that are currently inserted
   allxrefs = []
   for xdef in definitions
      if (not xdef.group?) and (xdef.get_attribute "XrefManager", "FilePath")
         if (xdef.count_instances > 0) ### inserted
            age =  (xdef.get_attribute "XrefManager", "FileAge", 0)
            pathx = (xdef.get_attribute "XrefManager", "FilePath", "???")
            if FileTest.exist?(pathx.to_s) ### NOT unpathed
               if File.mtime(pathx.to_s).to_f > age
                  allxrefs = allxrefs.push([ xdef, pathx ])
               end
            end
         end
      end
   end#for xdef
###
   if allxrefs[0] and (model.get_attribute "XrefManager", "Settings", "Do Nothing") != "Do Nothing"
      Xref.manager
  end#if allxrefs[0]
end #def check ### v1.4 / v1.8a /v1.9 / v2.0
########################################################################
###
end #class ### v1.3
###
   
### menu bits  #########################################################
#~ if( not file_loaded?("XrefManager.rb") ) ###
#~ ###if( not file_loaded?("99-XrefManager.rb") ) ###
   #~ UI.menu("Plugins").add_separator###
   #~ xref_menu = UI.menu("Plugins").add_submenu("Xrefs Tools...")
   #~ ###xref_menu = $submenu9.add_submenu("Xrefs Tools...")###
   #~ xref_menu.add_item("Add Xref Component") { Xref.add_component }### v3.0
   #~ xref_menu.add_item("Add Xref DWG\\DXF") { Xref.add_dwg("?","") } if [PLATFORM].grep(/mswin/) == [PLATFORM]### v3.0 ###v3.4
   #~ xref_menu.add_item("Make Into Xref") { Xref.make }
   #~ xref_menu.add_item("Xref Setting") { Xref.setting } if [PLATFORM].grep(/mswin/) == [PLATFORM]
   #~ xref_menu.add_item("Xref Manager") { Xref.manager  }
#~ end
#~ ###
#~ ###file_loaded("99-XrefManager.rb") ###
#~ file_loaded("XrefManager.rb") ###
   
### ####################################################################
### Sketchup.send_action "showRubyPanel:" ### v1.4
### v2.1 
### Xref.check now waits for model to load BUT ONLY if it's Windows ###
if [PLATFORM].grep(/mswin/) == [PLATFORM] ### = a Windows machine
 while not Sketchup.active_model.entities
    ###wait
 end#end while
 Xref.check
end#if
### v2.1
### ####################################################################
