#-----------------------------------------------------------------------------
# Copyright 2006, Didier Bur.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------
# Name        :   ado.rb
# Description :   Export pages to 2D various format files.
# Menu Item   :   Plugins->Automated 2D Output
# Context Menu:   None
# Usage       :   Select Pulgins/Automated 2D Output
# Date        :   31/01/2006 beta version
# Type        :   Tool
#-----------------------------------------------------------------------------
load 'convert.rb'
require 'sketchup.rb'
require 'Win32API'
#require 'win32ole'
#require 'extensions.rb'
#require 'LangHandler.rb'

#-----------------------------------------------------------------------------------------------------
#                                        GENERAL USAGE FUNCTIONS
#-----------------------------------------------------------------------------------------------------

def ado_select_export_folder(format)
model = Sketchup.active_model
current_dir = File.dirname(sutool_convert_string(model.path))
model_name = File.basename(model.path).split(".")[0]

# Create the export folder if it doesn't exist and the user didn't set it
if not $user_export_dir
  if not ( Dir.entries(current_dir).include?(model_name + "_" + format) )
    Dir.mkdir(current_dir + "\\" + model_name + "_" + format)
    $export_dir = current_dir + "\\" + model_name + "_" + format
    else
    $export_dir = current_dir + "\\" + model_name + "_" + format
  end
  else
  $export_dir = $user_export_dir
end
end

def ado_set_export_folder
model = Sketchup.active_model
current_dir = File.dirname(model.path)
model_name = File.basename(model.path)
folder = UI.openpanel("Select a file in the export folder:", "", "")
$user_export_dir = File.dirname(folder)
end

def ado_set_export_folder_default
if not $img_format
  format = "JPG"
  else
  format = $img_format.upcase
end
model = Sketchup.active_model
current_dir = File.dirname(sutool_convert_string(model.path))
model_name = File.basename(model.path).split(".")[0]
if $user_export_dir
  $user_export_dir = nil
  if not ( Dir.entries(current_dir).include?(model_name + "_" + format) )
    Dir.mkdir(current_dir + "\\" + model_name + "_" + format)
    $export_dir = current_dir + "\\" + model_name + "_" + format
    else
    $export_dir = current_dir + "\\" + model_name + "_" + format
  end
end
end

def exec_on_autoload
puts ("ADO loaded. By D. Bur, 01.06")
end

#------------------------------------------------ Switch from rad to deg
def rad2deg (a)
return ( a * 180.0 ) / Math::PI
end

#------------------------------------------------ Switch from deg to rad
def deg2rad (a)
return ( a * Math::PI ) / 180.0
end

#------------------------------------------------ Returns the first position of the camera and up vector, given 2 angles: azimuth and hauteur, distance is a constant of 1000
def ado_camera_position( azimuth, hauteur )
# default start point of camera
vec_start_position = Geom::Vector3d.new(0.0,-1000.0,0.0)
# rotation around Z
vec_z_world = Geom::Vector3d.new(0.0, 0.0, 1.0) 
tz = Geom::Transformation.rotation([0.0,0.0,0.0], vec_z_world, deg2rad( azimuth ))
vec_xy_position = vec_start_position.transform(tz)
# rotation around X
t_rot_plus_pis2 = Geom::Transformation.rotation([0.0,0.0,0.0], [0.0,0.0,1.0], deg2rad( 90.0 ))
vec_perp = vec_xy_position.transform(t_rot_plus_pis2)
tx = Geom::Transformation.rotation([0.0,0.0,0.0], vec_perp, deg2rad( 0.0 - hauteur ))
$vec_camera_position = vec_xy_position.transform(tx)

$vec_up = $vec_camera_position * vec_perp

end

#------------------------------------------------Turns off invisible layers in tmp_view during export
def set_page_layers( page, v)
model=Sketchup.active_model
model_layers = model.layers
model_layers.each do |l|
  l.visible = true
end
v.invalidate
layers = page.layers
if layers.length != 0
  layers.each do |l|
    l.visible = false
    v.invalidate
  end
end
end

#------------------------------------------------ Set rendering options of page in tmp_view during export
def set_page_rendering_options( page, v)
model=Sketchup.active_model
p_ropt = page.rendering_options
m_ropt = model.rendering_options
#puts p_ropt.to_a
if p_ropt
  p_ropt.each_pair { | key, value | m_ropt[key] = value }
end
v.invalidate
end

#-----------------------------------------------------------------------------------------------------
#                                        PAGES FUNCTIONS
#-----------------------------------------------------------------------------------------------------

#------------------------------------------------ Set the render mode for a page before updating it
# Wireframe: 0
# Hidden lines removal: 1
# Shading: 2
# Texturing: 2 + "Texture", true
def set_rendering_option( render, shadow, transp )
model=Sketchup.active_model

case render
  when "Wire"
  model.rendering_options["RenderMode"] = 0
  when "Hidden Lines"
  model.rendering_options["RenderMode"] = 1
  when "Color by Layer"
  model.rendering_options["RenderMode"] = 2
  model.rendering_options["DisplayColorByLayer"] = true
  model.rendering_options["Texture"] = false
  when "Color by Material"
  model.rendering_options["RenderMode"] = 2
  model.rendering_options["DisplayColorByLayer"] = false
  model.rendering_options["Texture"] = false
  when "Textured"
  model.rendering_options["RenderMode"] = 3
  model.rendering_options["DisplayColorByLayer"] = false
  model.rendering_options["Texture"] = true
end

case shadow
  when "Yes"
  model.shadow_info["DisplayShadows"] = true
  when "No"
  model.shadow_info["DisplayShadows"] = false
end

case transp
when "Yes"
  model.rendering_options["ModelTransparency"] = true
  when "No"
  model.rendering_options["ModelTransparency"] = false
end
end

#------------------------------------------------ Set all pages to ortho
def ado_set_all_pages_orthoview 
model=Sketchup.active_model
pages = model.pages
pages.each  do |p|
  if p.camera.perspective?
    p.camera.perspective = false
  end
end
end
#------------------------------------------------ Set all pages to perspective
def ado_set_all_pages_persview 
model=Sketchup.active_model
pages = model.pages
pages.each  do |p|
  if not p.camera.perspective?
    p.camera.perspective = true
  end
end
end
#------------------------------------------------ Set standard pages to ortho
def ado_set_standard_pages_orthoview 
model=Sketchup.active_model
pages = model.pages
standard_pages = ["Top", "Bottom", "Front", "Rear", "Left", "Right", "Front_Left", "Front_Right", "Rear_Left", "Rear_Right"]
pages.each  do |p|
  if( standard_pages.include? p.name)
    if p.camera.perspective?
      p.camera.perspective = false
    end
  end
end
end
#------------------------------------------------ Set standard pages to perspective
def ado_set_standard_pages_persview 
model=Sketchup.active_model
pages = model.pages
standard_pages = ["Top", "Bottom", "Front", "Rear", "Left", "Right", "Front_Left", "Front_Right", "Rear_Left", "Rear_Right"]
pages.each  do |p|
  if( standard_pages.include? p.name)
    if not p.camera.perspective?
      p.camera.perspective = true
    end
  end
end
end
#------------------------------------------------ Set user pages to ortho
def ado_set_user_pages_orthoview 
model=Sketchup.active_model
pages = model.pages
standard_pages = ["Top", "Bottom", "Front", "Rear", "Left", "Right", "Front_Left", "Front_Right", "Rear_Left", "Rear_Right"]
pages.each  do |p|
  if( not standard_pages.include? p.name)
    if p.camera.perspective?
      p.camera.perspective = false
    end
  end
end
end
#------------------------------------------------ Set user pages to perspective
def ado_set_user_pages_persview 
model=Sketchup.active_model
pages = model.pages
standard_pages = ["Top", "Bottom", "Front", "Rear", "Left", "Right", "Front_Left", "Front_Right", "Rear_Left", "Rear_Right"]
pages.each  do |p|
  if( not standard_pages.include? p.name)
    if not p.camera.perspective?
      p.camera.perspective = true
    end
  end
end
end

#------------------------------------------------ Refresh all pages with new settings
def ado_refresh_all_pages
model=Sketchup.active_model
pages = model.pages
# default values if not set
$render_pages = "Hidden Lines" if not $render_pages
$shadows_pages = "No" if not $shadows_pages
$trans_pages = "No" if not $trans_pages

# process each page
pages.each  do |p|
  tmp_view = model.active_view 
  tmp_view.camera = p.camera
  (set_rendering_option $render_pages, $shadows_pages, $trans_pages)
  (set_page_layers p, tmp_view)
  tmp_view.invalidate
  p.update
end
end
#------------------------------------------------ Refresh standard pages with new settings
def ado_refresh_standard_pages
model=Sketchup.active_model
pages = model.pages
standard_pages = ["Top", "Bottom", "Front", "Rear", "Left", "Right", "Front_Left", "Front_Right", "Rear_Left", "Rear_Right"]
# default values if not set
$render_pages = "Hidden Lines" if not $render_pages
$shadows_pages = "No" if not $shadows_pages
$trans_pages = "No" if not $trans_pages

# process each page
pages.each  do |p|
  if standard_pages.include? p.name
    tmp_view = model.active_view 
    tmp_view.camera = p.camera
    (set_rendering_option $render_pages, $shadows_pages, $trans_pages)
    (set_page_layers p, tmp_view)
    tmp_view.invalidate
    p.update
  end
end
end
#------------------------------------------------ Refresh user pages with new settings
def ado_refresh_user_pages
model=Sketchup.active_model
pages = model.pages
standard_pages = ["Top", "Bottom", "Front", "Rear", "Left", "Right", "Front_Left", "Front_Right", "Rear_Left", "Rear_Right"]
# default values if not set
$render_pages = "Hidden Lines" if not $render_pages
$shadows_pages = "No" if not $shadows_pages
$trans_pages = "No" if not $trans_pages

# process each page
pages.each  do |p|
  if not standard_pages.include? p.name
    tmp_view = model.active_view 
    tmp_view.camera = p.camera
    (set_rendering_option $render_pages, $shadows_pages, $trans_pages)
    (set_page_layers p, tmp_view)
    tmp_view.invalidate
    p.update
  end
end
end

#-----------------------------------------------------------------------------------------------------
#                                        PARAMETERS FUNCTIONS
#-----------------------------------------------------------------------------------------------------

#------------------------------------------------ Set pages parameters
def ado_set_pages_settings
#default values
$ortho_pages = "Yes" if not $ortho_pages
$axo_pages = "Yes" if not $axo_pages
$render_pages = "Hidden Lines" if not $render_pages
$shadows_pages = "No" if not $shadows_pages
$trans_pages = "No" if not $trans_pages
$angle_azimuth_pages = -45.0 if not $angle_azimuth_pages
$angle_hauteur_pages = 45.0 if not $angle_hauteur_pages

#Dialog box:
ortho_list = %w[Yes No].join("|")
axo_list = %w[Yes No].join("|")
render_list= ["Wire", "Hidden Lines", "Color by Layer", "Color by Material", "Textured"].join("|")
shadow_list = %w[Yes No].join("|")
transp_list = %w[Yes No].join("|")
array_of_dropdowns = [render_list, shadow_list, transp_list]
prompts=["Pages render mode  ", "Shadows  ", "X-Ray  ","Axonometric azimuth (0 -> -90)  ", "Axonometric height (0 -> +90)  "]
values=[$render_pages, $shadows_pages, $trans_pages,$angle_azimuth_pages.to_s, $angle_hauteur_pages.to_s]
results = inputbox prompts, values, array_of_dropdowns, "Pages settings"
return nil if not results
$render_pages = results[0]
$shadows_pages = results[1]
$trans_pages = results[2]
$angle_azimuth_pages = results[3].to_f
$angle_hauteur_pages = results[4].to_f
end

#------------------------------------------------ Set pages parameters
def ado_set_viewpoint_settings
#default values
$angle_azimuth_pages = -45.0 if not $angle_azimuth_pages
$angle_hauteur_pages = 45.0 if not $angle_hauteur_pages

#Dialog box:
prompts=["Axonometric azimuth (0 -> -90)  ", "Axonometric height (0 -> +90)  "]
values=[$angle_azimuth_pages.to_s, $angle_hauteur_pages.to_s]
results = inputbox prompts, values, "Axonometric viewpoint settings"
return nil if not results
$angle_azimuth_pages = results[0].to_f
$angle_hauteur_pages = results[1].to_f

if ($angle_azimuth_pages < -180.0) or ($angle_azimuth_pages > 180.0)
  $angle_azimuth_pages = 0.0
  UI.messagebox("Incorrect azimuth.\Azimuth reset to 0.")
end
if ($angle_hauteur_pages < -180.0) or ($angle_hauteur_pages > 180.0)
  $angle_hauteur_pages = 180.0
  UI.messagebox("Incorrect height.\Heigth rest to 0.")
end
end

#------------------------------------------------ About dialog box
def ado_about
UI.messagebox("Automated 2D Output\nD. Bur, Ruby Script - Version 1.0. January 31 2006\nThanks to TBD and TIG for their help." , MB_MULTILINE , "About ADO") 
end

#------------------------------------------------ Set images parameters for later export
def ado_set_img_settings
# sets the default values if not set by the user
$img_format = "jpg" if not $img_format
$img_alias = "Yes" if not $img_alias
$img_width = "640" if not $img_width
$img_height = "480" if not $img_height

#Dialog box:
format_list = %w[jpg bmp tga tif png].join("|")
alias_list = %w[Yes No].join("|")
array_of_dropdowns = [format_list, alias_list]
prompts=["Format:  ", "Antialias:  ", "Width (pixels):  ", "Height (pixels):  "]
values=[$img_format, $img_alias, $img_width, $img_height]
results = inputbox prompts, values, array_of_dropdowns, "Images settings"
return nil if not results
$img_format = results[0]
$img_alias = results[1]
$img_width = results[2]
$img_height = results[3]
end

#-----------------------------------------------------------------------------------------------------
#                                        IMAGES EXPORT
#-----------------------------------------------------------------------------------------------------

#------------------------------------------------ Export pages to images
def ado_export_img_from_pages( what )
model = Sketchup.active_model
model_name = File.basename(model.path).split(".")[0]
pages = model.pages
$img_exported = 0
standard_pages = ["Top", "Bottom", "Front", "Rear", "Left", "Right", "Front_Left", "Front_Right", "Rear_Left", "Rear_Right"]
pages_to_export = []

# Select export folder
case $img_format
  when "jpg"
    (ado_select_export_folder "JPG")
  when "png"
    (ado_select_export_folder "PNG")
  when "bmp"
    (ado_select_export_folder "BMP")
  when "tif"
    (ado_select_export_folder "TIF")
  when "tga"
    (ado_select_export_folder "TGA")
end

case what
  when "all"
    pages_to_export = pages
  when "user"
    pages.each do |p|
      if( not standard_pages.include? p.name)
        pages_to_export.push p
      end
    end
  when "standard"
    pages.each do |p|
      if( standard_pages.include? p.name)
        pages_to_export.push p
      end
    end
end

# Default values if not already set
$img_format = "jpg" if not $img_format
$img_alias = "Yes" if not $img_alias
$img_width = "640" if not $img_width
$img_height = "480" if not $img_height
$export_dir = $user_export_dir if $user_export_dir

if( $img_alias == "Yes")
  aa = true
  else
  aa = false
end

pages_to_export.each do |p|
  puts "Page: " + p.name
  page_camera = p.camera
  page_name = p.name
  tmp_view = model.active_view 
  tmp_view.camera = page_camera
  # Rendering options
  (set_page_rendering_options p, tmp_view)
  # Layers
  layers = p.layers
  if layers and layers.length != 0
    layers.each do |l|
      l.visible = false
      tmp_view.invalidate
    end
  end
  tmp_view.invalidate
  
  
  model.shadow_info["ShadowTime"]=p.shadow_info["ShadowTime"]
  
  # Export image
  #tmp_view.write_image $export_dir + File.dirname(Sketchup.active_model.path) + "\\" + model_name+ "_" + page_name + "." + $img_format, $img_width.to_i, $img_height.to_i, aa
  filebasename=model_name+ "_" + page_name + "." + $img_format
  filebasename.gsub!(/[\\|\/|\*|\:|\?|\<|\>|\"]/,"_")
  tmp_view.write_image $export_dir + "\\" + filebasename, $img_width.to_i, $img_height.to_i, aa
  $img_exported = $img_exported + 1
  Sketchup.set_status_text("Exporting image " + $img_exported.to_s + ": " + page_name)
end

# Back to current folder
Dir.chdir(File.dirname(sutool_convert_string(model.path)))

#UI.messagebox($img_exported.to_s + " images exported\n in folder:\n" + File.dirname(Sketchup.active_model.path) )
UI.messagebox($img_exported.to_s + " images exported\n in folder:\n" + $export_dir )
end

#------------------------------------------------ Create standard pages
def ado_create_standard_pages
model=Sketchup.active_model
#default values
$ortho_pages = "Yes" if not $ortho_pages
$axo_pages = "Yes" if not $axo_pages
$render_pages = "Hidden Lines" if not $render_pages
$shadows_pages = "No" if not $shadows_pages
$trans_pages = "No" if not $trans_pages

# Existing pages names
pages = model.pages
$pages_names = []
pages.each  do |p|
  $pages_names.push( p.name )
end

# Ortho pages creation if not created yet
if( $ortho_pages == "Yes" ) and not ( $pages_names.include?( "Top") )
  (ado_create_ortho_pages $render_pages, $shadows_pages, $trans_pages)
  else
  UI.messagebox("No need to re-create ortho views pages.")
end
# Axonometric pages creation if not created yet
  if( $axo_pages == "Yes" ) and not ( $pages_names.include?( "Front_Left") )
    (ado_create_axo_pages $render_pages, $shadows_pages, $trans_pages)
    else
    UI.messagebox("No need to re-create axonometric views pages.")
  end
end


#------------------------------------------------ Create axonometric pages with specified render type
def ado_create_ortho_pages( render_type, shadows, transparency)

#default values
$ortho_pages = "Yes" if not $ortho_pages
$axo_pages = "Yes" if not $axo_pages
$render_pages = "Hidden Lines" if not $render_pages
$shadows_pages = "No" if not $shadows_pages
$trans_pages = "No" if not $trans_pages

# Current params
model=Sketchup.active_model
pages=model.pages
num_pages=pages.count
pages_names = []
pages.each {|page| pages_names.push( page.name )}
view=model.active_view
camera=view.camera

pages_labels = ["Top", "Bottom", "Front", "Rear", "Left", "Right"]

if pages_names.include?("Top")
  button_press = UI.messagebox("Ortho pages still exist. Re-create them ?", MB_YESNO)
  if button_press == 6
    puts ("Updating pages with current display settings")
    pages.each do |p|
      if pages_labels.include?(p.name)
        (set_rendering_option $render_pages, $shadows_pages, $trans_pages)
        p.update
      end
    end
  else
    break
  end
end

# Top view
camera.set [0,0,1000], [0,0,0], [0,1,0], false
view.zoom_extents
new_page = pages.add "Top"
(set_rendering_option $render_pages, $shadows_pages, $trans_pages)
view.zoom_extents
view.invalidate
new_page.update
#  Bottom view
camera.set [0,0,-1000], [0,0,0], [0,1,0], false
view.zoom_extents
new_page = pages.add "Bottom"
(set_rendering_option $render_pages, $shadows_pages, $trans_pages)
view.zoom_extents
view.invalidate
new_page.update
# Front view
camera.set [0,-1000,0], [0,0,0], [0,0,1], false
view.zoom_extents
new_page = pages.add "Front"
(set_rendering_option $render_pages, $shadows_pages, $trans_pages)
view.zoom_extents
view.invalidate
new_page.update
# Rear view
camera.set [0,1000,0], [0,0,0], [0,0,1], false
view.zoom_extents
new_page = pages.add "Rear"
(set_rendering_option $render_pages, $shadows_pages, $trans_pages)
view.zoom_extents
view.invalidate
new_page.update
# Left view
camera.set [-1000,0,0], [0,0,0], [0,0,1], false
view.zoom_extents
new_page = pages.add "Left"
(set_rendering_option $render_pages, $shadows_pages, $trans_pages)
view.zoom_extents
view.invalidate
new_page.update
# Right view
camera.set [1000,0,0], [0,0,0], [0,0,1], false
view.zoom_extents
new_page = pages.add "Right"
(set_rendering_option $render_pages, $shadows_pages, $trans_pages)
view.zoom_extents
view.invalidate
new_page.update
end

#------------------------------------------------ Create axonometric pages with specified render type
def ado_create_axo_pages( render_type, shadows, transparency)
# Default values
#default values
$ortho_pages = "Yes" if not $ortho_pages
$axo_pages = "Yes" if not $axo_pages
$render_pages = "Hidden Lines" if not $render_pages
$shadows_pages = "No" if not $shadows_pages
$trans_pages = "No" if not $trans_pages
$angle_azimuth_pages = -45.0 if not $angle_azimuth_pages
$angle_hauteur_pages = 45.0 if not $angle_hauteur_pages
# Current params
model = Sketchup.active_model
pages = model.pages
num_pages = pages.count
view = model.active_view
camera = view.camera
t_rot_plus_pis2 = Geom::Transformation.rotation([0.0,0.0,0.0], [0.0,0.0,1.0], deg2rad( 90.0 ))

# Compute camera viewpoint position = $camera_position
(ado_camera_position $angle_azimuth_pages, $angle_hauteur_pages)


# Front-Left view
camera.set [$vec_camera_position.x,$vec_camera_position.y,$vec_camera_position.z] , [0,0,0], $vec_up, false
view.zoom_extents
new_page = pages.add "Front_Left"
(set_rendering_option $render_pages, $shadows_pages, $trans_pages)
view.zoom_extents
view.invalidate
new_page.update
# Front-Right view
$vec_camera_front_right = $vec_camera_position.transform(t_rot_plus_pis2)
$vec_up_front_right = $vec_up.transform(t_rot_plus_pis2)
#camera.set [1000,-1000,1000], [0,0,0], [-1,1,1], false
camera.set [$vec_camera_front_right.x,$vec_camera_front_right.y,$vec_camera_front_right.z], [0,0,0], $vec_up_front_right, false
view.zoom_extents
new_page = pages.add "Front_Right"
(set_rendering_option $render_pages, $shadows_pages, $trans_pages)
view.zoom_extents
view.invalidate
new_page.update
# Rear-Left view
$vec_camera_rear_left = $vec_camera_front_right.transform(t_rot_plus_pis2)
$vec_up_rear_left = $vec_up_front_right.transform(t_rot_plus_pis2)
#camera.set [1000,1000,1000], [0,0,0], [-1,-1,1], false
camera.set [$vec_camera_rear_left.x,$vec_camera_rear_left.y,$vec_camera_rear_left.z], [0,0,0], $vec_up_rear_left, false
view.zoom_extents
new_page = pages.add "Rear_Left"
(set_rendering_option $render_pages, $shadows_pages, $trans_pages)
view.zoom_extents
view.invalidate
new_page.update
# Rear-Right view
$vec_camera_rear_right = $vec_camera_rear_left.transform(t_rot_plus_pis2)
$vec_up_rear_right = $vec_up_rear_left.transform(t_rot_plus_pis2)
#camera.set [-1000,1000,1000], [0,0,0], [1,-1,1], false
camera.set [$vec_camera_rear_right.x,$vec_camera_rear_right.y,$vec_camera_rear_right.z], [0,0,0], $vec_up_rear_right, false
view.zoom_extents
new_page = pages.add "Rear_Right"
(set_rendering_option $render_pages, $shadows_pages, $trans_pages)
view.zoom_extents
view.invalidate
new_page.update

end

#-----------------------------------------------------------------------------------------------------
#                                        EXPORT FUNCTIONS
#-----------------------------------------------------------------------------------------------------

#-----------------------------------------------------------------------------------------------------
#                                        DWG EXPORT
#-----------------------------------------------------------------------------------------------------

#------------------------------------------------ Export pages to DWG files
def ado_export_dwg_from_pages( what )
model = Sketchup.active_model
pages = model.pages
$dwg_exported = 0
standard_pages = ["Top", "Bottom", "Front", "Rear", "Left", "Right", "Front_Left", "Front_Right", "Rear_Left", "Rear_Right"]
pages_to_export = []

(ado_select_export_folder "DWG")

case what
  when "all"
    pages_to_export = pages
  when "user"
    pages.each do |p|
      if( not standard_pages.include? p.name)
        pages_to_export.push p
      end
    end
  when "standard"
    pages.each do |p|
      if( standard_pages.include? p.name)
        pages_to_export.push p
      end
    end
end

pages_to_export.each do |p|
  page_camera = p.camera
  page_name = p.name
  tmp_view = model.active_view 
  tmp_view.camera = page_camera
  # Rendering options
  (set_page_rendering_options p, tmp_view)
  # Layers
  layers = p.layers
  if layers and layers.length != 0
    layers.each do |l|
      l.visible = false
      tmp_view.invalidate
    end
  end
  tmp_view.invalidate
  
  # Export dwg
    puts "Page: " + page_name
    (ado_export_dwg_page page_name)
    UI.messagebox("Page: " + page_name + " exists and has been overwriten.")
    $dwg_exported = $dwg_exported + 1
    Sketchup.set_status_text("Exporting DWG: " + $dwg_exported.to_s + ": " + page_name)
  #end
end

# Back to current folder
Dir.chdir(File.dirname(model.path))

# End message
UI.messagebox( $dwg_exported.to_s + " DWG files\nexported in folder:\n" + $export_dir )
end

#------------------------------------------------ Export a page to DWG file
def ado_export_dwg_page( p_name )
model = Sketchup.active_model
model_name = File.basename(model.path).split(".")[0]
tmp_view = model.active_view
page_camera = model.pages[p_name].camera
tmp_view.camera = page_camera
tmp_view.zoom_extents
tmp_view.invalidate
i = 2

if ( (i.divmod 2)  == 0 )
# open Export as Graphics
Sketchup.send_action 21237
send = Win32API.new("./Plugins/sutool/ro/AutoItX3.dll", "AU3_Send", ['P','L'], 'V')
# enter filename
send.call($export_dir + "\\" + model_name + "_" + p_name,0)
# select EPS
send.call("{TAB}",0)
#send.call("{a}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{UP}",0)
send.call("{TAB}",0)
# export
send.call("{Enter}",0)
# if file already exists overwrite
send.call("{LEFT}",0)
send.call("{Enter}",0)
# validate the "audit export OK" dialog box from SU
send.call("{Enter}",0)
i = i + 1
else
# open Export as Graphics
Sketchup.send_action 21237
send = Win32API.new("./Plugins/sutool/ro/AutoItX3.dll", "AU3_Send", ['P','L'], 'V')
# enter filename
send.call($export_dir + "\\" + model_name + "_" + p_name,0)
# select EPS
send.call("{TAB}",0)
#send.call("{a 2}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{UP}",0)
send.call("{TAB}",0)
# export
send.call("{Enter}",0)
# if file already exists overwrite
send.call("{LEFT}",0)
send.call("{Enter}",0)
# validate the "audit export OK" dialog box from SU
send.call("{Enter}",0)
i = i + 1
end

end


#-----------------------------------------------------------------------------------------------------
#                                        DXF EXPORT
#-----------------------------------------------------------------------------------------------------

#------------------------------------------------ Export pages to DXF files
def ado_export_dxf_from_pages( what )
model = Sketchup.active_model
pages = model.pages
$dxf_exported = 0
standard_pages = ["Top", "Bottom", "Front", "Rear", "Left", "Right", "Front_Left", "Front_Right", "Rear_Left", "Rear_Right"]
pages_to_export = []
(ado_select_export_folder "DXF")

case what
  when "all"
    pages_to_export = pages
  when "user"
    pages.each do |p|
      if( not standard_pages.include? p.name)
        pages_to_export.push p
      end
    end
  when "standard"
    pages.each do |p|
      if( standard_pages.include? p.name)
        pages_to_export.push p
      end
    end
end

pages_to_export.each do |p|
  page_camera = p.camera
  page_name = p.name
  tmp_view = model.active_view 
  tmp_view.camera = page_camera
  # Rendering options
  (set_page_rendering_options p, tmp_view)
  # Layers
  layers = p.layers
  if layers and layers.length != 0
    layers.each do |l|
      l.visible = false
      tmp_view.invalidate
    end
  end
  tmp_view.invalidate
  
  # Export dxf 
    puts "Page: " + page_name
    (ado_export_dxf_page page_name)
    UI.messagebox("Page: " + page_name + " exists and has been overwriten.")
    $dxf_exported = $dxf_exported + 1
    Sketchup.set_status_text("Exporting DXF: " + $dwg_exported.to_s + ": " + page_name)
  #end
end

# Back to current folder
Dir.chdir(File.dirname(model.path))

# End message
UI.messagebox( $dxf_exported.to_s + " DXF files\nexported in folder:\n" + $export_dir )
end

#------------------------------------------------ Export a page to DXF file
def ado_export_dxf_page( p_name )
model = Sketchup.active_model
model_name = File.basename(model.path).split(".")[0]
tmp_view = model.active_view
page_camera = model.pages[p_name].camera
tmp_view.camera = page_camera
tmp_view.zoom_extents
tmp_view.invalidate
# open Export as Graphics
Sketchup.send_action 21237
send = Win32API.new("./Plugins/sutool/ro/AutoItX3.dll", "AU3_Send", ['P','L'], 'V')
# enter filename
send.call($export_dir + "\\" + model_name + "_" +  p_name,0)
# select EPS
send.call("{TAB}",0)
#send.call("{a 2}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{TAB}",0)
# export
send.call("{Enter}",0)
# if file already exists overwrite
send.call("{LEFT}",0)
send.call("{Enter}",0)
# validate the "audit export OK" dialog box from SU
send.call("{Enter}",0)
end


#-----------------------------------------------------------------------------------------------------
#                                        EPS EXPORT
#-----------------------------------------------------------------------------------------------------

#------------------------------------------------ Export ikea pages to postscript files
def ado_export_eps_from_pages( what )
model = Sketchup.active_model
pages = model.pages
$eps_exported = 0
standard_pages = ["Top", "Bottom", "Front", "Rear", "Left", "Right", "Front_Left", "Front_Right", "Rear_Left", "Rear_Right"]
pages_to_export = []
(ado_select_export_folder "EPS")

case what
  when "all"
    pages_to_export = pages
  when "user"
    pages.each do |p|
      if( not standard_pages.include? p.name)
        pages_to_export.push p
      end
    end
  when "standard"
    pages.each do |p|
      if( standard_pages.include? p.name)
        pages_to_export.push p
      end
    end
end

pages_to_export.each do |p|
  page_camera = p.camera
  page_name = p.name
  tmp_view = model.active_view 
  tmp_view.camera = page_camera
  # Rendering options
  (set_page_rendering_options p, tmp_view)
  # Layers
  #(set_page_layers p, tmp_view) #unuseful
  layers = p.layers
  if layers and layers.length != 0
    layers.each do |l|
      l.visible = false
      tmp_view.invalidate
    end
  end
  #tmp_view.zoom_extents
  tmp_view.invalidate
  
  # Export eps
    puts "Page: " + page_name
    (ado_export_eps_page page_name)
    UI.messagebox("Page: " + page_name + " exists and has been overwriten.")
    $eps_exported = $eps_exported + 1
    Sketchup.set_status_text("Exporting EPS: " + $eps_exported.to_s + ": " + page_name)
  #end
end

# Back to current folder
Dir.chdir(File.dirname(model.path))

# End message
UI.messagebox( $eps_exported.to_s + " EPS files\nexported in folder:\n" + $export_dir )
end

#------------------------------------------------ Export a page to postscript file
def ado_export_eps_page( p_name )
model = Sketchup.active_model
model_name = File.basename(model.path).split(".")[0]
tmp_view = model.active_view
page_camera = model.pages[p_name].camera
tmp_view.camera = page_camera
tmp_view.zoom_extents
tmp_view.invalidate
# open Export as Graphics
Sketchup.send_action 21237
send = Win32API.new("./Plugins/sutool/ro/AutoItX3.dll", "AU3_Send", ['P','L'], 'V')
# enter filename
send.call($export_dir + "\\" + model_name + "_" + p_name,0)
# select EPS
send.call("{TAB}",0)
#send.call("{e 2}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{UP}",0)
send.call("{UP}",0)
send.call("{UP}",0)
send.call("{UP}",0)
send.call("{UP}",0)
send.call("{UP}",0)
send.call("{UP}",0)
send.call("{TAB}",0)
# export
send.call("{Enter}",0)
# if file already exists overwrite
send.call("{LEFT}",0)
send.call("{Enter}",0)

end

#-----------------------------------------------------------------------------------------------------
#                                        PDF EXPORT
#-----------------------------------------------------------------------------------------------------

#------------------------------------------------ Export pages to PDF files
def ado_export_pdf_from_pages( what )
model = Sketchup.active_model
pages = model.pages
$pdf_exported = 0
standard_pages = ["Top", "Bottom", "Front", "Rear", "Left", "Right", "Front_Left", "Front_Right", "Rear_Left", "Rear_Right"]
pages_to_export = []
(ado_select_export_folder "PDF")

case what
  when "all"
    pages_to_export = pages
  when "user"
    pages.each do |p|
      if( not standard_pages.include? p.name)
        pages_to_export.push p
      end
    end
  when "standard"
    pages.each do |p|
      if( standard_pages.include? p.name)
        pages_to_export.push p
      end
    end
end

pages_to_export.each do |p|
  page_camera = p.camera
  page_name = p.name
  tmp_view = model.active_view 
  tmp_view.camera = page_camera
  # Rendering options
  (set_page_rendering_options p, tmp_view)
  # Layers
  #(set_page_layers p, tmp_view)
  layers = p.layers
  if layers and layers.length != 0
    layers.each do |l|
      l.visible = false
      tmp_view.invalidate
    end
  end
  #tmp_view.zoom_extents
  tmp_view.invalidate
  
  # Export pdf
    puts "Page: " + page_name
    (ado_export_pdf_page page_name)
    UI.messagebox("Page: " + page_name + " exists and has been overwriten.")
    $pdf_exported = $pdf_exported + 1
    Sketchup.set_status_text("Exporting PDF: " + $pdf_exported.to_s + ": " + page_name)
  #end
end

# Back to current folder
Dir.chdir(File.dirname(model.path))

# End message
UI.messagebox( $pdf_exported.to_s + " PDF files\nexported in folder:\n" + $export_dir )
end

#------------------------------------------------ Export a page to DWG file
def ado_export_pdf_page( p_name )
model = Sketchup.active_model
model_name = File.basename(model.path).split(".")[0]
tmp_view = model.active_view
page_camera = model.pages[p_name].camera
tmp_view.camera = page_camera
tmp_view.zoom_extents
tmp_view.invalidate

# open Export as Graphics
Sketchup.send_action 21237
send = Win32API.new("./Plugins/sutool/ro/AutoItX3.dll", "AU3_Send", ['P','L'], 'V')
# enter filename
send.call($export_dir + "\\" + model_name + "_" + p_name,0)
# select EPS
send.call("{TAB}",0)
#send.call("{p 1}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{DOWN}",0)
send.call("{UP}",0)
send.call("{UP}",0)
send.call("{UP}",0)
send.call("{UP}",0)
send.call("{UP}",0)
send.call("{UP}",0)
send.call("{UP}",0)
send.call("{UP}",0)
send.call("{TAB}",0)
# export
send.call("{Enter}",0)
# if file already exists overwrite
send.call("{LEFT}",0)
send.call("{Enter}",0)

end

#-----------------------------------------------------------------------------------------------------
#                                        MENU ITEMS
#-----------------------------------------------------------------------------------------------------

if( not $ado_menu_loaded )
    #~ add_separator_to_menu("Plugins")
    #~ ado_menu = UI.menu("Plugins").add_submenu("Automated 2D Output")
    #~ ado_menu.add_item("Select export folder") { ado_set_export_folder }
    #~ ado_menu.add_item("Reset export folder to default") { ado_set_export_folder_default }
    #~ ado_menu.add_separator
    #~ ado_menu.add_item("Pages settings") { (ado_set_pages_settings) }
    #~ ado_menu.add_item("Images settings") { (ado_set_img_settings) }
    #~ edit_pages_menu = ado_menu.add_submenu("Edit Pages") { UI.messagebox("No command here !") }
    #~ edit_pages_menu.add_item("Set all pages orthogonal") { (ado_set_all_pages_orthoview) }
    #~ edit_pages_menu.add_item("Set all pages perspective") { (ado_set_all_pages_persview) }
    #~ edit_pages_menu.add_item("Set standard pages orthogonal") { (ado_set_standard_pages_orthoview) }
    #~ edit_pages_menu.add_item("Set standard pages perspective") { (ado_set_standard_pages_persview) }
    #~ edit_pages_menu.add_item("Set user pages orthogonal") { (ado_set_user_pages_orthoview) }
    #~ edit_pages_menu.add_item("Set user pages perspective") { (ado_set_user_pages_persview) }
    #~ edit_pages_menu.add_separator
    #~ edit_pages_menu.add_item("Refresh all pages") { (ado_refresh_all_pages) }
    #~ edit_pages_menu.add_item("Refresh standard pages") { (ado_refresh_standard_pages) }
    #~ edit_pages_menu.add_item("Refresh user pages") { (ado_refresh_user_pages) }
    #~ ado_menu.add_item("Create ortho pages") { (ado_create_ortho_pages $render_type, $shadows_pages, $trans_pages) }
    #~ ado_menu.add_item("Create axonometric pages") { (ado_create_axo_pages $render_type, $shadows_pages, $trans_pages) }
    #~ ado_menu.add_separator
    #~ export0_menu = ado_menu.add_submenu("Export all pages to") { UI.messagebox("No command here !") }
    #~ export0_menu.add_item("DWG") { (ado_export_dwg_from_pages "all") }
    #~ export0_menu.add_item("DXF") { (ado_export_dxf_from_pages "all") }
    #~ export0_menu.add_item("EPS") { (ado_export_eps_from_pages "all") }
    #~ export0_menu.add_item("PDF") { (ado_export_pdf_from_pages "all") }
    #~ export0_menu.add_item("Images") { (ado_export_img_from_pages "all") }
    #~ export1_menu = ado_menu.add_submenu("Export standard pages to") { UI.messagebox("No command here !") }
    #~ export1_menu.add_item("DWG") { (ado_export_dwg_from_pages "standard") }
    #~ export1_menu.add_item("DXF") { (ado_export_dxf_from_pages "standard") }
    #~ export1_menu.add_item("EPS") { (ado_export_eps_from_pages "standard") }
    #~ export1_menu.add_item("PDF") { (ado_export_pdf_from_pages "standard") }
    #~ export1_menu.add_item("Images") { (ado_export_img_from_pages "standard") }
    #~ export2_menu = ado_menu.add_submenu("Export user pages to") { UI.messagebox("No command here !") }
    #~ export2_menu.add_item("DWG") { (ado_export_dwg_from_pages "user") }
    #~ export2_menu.add_item("DXF") { (ado_export_dxf_from_pages "user") }
    #~ export2_menu.add_item("EPS") { (ado_export_eps_from_pages "user") }
    #~ export2_menu.add_item("PDF") { (ado_export_pdf_from_pages "user") }
    #~ export2_menu.add_item("Images") { (ado_export_img_from_pages "user") }
    #~ ado_menu.add_item("About ADO...") { (ado_about) }
    #~ $ado_menu_loaded = true
end

file_loaded("ado.rb")

(exec_on_autoload)