require 'sketchup.rb'

class ProgressBar 

@@err_total_notnumeric = "ProgressBar: Total must be a positive integer."
@@err_count_notnumeric = "ProgressBar: Iteration Count must be numeric"
@@end_time             = "End @ " 
@@progresschar = ">" ; 
@@initial_block = "-" * 50     # Default progress bar line sequence.

def initialize(total,phase=nil)  
  if (!total.integer? or total < 0)
    raise(ArgumentError,@@err_total_notnumeric) 
    return ; 
    end ; 
  @total = total.to_i ; 
  @phase = phase ; 
  @firsttime = true ; 
  end ; 

def update(iteration) 
  if !iteration.integer? 
    raise(ArgumentError,@@err_count_notnumeric)  
    return ; 
    end ; 
  iteration = [iteration.abs,@total].min  # make sure we don't exceed the total count or have a value < 0
  pct = [1,(iteration*100)/@total].max    # Calculate percentage complete.
                                          # round up to 1% if anything less than 1%.
  end_time = "?" ; 
  if @firsttime then ; 
    # Get the current time of day. 
    # set up an elapsed timer so we can calculate expected end time.  
    @time1 = Time.now ;     # Get current time of day. 
    @firsttime = false ;    # turn off switch 
  else 
    # divide the elapsed time by the pct complete, then multiple that by 100, then add that to the 
    # start time, and that is the expected end time.  
    end_time = Time.at(((((Time.now-@time1).to_f)/pct)*100 + @time1.to_f).to_i).strftime("%H:%M:%S") 
    end ; 
  pct_pos = [pct/2,1].max ;
  current_block = @@initial_block[0,pct_pos-1] << @@progresschar << @@initial_block[pct_pos,@@initial_block.length]
  Sketchup.set_status_text(current_block << "   " << (pct.to_s) << "%. #{@@end_time} " << end_time << " #{@phase}")
  #Sketchup.set_status_text(current_block << "   " << (pct.to_s) << "%")
  if (((Time.now-@time1).to_f/pct)*100) > $cbt_delay
    $cbt_stop_timer = true
  end
  #if (((Time.now-@time1).to_f > 300
    #$cbt_stop_timer = true
  #end
  end ; 

end ; # class ProgressBar ;

#----------------------------------------------------------------------------- Returns highest/lowest Z of model
def cbt_model_top
model = Sketchup.active_model
return model.bounds.max.z
end
def cbt_model_bottom
model = Sketchup.active_model
return model.bounds.min.z
end
#----------------------------------------------------------------------------- Translate an array of points to altitude h
def map_z(points_array, h)
mapped_array =[]
#points_array.each do |p|
  #mapped_array.push Geom::Point3d.new(p.x, p.y, h)
#end
points_array.each { |p| mapped_array.push Geom::Point3d.new(p.x, p.y, h) }
return mapped_array
end
#----------------------------------------------------------------------------- Returns average width and depth of component definition
def cbt_average_bounds(comps_def_array)
  area = 0.0
  average_scale = ($cbt_low_scale.abs + $cbt_high_scale.abs) / 2.0
  #comps_def_array.each do |cd|
    #area = area + cbt_area_bounds(cd) 
  #end
  comps_def_array.each { |cd| area = area + cbt_area_bounds(cd) }
  return (area * average_scale) / comps_def_array.length
end
#----------------------------------------------------------------------------- Returns average width and depth of component definition
def cbt_average_widths(comps_def_array)
  w = 0.0
  average_scale = ($cbt_low_scale.abs + $cbt_high_scale.abs) / 2.0
  #comps_def_array.each do |cd|
    #w = w + cd.bounds.width
  #end
  comps_def_array.each { |cd| w = w + cd.bounds.width }
  return (w * average_scale) / comps_def_array.length 
end
#----------------------------------------------------------------------------- Returns area of component definition
def cbt_area_bounds(c)
  return c.bounds.width * c.bounds.depth
  #return (c.bounds.width * c.bounds.depth * c.bounds.height) * 0.6666
end
#----------------------------------------------------------------------------- Returns BB of faces set
def cbt_faces_bounds(faces)
Sketchup.set_status_text("Computing faces bounds...")
fbb = Geom::BoundingBox.new
#faces.each do |f|
  #fbb.add f.bounds
#end
faces.each { |f| fbb.add f.bounds }
return fbb
end
#----------------------------------------------------------------------------- Returns area of faces set
def cbt_faces_area(faces)
Sketchup.set_status_text("Computing faces area...")
a = 0.0
#faces.each do |f|
  #a = a + f.area
#end
faces.each { |f| a = a + f.area }
return a
end
#----------------------------------------------------------------------------- Returns slope angle of face [0->PI/2]
def cbt_slope(hit_objet)
case hit_objet[1][0].typename
  when "Face"
    f = hit_objet[1][0]
  when "ComponentInstance"
    f = hit_objet[1][1]
  when "Group"
    f = hit_objet[1][1]
  when "Image"
    f = hit_objet[1][1]
  when "Edge"
    p = hit_objet[1][0].start.position
    pz = hit_objet[1][0].end.position
    p3 = Geom::Point3d.new(pz.x,pz.y,p.z)
    a = hit_objet[1][0].line[1].angle_between p.vector_to(p3)
    return a
  else
    return nil
end
if f.typename == "Face"
  n = f.normal
  p = Geom::Point3d.new(0,0,0)
  pz = p.offset(n)
  p0 = Geom::Point3d.new(pz.x,pz.y,0)
  a = p.vector_to(p0).angle_between n
  return a
else
  return 0.0
end
end
#-----------------------------------------------------------------------------  Test if low_steep < slope of face < high_steep
def cbt_match_slope(f)
return true if $cbt_low_steep == 0 and $cbt_high_steep == 90
l = Math::sin($cbt_low_steep.degrees)
h = Math::sin($cbt_high_steep.degrees)
fa = Math::cos(cbt_slope(f))
if l <= fa and h > fa
  return true
  else
  return false
end
end
#-----------------------------------------------------------------------------  Test if low_alt < p.z < high_alt
def cbt_match_alt(p)
if p.z >= $cbt_low_alt and p.z <= $cbt_high_alt
  return true
  else
  return false
end
end
#----------------------------------------------------------------------------- Returns array of comps definitions
def cbt_load_comps(comps_array)
  $cbt_comps=[]
  Sketchup.active_model.definitions.each do |d|
    if comps_array.include? d.name
      $cbt_comps.push d
    end
  end
  return $cbt_comps
end
#----------------------------------------------------------------------------- Returns number of comps to spray
def cbt_n_points(shape,center,radius,pressure,shape_points)
  press = pressure.split("%")[0].to_i 
  case shape
    when "Line"
      w = cbt_average_widths($cbt_comps)
    when "Circle"
      area = $cbt_radius * $cbt_radius * Math::PI
    when "Square"
      area = (($cbt_radius / Math.sqrt(2))*2) * (($cbt_radius / Math.sqrt(2))*2)
    when "Triangle"
      area = ($cbt_radius * $cbt_radius) * 3.0
    when "Rectangle"
      area = (shape_points[0].distance shape_points[1]) * (shape_points[1].distance shape_points[2])
    when "Selection"
      area = cbt_faces_area(cbt_sort_faces(Sketchup.active_model.selection))
  end
  case shape
    when "Line"
      n = (center.distance(radius) / w)
    else
      average_area = cbt_average_bounds($cbt_comps)
      n = area / average_area
  end

    puts shape + "   " + pressure + "   " + n.to_i.to_s + " -> " + ((n.to_i * press) / 100).floor.to_s
    cbt_warning((n.to_i * press) / 100)
if (n.to_i * press) / 100 < 1
  return 1
  else
  return (n.to_i * press) / 100
end
end
#----------------------------------------------------------------------------- Returns random number: low_range < number < high_range
def cbt_rand_range(low,high)
  v1 = rand()* (high.to_f.abs - low.to_f.abs)
  v2 = rand()* (high.to_f.abs - low.to_f.abs)
  #v3 = rand()* (high.to_f.abs - low.to_f.abs)
  if $cbt_proportion == "Yes"
    v2 = v1
  end
  if low.to_f < 0 or high.to_f < 0
    return [neg_rand(low.to_f.abs + v1),(low.to_f.abs + v2),(low.to_f.abs + v1)]
    else
    return [(low.to_f.abs + v1),(low.to_f.abs + v2),(low.to_f.abs + v1)]
  end
end
#----------------------------------------------------------------------------- Returns either x or -x, 1/2 probability
def neg_rand(x)
if rand() < 0.5
  return x
  else
  return -x
end
end
#----------------------------------------------------------------------------- Problem ?
def cbt_control_stop(trials_done, n_done, n_target)
if (n_done.to_f + 1)/trials_done.to_f < $cbt_ratio # default 0.001
  rep = UI.messagebox((trials_done-1).to_s + " trials to drop components done.\nMaybe there is no surface on which to drop,\nor there are too much constraints.\nContinue ?", MB_YESNO)
  if rep == 7
    break
    else
    $cbt_ratio = $cbt_ratio / 2.0
  end
end

if $cbt_stop_timer == true
  rep = UI.messagebox("More than " + ($cbt_delay / 60).to_s + " minutes remaining to complete.\nContinue ?", MB_YESNO)
  if rep == 7
    break
    else
    $cbt_delay = $cbt_delay * 2
    $cbt_stop_timer = false
  end
end
end
#----------------------------------------------------------------------------- Collect all components, returns array
def cbt_collect_components
model=Sketchup.active_model
comps = model.definitions
comps_names = []
comps.each do |c|
  if not c.group? and not c.image?
    comps_names.push(c.name)
  end
end
return comps_names.sort!
end
#----------------------------------------------------------------------------- Collect level 1 components, returns array
def cbt_collect_top_components
Sketchup.set_status_text("Collecting components...")
model=Sketchup.active_model
all_comps = model.definitions
all_comps_names = cbt_collect_components
top_comps = all_comps_names
all_comps.each do |cd|
  if not cd.image?
    # Search for sub-components in this comp def
    cd.entities.each do |e|
      if e.typename == "ComponentInstance"
        top_comps.delete e.definition.name
      end
    end
  end
end
Sketchup.set_status_text("")
return top_comps.sort!
end
#----------------------------------------------------------------------------- Collect layers names, returns array
def cbt_collect_layers
Sketchup.set_status_text("Collecting layers...")
model=Sketchup.active_model
layers_names = []
#model.layers.each do |l|
  #layers_names.push(l.name)
#end
model.layers.each { |l| layers_names.push(l.name) }
Sketchup.set_status_text("")
return layers_names.sort!
end

def cbt_sort_faces(s)
Sketchup.set_status_text("Sorting faces...")
f=[]
#s.each do |obj|
  #f.push(obj) if obj.typename == "Face"
#end
s.each { |obj| f.push(obj) if obj.typename == "Face" }
return f
end
#----------------------------------------------------------------------------- Eradicate non-faces objects through selection
def clean_sel
Sketchup.set_status_text("Sorting selection...")
s = Sketchup.active_model.selection
#s.each do |e|
  #s.remove(e) if not e.kind_of?(Sketchup::Face)
#end
s.each { |e| s.remove(e) if not e.kind_of?(Sketchup::Face) }
end
#----------------------------------------------------------------------------- Cast ray through point p and return [intersection,face] with faces set, if any
def cbt_raycast_faces(p,faces)
hit = false
hit_face = nil
v = Geom::Vector3d.new(0, 0, -100000)
faces.each do |face|
  raycast = Geom.intersect_line_plane([p,p.offset(v)], face.plane)
  if raycast and Geom.point_in_polygon_2D(p, face.vertices, true)
    hit = true
    hit_face = face
    return [Geom.intersect_line_plane([p,p.offset(v)], hit_face.plane), hit_face]
  end
end
if hit == true
  return [Geom.intersect_line_plane([p,p.offset(v)], hit_face.plane), hit_face]
else
  return false
end
end
#----------------------------------------------------------------------------- Modify comp transform, depending on options
def cbt_rotate_scale(h,c)
# Rotation
if $cbt_rotate == "Yes"
  o=c.transformation.origin
  if $cbt_perpendicular == "No"
    if h[1][0].class.to_s == "Sketchup::Face"
      r = Geom::Transformation.new([0,0,0], h[1][0].normal)
      c.transformation = c.transformation * r
    else
      r = Geom::Transformation.new([0,0,0],h[1][1].normal)
      c.transformation = c.transformation * r
    end
  else
    r=Geom::Transformation.rotation(o,[0,0,1],rand(Math::PI*2.0).to_f)
    c.transform! r
  end
end
if $cbt_low_scale != 1.0 or $cbt_high_scale != 1.0
  scales = cbt_rand_range($cbt_low_scale,$cbt_high_scale)
  c.transformation = c.transformation * Geom::Transformation.scaling(scales[0],scales[1], scales[0].abs)
end
end
#----------------------------------------------------------------------------- Returns an array of n random points based on shape
def cbt_random_points(n, shape, center, corner)
  model = Sketchup.active_model
  sel = Sketchup.active_model.selection
  v = Geom::Vector3d.new(0,0,-10000)
  rand_pts=[]

  # Set Z of random points to highest Z altitude of model + 1000 units (allow stacking higher than model top)
  z = cbt_model_top + 1000
  center = [center.x,center.y, z]
  corner = [corner.x,corner.y, z] if shape != "Rectangle" and shape != "Selection"
  p1_line = center if shape == "Line"
  p2_line = corner if shape == "Line"
  @pb = ProgressBar.new(n) 
  count = 0
  trials = 0
  
  # Layer management
  if $cbt_layer != ""
    old_layer = model.active_layer
    model.active_layer = $cbt_layer
  end
  
  case shape
    when "Selection"
      pts = map_z(corner, z)
      center = [center.x,center.y, z]
      faces = cbt_sort_faces(sel)
      while count < n
        p = polar(center, Geom::Point3d.new($cbt_radius,center.y,center.z), rand($cbt_radius), rand*(Math::PI*2))
        hit =  model.raytest [p,v]
        trials = trials + 1
        if hit != nil and Geom.point_in_polygon_2D(hit[0], pts, true) and cbt_raycast_faces(p,faces) and hit[1][0].class.to_s != $cbt_prevent and cbt_match_slope(hit) and cbt_match_alt(hit[0])
          comp = Sketchup.active_model.entities.add_instance $cbt_comps[rand($cbt_comps.length)], hit[0]
          # Rotate and scale
          cbt_rotate_scale(hit, comp)
           
          count = count + 1
        end
        @pb.update(count)
        cbt_control_stop(trials, count, n)
      end
    when "Line"
      vline = p1_line.vector_to(p2_line)
      while count < n
        vline.length = rand(p1_line.distance(p2_line)).to_f
        p = p1_line.offset(vline)
        hit =  model.raytest [p,v]
        trials = trials + 1
        if hit != nil and hit[1][0].class.to_s != $cbt_prevent and cbt_match_slope(hit) and cbt_match_alt(hit[0])
          comp = Sketchup.active_model.entities.add_instance $cbt_comps[rand($cbt_comps.length)], hit[0]
          # Rotate and scale
          cbt_rotate_scale(hit, comp)

          count = count + 1
        end
        @pb.update(count)
        cbt_control_stop(trials, count, n)
      end
    when "Rectangle"
      pts = map_z(corner, z)
      center = [center.x,center.y, z]
      while count < n
        p = polar(center, Geom::Point3d.new($cbt_radius,center.y,center.z), rand($cbt_radius), rand*(Math::PI*2))
        hit =  model.raytest [p,v]
        trials = trials + 1
        if hit != nil and Geom.point_in_polygon_2D(hit[0], pts, true) and hit[1][0].class.to_s != $cbt_prevent and cbt_match_slope(hit) and cbt_match_alt(hit[0])
          comp = Sketchup.active_model.entities.add_instance $cbt_comps[rand($cbt_comps.length)], hit[0]
          # Rotate and scale
          cbt_rotate_scale(hit, comp)

          count = count + 1
          @pb.update(count)
        end
        cbt_control_stop(trials, count, n)
      end
    when "Circle"
      ang = (Math::PI * 2) / n
      while count < n
        p = polar(center, Geom::Point3d.new($cbt_radius,center.y,center.z), rand($cbt_radius), rand*(Math::PI*2))
        hit =  model.raytest [p,v]
        trials = trials + 1
        if hit != nil and hit[1][0].class.to_s != $cbt_prevent and cbt_match_slope(hit) and cbt_match_alt(hit[0])
          comp = Sketchup.active_model.entities.add_instance $cbt_comps[rand($cbt_comps.length)], hit[0]
          # Rotate and scale
          cbt_rotate_scale(hit, comp)
          
          count = count + 1
          @pb.update(count)
        end
        cbt_control_stop(trials, count, n)
      end
    when "Square"
      # Calculate virtual square
      ang = Math::PI*2/4.0
      pts=[corner] 
      1.upto(3) do |i|
        p = corner.transform(Geom::Transformation.rotation(center,[0,0,1],i*(Math::PI/2.0)))
        pts.push p
      end
      while count < n
        # Create random point
        p = polar(center, Geom::Point3d.new($cbt_radius,center.y,center.z), rand($cbt_radius), rand*(Math::PI*2))
        # Add it if it is in virtual polygon
        if Geom.point_in_polygon_2D p, pts, true
          hit =  model.raytest [p,v]
          trials = trials + 1
          if hit != nil and hit[1][0].class.to_s != $cbt_prevent and cbt_match_slope(hit) and cbt_match_alt(hit[0])
            comp = Sketchup.active_model.entities.add_instance $cbt_comps[rand($cbt_comps.length)], hit[0]
            # Rotate and scale
            cbt_rotate_scale(hit, comp)
            
            count = count + 1
          end
          @pb.update(count)
        end
        cbt_control_stop(trials, count, n)
      end
    when "Triangle"
      # Calculate virtual triangle
      ang = Math::PI*2/3.0
      pts=[corner] 
      1.upto(2) do |i|
        p = corner.transform(Geom::Transformation.rotation(center,[0,0,1],i*(Math::PI*2.0/3.0)))
        pts.push p
      end
      count = 0
      while count < n
        # Create random point
        p = polar(center, Geom::Point3d.new($cbt_radius,center.y,center.z), rand($cbt_radius), rand*(Math::PI*2))
        # Add it if it is in virtual polygon
        if Geom.point_in_polygon_2D p, pts, true
          hit =  model.raytest [p,v]
          trials = trials + 1
          if hit != nil and hit[1][0].class.to_s != $cbt_prevent and cbt_match_slope(hit) and cbt_match_alt(hit[0])
            comp = Sketchup.active_model.entities.add_instance $cbt_comps[rand($cbt_comps.length)], hit[0]
            # Rotate and scale
            cbt_rotate_scale(hit, comp)

          count = count + 1
          @pb.update(count)
        end # if hit OK
        end
        
      end
      cbt_control_stop(trials, count, n)
  end
  
  # Layer management
  if $cbt_layer != ""
    model.active_layer = old_layer
  end
  
  return rand_pts
end
#----------------------------------------------------------------------------- Return point @ dist,angle
def polar(p1, p2, d, angle)
  v = p2 - p1
  v.length = d
  translate = Geom::Transformation.translation(v)
  p_temp = p1.transform(translate)
  rotate = Geom::Transformation.rotation( p1, Geom::Vector3d.new(0, 0, 1), angle)
  #result = p_temp.transform(rotate)
  #return result
  return p_temp.transform(rotate)
end
#----------------------------------------------------------------------------- Main dialog: prompt for brush style
def cbt_dialog_main
  model = Sketchup.active_model
  sel = model.selection
  comps = cbt_collect_top_components
  l1_comps = comps.join("|")
  l2_comps = comps.unshift("").join("|")
  layers = cbt_collect_layers
  Sketchup.set_status_text("Dialog box...")
  
  shapes = ["Point", "Line", "Triangle", "Square", "Rectangle", "Circle", "Spray"]
  # Case selection
  if not sel.empty? and cbt_sort_faces(sel) != []
    shapes.push "Selection"
    $cbt_shape = "Selection"
  end
  if $cbt_shape == "Selection" and sel.empty?
    $cbt_shape = "Circle"
  end
  # Trials ratio and stop_timer
  $cbt_ratio = 0.001
  $cbt_stop_timer = false
  $cbt_delay = 180
  
  $cbt_use_settings = "Dialog" if not $cbt_use_settings
  pressures = ["10%","20%","30%","40%","50%","60%","70%","80%","90%","100%"]
  $cbt_name_1 = comps[0] if not $cbt_name_1 and comps.length > 0
  $cbt_name_2 = "" if not $cbt_name_2
  $cbt_name_3 = "" if not $cbt_name_3
  $cbt_name_4 = "" if not $cbt_name_4
  $cbt_name_5 = "" if not $cbt_name_5
  $cbt_name_6 = "" if not $cbt_name_6
  $cbt_shape = "Circle" if not $cbt_shape
  $cbt_radius = 50.m if not $cbt_radius
  $cbt_spray_radius = 10.m if not $cbt_spray_radius
  $cbt_pressure = "50%" if not $cbt_pressure
  $cbt_rotate = "Yes" if not $cbt_rotate
  $cbt_stacking = "Yes" if not $cbt_stacking
  $cbt_perpendicular = "Yes" if not $cbt_perpendicular
  $cbt_layer = model.active_layer if not $cbt_layer
  $cbt_low_scale = 1.0 if not $cbt_low_scale
  $cbt_high_scale = 1.0 if not $cbt_high_scale
  $cbt_proportion = "Yes" if not $cbt_proportion
  $cbt_low_steep = 0 if not $cbt_low_steep
  $cbt_high_steep = 90 if not $cbt_high_steep
  $cbt_low_alt = cbt_model_bottom
  $cbt_high_alt = cbt_model_top
  
  prompts = ["Use settings: ", "Component 1: ", "Component 2: ", "Component 3: ", "Component 4: ", "Component 5: ", "Component 6: ", "Spray shape: ", "Spray pressure: ", "Random rotate: ", "Prevent stacking: ", "Keep vertical: ", "Layer: ", "Maintain proportions: ", "Low scale value: ", "High scale value: ", "Low steepness: ", "High steepness: " , "Low altitude: ", "High altitude: "]
  enums = [["Basic","Defaults","Dialog"].join("|"),l1_comps,l2_comps,l2_comps,l2_comps,l2_comps,l2_comps,shapes.join("|"), pressures.join("|"), ["Yes", "No"].join("|"), ["Yes", "No"].join("|"), ["Yes", "No"].join("|"),layers.unshift("").join("|"), ["Yes", "No"].join("|")]
  values = [$cbt_use_settings, $cbt_name_1, $cbt_name_2, $cbt_name_3, $cbt_name_4, $cbt_name_5, $cbt_name_6, $cbt_shape, $cbt_pressure, $cbt_rotate, $cbt_stacking, $cbt_perpendicular, $cbt_layer, $cbt_proportion, $cbt_low_scale, $cbt_high_scale, $cbt_low_steep, $cbt_high_steep, $cbt_low_alt, $cbt_high_alt]
  results = inputbox prompts, values, enums, comps.length.to_s + " components available:"
  return if not results
  $cbt_use_settings, $cbt_name_1, $cbt_name_2, $cbt_name_3, $cbt_name_4, $cbt_name_5, $cbt_name_6, $cbt_shape, $cbt_pressure, $cbt_rotate, $cbt_stacking, $cbt_perpendicular, $cbt_layer, $cbt_proportion, $cbt_low_scale, $cbt_high_scale, $cbt_low_steep, $cbt_high_steep, $cbt_low_alt, $cbt_high_alt = results
  
  # Load components definitions
  $cbt_comps = cbt_load_comps([$cbt_name_1, $cbt_name_2, $cbt_name_3, $cbt_name_4, $cbt_name_5, $cbt_name_6])
  
  # Stacking
  if $cbt_stacking == "Yes"
    $cbt_prevent = "Sketchup::ComponentInstance"
    else
    $cbt_prevent = ""
  end
  
  # Altitude
  if $cbt_low_alt > $cbt_high_alt
    $cbt_high_alt = $cbt_low_alt
    $cbt_low_alt = 0
  end
  
  # Steepness
  if $cbt_low_steep > $cbt_high_steep
    $cbt_high_steep = $cbt_low_steep
    $cbt_low_steep = 0
  end
  
  # Store current dialog settings
  cbt_store_dialog_settings
  
  # Use correct settings
  case $cbt_use_settings
    when "Basic"
      cbt_use_basic
    when "Defaults"
      cbt_use_default
  end

end

#----------------------------------------------------------------------------- Set default settings dialog
def cbt_dialog_set_defaults
  model = Sketchup.active_model
  ud_shapes = ["Point", "Line", "Triangle", "Square", "Rectangle", "Circle", "Selection"]
  ud_pressures = ["10%","20%","30%","40%","50%","60%","70%","80%","90%","100%"]
  # Trials ratio
  $cbt_ratio = 0.001
  
  $cbt_de_shape = "Circle" if not $cbt_de_shape
  $cbt_de_radius = 50.m if not $cbt_de_radius
  $cbt_de_spray_radius = 10.m if not $cbt_de_spray_radius
  $cbt_de_pressure = "50%" if not $cbt_de_pressure
  $cbt_de_rotate = "Yes" if not $cbt_de_rotate
  $cbt_de_stacking = "Yes" if not $cbt_de_stacking
  $cbt_de_perpendicular = "Yes" if not $cbt_de_perpendicular
  $cbt_de_proportion = "Yes" if not $cbt_de_proportion
  $cbt_de_low_scale = 1.0 if not $cbt_de_low_scale
  $cbt_de_high_scale = 1.0 if not $cbt_de_high_scale
  $cbt_de_low_steep = 0 if not $cbt_de_low_steep
  $cbt_de_high_steep = 90 if not $cbt_de_high_steep
  $cbt_de_low_alt = cbt_model_bottom
  $cbt_de_high_alt = cbt_model_top
  
  prompts = ["Spray shape: ", "Spray pressure: ", "Random rotate: ", "Prevent stacking: ", "Keep vertical: ", "Maintain proportions: ", "Low scale value: ", "High scale value: ", "Low steepness: ", "High steepness:", "Low altitude: ", "High altitude: "]
  enums = [ud_shapes.join("|"), ud_pressures.join("|"), ["Yes", "No"].join("|"), ["Yes", "No"].join("|"), ["Yes", "No"].join("|"), ["Yes", "No"].join("|")]
  values = [$cbt_de_shape, $cbt_de_pressure, $cbt_de_rotate, $cbt_de_stacking, $cbt_de_perpendicular, $cbt_de_proportion, $cbt_de_low_scale, $cbt_de_high_scale, $cbt_de_low_steep, $cbt_de_high_steep, $cbt_de_low_alt, $cbt_de_high_alt]
  results = inputbox prompts, values, enums, "Defaults settings"
  return if not results
  $cbt_de_shape, $cbt_de_pressure, $cbt_de_rotate, $cbt_de_stacking, $cbt_de_perpendicular, $cbt_de_proportion, $cbt_de_low_scale, $cbt_de_high_scale, $cbt_de_low_steep, $cbt_de_high_steep, $cbt_de_low_alt, $cbt_de_high_alt = results
  # Stacking
  if $cbt_de_stacking == "Yes"
    $cbt_de_prevent = "Sketchup::ComponentInstance"
    else
    $cbt_de_prevent = ""
  end
end
#----------------------------------------------------------------------------- Set basic and default settings
def cbt_set_basic
  # Trials ratio
  $cbt_ratio = 0.001
  # Spray radius default
  $cbt_spray_radius = 10.m
  # Basic
  $cbt_ba_shape = "Circle"
  $cbt_ba_radius = 50.m
  $cbt_ba_spray_radius = 10.m
  $cbt_ba_pressure = "50%"
  $cbt_ba_rotate = "Yes"
  $cbt_ba_stacking = "Yes"
  $cbt_ba_perpendicular = "Yes"
  $cbt_ba_proportion = "Yes"
  $cbt_ba_low_scale = 1.0
  $cbt_ba_high_scale = 1.0
  $cbt_ba_low_steep = 0
  $cbt_ba_high_steep = 90
  $cbt_ba_low_alt = cbt_model_bottom
  $cbt_ba_high_alt = cbt_model_top
  $cbt_ba_prevent = "Sketchup::ComponentInstance"
  # Default
  $cbt_de_shape = "Circle" if not $cbt_de_shape
  $cbt_de_radius = 50.m if not $cbt_de_radius
  $cbt_de_spray_radius = 10.m if not $cbt_de_spray_radius
  $cbt_de_pressure = "50%" if not $cbt_de_pressure
  $cbt_de_rotate = "Yes" if not $cbt_de_rotate
  $cbt_de_stacking = "Yes" if not $cbt_de_stacking
  $cbt_de_perpendicular = "Yes" if not $cbt_de_perpendicular
  $cbt_de_proportion = "Yes" if not $cbt_de_proportion
  $cbt_de_low_scale = 1.0 if not $cbt_de_low_scale
  $cbt_de_high_scale = 1.0 if not $cbt_de_high_scale
  $cbt_de_low_steep = 0 if not $cbt_de_low_steep
  $cbt_de_high_steep = 90 if not $cbt_de_high_steep
  $cbt_de_low_alt = cbt_model_bottom
  $cbt_de_high_alt = cbt_model_top
  $cbt_de_prevent = "Sketchup::ComponentInstance"
end
#----------------------------------------------------------------------------- Overwrite current settings with basic settings
def cbt_use_basic
  #puts "settings to basic"
  $cbt_shape = $cbt_ba_shape
  $cbt_radius = $cbt_ba_radius
  $cbt_spray_radius = $cbt_ba_spray_radius
  $cbt_pressure = $cbt_ba_pressure
  $cbt_rotate = $cbt_ba_rotate
  $cbt_stacking = $cbt_ba_stacking
  $cbt_perpendicular = $cbt_ba_perpendicular
  $cbt_proportion = $cbt_ba_proportion
  $cbt_low_scale = $cbt_ba_low_scale
  $cbt_high_scale = $cbt_ba_high_scale
  $cbt_low_steep = $cbt_ba_low_steep
  $cbt_high_steep = $cbt_ba_high_steep
  $cbt_low_alt = cbt_model_bottom
  $cbt_high_alt = cbt_model_top
  $cbt_prevent = "Sketchup::ComponentInstance"
end
#----------------------------------------------------------------------------- Overwrite current settings with basic settings
def cbt_use_default
  #puts "settings to default"
  $cbt_shape = $cbt_de_shape
  $cbt_radius = $cbt_de_radius
  $cbt_spray_radius = $cbt_de_spray_radius
  $cbt_pressure = $cbt_de_pressure
  $cbt_rotate = $cbt_de_rotate
  $cbt_stacking = $cbt_de_stacking
  $cbt_perpendicular = $cbt_de_perpendicular
  $cbt_proportion = $cbt_de_proportion
  $cbt_low_scale = $cbt_de_low_scale
  $cbt_high_scale = $cbt_de_high_scale
  $cbt_low_steep = $cbt_de_low_steep
  $cbt_high_steep = $cbt_de_high_steep
  $cbt_low_alt = cbt_model_bottom
  $cbt_high_alt = cbt_model_top
  $cbt_prevent = "Sketchup::ComponentInstance"
end
#----------------------------------------------------------------------------- Temp save of dialog settings, to be restored when command ends
def cbt_store_dialog_settings
  $cbt_t_shape = $cbt_shape
  $cbt_t_radius = $cbt_radius
  $cbt_t_pressure = $cbt_pressure
  $cbt_t_rotate = $cbt_rotate
  $cbt_t_stacking = $cbt_stacking
  $cbt_t_perpendicular = $cbt_perpendicular
  $cbt_t_proportion = $cbt_proportion
  $cbt_t_low_scale = $cbt_low_scale
  $cbt_t_high_scale = $cbt_high_scale
  $cbt_t_low_steep = $cbt_low_steep
  $cbt_t_high_steep = $cbt_high_steep
  $cbt_t_low_alt = $cbt_low_alt
  $cbt_t_high_alt = $cbt_high_alt
  $cbt_t_prevent = $cbt_prevent
end

#----------------------------------------------------------------------------- Restore dialog settings
def cbt_restore_dialog_settings
  $cbt_shape = $cbt_t_shape
  $cbt_radius = $cbt_t_radius
  $cbt_pressure = $cbt_t_pressure
  $cbt_rotate = $cbt_t_rotate
  $cbt_stacking = $cbt_t_stacking
  $cbt_perpendicular = $cbt_t_perpendicular
  $cbt_proportion = $cbt_t_proportion
  $cbt_low_scale = $cbt_t_low_scale
  $cbt_high_scale = $cbt_t_high_scale
  $cbt_low_steep = $cbt_t_low_steep
  $cbt_high_steep = $cbt_t_high_steep
  $cbt_low_alt = $cbt_t_low_alt
  $cbt_high_alt = $cbt_t_high_alt
  $cbt_prevent = $cbt_t_prevent
end
#----------------------------------------------------------------------------- Warning: lot of components !
def cbt_warning(n)
return if n < 500
n2 = n/100 * 100
rep = UI.messagebox("About to create more than " + n2.to_s + " components.\n Proceed ?",MB_YESNO)
break if rep == 7
end
#----------------------------------------------------------------------------- Tool class for Ngons
class ComponentNgonBrushTool

def initialize
  model = Sketchup.active_model
  entities = model.entities

  @ip1 = nil
  @ip2 = nil
  @ip3 = nil
  @xdown = 0
  @ydown = 0
  @state = 0
  @shape = $cbt_shape
  #puts "Init NgonTool with: " + $cbt_shape
end

def activate
    @ip1 = Sketchup::InputPoint.new
    @ip2 = Sketchup::InputPoint.new
    @ip3 = Sketchup::InputPoint.new
    @ip = Sketchup::InputPoint.new
    @drawn = false
    # This sets the label for the VCB
    Sketchup::set_status_text "Radius or Distance", SB_VCB_LABEL
    self.reset(nil)
    #puts "Activate NgonTool with: " + $cbt_shape
end

def onMouseLeave(view)
  Sketchup.send_action "selectSelectionTool:"
end

def deactivate(view)
    view.invalidate if @drawn
end

def onMouseMove(flags, x, y, view)
    if( @state == 0 )
      #puts"state 0: " + $cbt_shape
        @ip.pick view, x, y
        if( @ip != @ip1 )
            view.invalidate if( @ip.display? or @ip1.display? )
            @ip1.copy! @ip
            # set the tooltip that should be displayed to this point
            view.tooltip = @ip1.tooltip
        end
    else
        # Getting the radius point or the second point for rectangle
        @ip2.pick view, x, y, @ip1
        view.tooltip = @ip2.tooltip if( @ip2.valid? )
        view.invalidate
        # Update the length displayed in the VCB
        if( @ip2.valid? )
            @length = @ip1.position.distance(@ip2.position)
            Sketchup::set_status_text @length.to_s, SB_VCB_VALUE
        end
        # Check to see if the mouse was moved far enough to create a circle.
        # This is used so that you can create a circle by either draggin or doing click-move-click
        if( (x-@xdown).abs > 10 || (y-@ydown).abs > 10 )
            @dragging = true
            self.draw_geometry(@ip1.position, @ip2.position, @length, view)
        end
    end
end

def onLButtonDown(flags, x, y, view)
    # When the user clicks the first time, we switch to getting the
    # second point.  When he clicks a second time we create the circle
    if( @state == 0 )
        @ip1.pick view, x, y
        if( @ip1.valid? )
            @state = 1
            Sketchup::set_status_text "Radius", SB_PROMPT
            @xdown = x
            @ydown = y
        end
    end
    if( @state == 1 )
        # create the ngon on the second click
        #if( @ip2.valid? and ($cbt_shape == "Circle" or $cbt_shape == "Square" or $cbt_shape == "Triangle"))
        if( @ip2.valid? and (@shape == "Circle" or @shape == "Square" or @shape == "Triangle"))
          @state = 2
          # spray components
          self.create_geometry(@ip1.position, @ip2.position,view)
          self.reset(view)
        end
    end
    # Clear any inference lock
    view.lock_inference
end

def onLButtonUp(flags, x, y, view)
    # If we are doing a drag, then create the circle on the mouse up event
    if( @dragging && @ip2.valid? )
        # spray components)
        self.create_geometry(@ip1.position, @ip2.position,view)
        self.reset(view)
    end
end

def onKeyDown(key, repeat, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY && repeat == 1 )
        @shift_down_time = Time.now
        
        # if we already have an inference lock, then unlock it
        if( view.inference_locked? )
            # calling lock_inference with no arguments actually unlocks
            view.lock_inference
        elsif( @state == 0 && @ip1.valid? )
            view.lock_inference @ip1
        elsif( @state == 1 && @ip2.valid? )
            view.lock_inference @ip2, @ip1
        end
    end
end

def onKeyUp(key, repeat, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY &&
        view.inference_locked? &&
        (Time.now - @shift_down_time) > 0.5 )
        view.lock_inference
    end
end

def onUserText(text, view)
    return if not @state == 1
    return if not @ip2.valid?
    
    begin
        value = text.to_l
    rescue
        # Error parsing the text
        UI.beep
        puts "Cannot convert #{text} to a Length"
        value = nil
        Sketchup::set_status_text "", SB_VCB_VALUE
    end
    return if !value

    # Compute the direction and the second point
    pt1 = @ip1.position
    vec = @ip2.position - pt1
    if( vec.length == 0.0 )
        return
    end
    vec.length = value
    pt2 = pt1 + vec

    # spray components
    self.create_geometry(pt1, pt2, view)
    self.reset(view)
end

# The draw method is called whenever the view is refreshed.
def draw(view)
    if( @ip1.valid? )
        if( @ip1.display? )
            @ip1.draw(view)
            @drawn = true
        end
        if( @ip2.valid? )
            @ip2.draw(view) if( @ip2.display? )
            view.set_color_from_line(@ip1, @ip2)
            self.draw_geometry(@ip1.position, @ip2.position, @length, view)
            @drawn = true
        end
    end
end

# onCancel is called when the user hits the escape key
def onCancel(flag, view)
    self.reset(view)
    Sketchup.send_action "selectSelectionTool:"
end

# Reset the tool back to its initial state
def reset(view)
    @state = 0
    # Display a prompt on the status bar
    Sketchup::set_status_text("Center of circle", SB_PROMPT)
    # clear the InputPoints
    @ip1.clear
    @ip2.clear
    if( view )
        view.tooltip = nil
        view.invalidate if @drawn
    end
    @drawn = false
    @dragging = false
end

# Create new geometry when the user has selected two points.
def create_geometry(p1, p2, view)
  model = Sketchup.active_model
  model.start_operation "Component Paintbrush"
  $cbt_radius = p1.distance p2
  #n_comps = cbt_n_points($cbt_shape, @ip1.position, $cbt_radius, $cbt_pressure,nil)
  #cbt_random_points(n_comps, $cbt_shape, @ip1.position, @ip2.position)
  n_comps = cbt_n_points(@shape, @ip1.position, $cbt_radius, $cbt_pressure,nil)
  cbt_random_points(n_comps, @shape, @ip1.position, @ip2.position)
  model.commit_operation
    
end #of def create_geometry


# Draw the geometry
def draw_geometry(pt1, pt2, radius, view)
  case @shape
    when "Circle"
      sides = 30
    when "Square"
      sides = 4
    when "Triangle"
      sides = 3
  end
  ang = Math::PI*2/sides
  pts=[pt2]
  0.upto(sides-1) do |i|
    trans = Geom::Transformation.rotation(pt1,[0,0,1],i*ang)
    pts.push pt2.transform(trans)
  end
  # Ngon temp draw
  view.draw_points(pt1,8,5,"red")
  pts.push pts.first
  view.draw_polyline pts
end

end # class ComponentNgonBrushTool

#----------------------------------------------------------------------------------------------------------
#----------------------------------------------------------------------------------------------------------

class ComponentPointBrushTool

def initialize
    @ip1 = nil
    @xdown = 0
    @ydown = 0
    @shape = $cbt_shape
    # Iterate through comp list
    @indice = 0
end

def activate
    @ip1 = Sketchup::InputPoint.new
    @ip = Sketchup::InputPoint.new
    @drawn = false
    Sketchup::set_status_text("Point", SB_VCB_LABEL)
    self.reset(nil)
end

def onMouseLeave(view)
  Sketchup.send_action "selectSelectionTool:"
end

def deactivate(view)
    view.invalidate if @drawn
end

def onMouseMove(flags, x, y, view)
  @ip.pick view, x, y
  if( @ip != @ip1 )
    view.invalidate if( @ip.display? or @ip1.display? )
    @ip1.copy! @ip
    view.tooltip = @ip1.tooltip
  end
end

def onLButtonDown(flags, x, y, view)
        @ip1.pick view, x, y
        if( @ip1.valid? )
            Sketchup::set_status_text("Point", SB_PROMPT)
            @xdown = x
            @ydown = y
        end
    # Clear any inference lock
    view.lock_inference
    self.create_geometry(@ip1.position,view)
end

def onLButtonUp(flags, x, y, view)
  if( @dragging && @ip2.valid? )
    self.reset(view)
  end
end

def onKeyDown(key, repeat, flags, view)
  if( key == CONSTRAIN_MODIFIER_KEY && repeat == 1 )
    @shift_down_time = Time.now
    # if we already have an inference lock, then unlock it
    if( view.inference_locked? )
      # calling lock_inference with no arguments actually unlocks
      view.lock_inference
    end
  end
end

def onKeyUp(key, repeat, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY &&
        view.inference_locked? &&
        (Time.now - @shift_down_time) > 0.5 )
        view.lock_inference
    end
end

def onUserText(text, view)
    self.reset(view)
end

def draw(view)
  if( @ip1.valid? )
    if( @ip1.display? )
      @ip1.draw(view)
      @drawn = true
    end
  end
end

def onCancel(flag, view)
    self.reset(view)
    Sketchup.send_action "selectSelectionTool:"
end

def reset(view)
  @state = 0
  Sketchup::set_status_text($exStrings.GetString("Point"), SB_PROMPT)
  @ip1.clear
  if( view )
    view.tooltip = nil
    view.invalidate if @drawn
  end
  @drawn = false
  @dragging = false
end

def create_geometry(p1, view)
  model = Sketchup.active_model
  model.start_operation "Component Paintbrush"
  z = cbt_model_top + 1000
  p = [@ip1.position.x, @ip1.position.y, z]
  v = Geom::Vector3d.new(0,0,-10000)
  hit =  model.raytest [p,v]
  
  if hit != nil and hit[1][0].class.to_s != $cbt_prevent and cbt_match_slope(hit) and cbt_match_alt(hit[0])
    
    # Random type
    #comp = Sketchup.active_model.entities.add_instance $cbt_comps[rand($cbt_comps.length)], hit[0]
    
    # Iterate through selected components
    if @indice == $cbt_comps.length
      @indice = 0
    end
    comp = Sketchup.active_model.entities.add_instance $cbt_comps[@indice], hit[0]
    @indice = @indice + 1
    
    # Rotation and scale
    cbt_rotate_scale(hit,comp)
    else
    Sketchup.set_status_text("Cannot place a component here !")
  end
  if not cbt_match_slope(hit)
      Sketchup.set_status_text("Slope out of range. Cannot place a component here !")
  end
  if not cbt_match_alt(hit[0])
      Sketchup.set_status_text("Altitude out of range. Cannot place a component here !")
  end
  
  model.commit_operation
end

def draw_geometry(pt1, view)
    view.draw_point(pt1)
end

end # class ComponentPointBrushTool

#----------------------------------------------------------------------------------------------------------
#----------------------------------------------------------------------------------------------------------

class ComponentLineBrushTool

def initialize
    @ip1 = nil
    @ip2 = nil
    @xdown = 0
    @ydown = 0
    @shape = $cbt_shape
end

def activate
    @ip1 = Sketchup::InputPoint.new
    @ip2 = Sketchup::InputPoint.new
    @ip = Sketchup::InputPoint.new
    @drawn = false
    Sketchup::set_status_text $exStrings.GetString("Length"), SB_VCB_LABEL
    self.reset(nil)
end

def onMouseLeave(view)
  Sketchup.send_action "selectSelectionTool:"
end

def deactivate(view)
    view.invalidate if @drawn
end

def onMouseMove(flags, x, y, view)
    if( @state == 0 )
        @ip.pick view, x, y
        if( @ip != @ip1 )
            view.invalidate if( @ip.display? or @ip1.display? )
            @ip1.copy! @ip
            view.tooltip = @ip1.tooltip
        end
    else
        @ip2.pick view, x, y, @ip1
        view.tooltip = @ip2.tooltip if( @ip2.valid? )
        view.invalidate

        if( @ip2.valid? )
            length = @ip1.position.distance(@ip2.position)
            Sketchup::set_status_text length.to_s, SB_VCB_VALUE
        end
        
        if( (x-@xdown).abs > 10 || (y-@ydown).abs > 10 )
            @dragging = true
        end
    end
end

def onLButtonDown(flags, x, y, view)
    if( @state == 0 )
        @ip1.pick view, x, y
        if( @ip1.valid? )
            @state = 1
            Sketchup::set_status_text $exStrings.GetString("Select second end"), SB_PROMPT
            @xdown = x
            @ydown = y
        end
    else
        # create the line on the second click
        if( @ip2.valid? )
            self.create_geometry(@ip1.position, @ip2.position,view)
            self.reset(view)
        end
    end
    # Clear any inference lock
    view.lock_inference
end

def onLButtonUp(flags, x, y, view)
    if( @dragging && @ip2.valid? )
        self.create_geometry(@ip1.position, @ip2.position,view)
        self.reset(view)
    end
end

def onKeyDown(key, repeat, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY && repeat == 1 )
        @shift_down_time = Time.now
        
        # if we already have an inference lock, then unlock it
        if( view.inference_locked? )
            # calling lock_inference with no arguments actually unlocks
            view.lock_inference
        elsif( @state == 0 && @ip1.valid? )
            view.lock_inference @ip1
        elsif( @state == 1 && @ip2.valid? )
            view.lock_inference @ip2, @ip1
        end
    end
end

def onKeyUp(key, repeat, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY &&
        view.inference_locked? &&
        (Time.now - @shift_down_time) > 0.5 )
        view.lock_inference
    end
end

def onUserText(text, view)
    return if not @state == 1
    return if not @ip2.valid?

    begin
        value = text.to_l
    rescue
        # Error parsing the text
        UI.beep
        puts "Cannot convert #{text} to a Length"
        value = nil
        Sketchup::set_status_text "", SB_VCB_VALUE
    end
    return if !value

    # Compute the direction and the second point
    pt1 = @ip1.position
    vec = @ip2.position - pt1
    if( vec.length == 0.0 )
        UI.beep
        return
    end
    vec.length = value
    pt2 = pt1 + vec

    # Create a line
    self.create_geometry(pt1, pt2, view)
    self.reset(view)
end

def draw(view)
    if( @ip1.valid? )
        if( @ip1.display? )
            @ip1.draw(view)
            @drawn = true
        end
        
        if( @ip2.valid? )
            @ip2.draw(view) if( @ip2.display? )
            view.set_color_from_line(@ip1, @ip2)
            self.draw_geometry(@ip1.position, @ip2.position, view)
            @drawn = true
        end
    end
end

def onCancel(flag, view)
    self.reset(view)
    Sketchup.send_action "selectSelectionTool:"
end

def reset(view)
    @state = 0
    Sketchup::set_status_text($exStrings.GetString("Select first end"), SB_PROMPT)
    @ip1.clear
    @ip2.clear
    
    if( view )
        view.tooltip = nil
        view.invalidate if @drawn
    end
    
    @drawn = false
    @dragging = false
end

def create_geometry(p1, p2, view)
  model = Sketchup.active_model
  model.start_operation "Component Paintbrush"
  $cbt_radius = p1.distance p2
  n_comps = cbt_n_points(@shape, @ip1.position, @ip2.position, $cbt_pressure,nil)
  cbt_random_points(n_comps, @shape, @ip1.position, @ip2.position)
  model.commit_operation
end

def draw_geometry(pt1, pt2, view)
    view.draw_line(pt1, pt2)
end

end # class ComponentLineBrushTool


#----------------------------------------------------------------------------------------------------------
#----------------------------------------------------------------------------------------------------------

class ComponentSprayBrushTool

def initialize
    @ip1 = nil
    @xdown = 0
    @ydown = 0
    @shape = "Circle"
end

def activate
    @ip1 = Sketchup::InputPoint.new
    @ip = Sketchup::InputPoint.new
    @drawn = false
    Sketchup::set_status_text("Spray radius", SB_VCB_LABEL)
    self.reset(nil)
end

def onMouseLeave(view)
  Sketchup.send_action "selectSelectionTool:"
end

def deactivate(view)
    view.invalidate if @drawn
end

def onMouseMove(flags, x, y, view)
  @ip.pick view, x, y
  if( @ip != @ip1 )
    view.invalidate if( @ip.display? or @ip1.display? )
    @ip1.copy! @ip
    view.tooltip = @ip1.tooltip
  end
  
end

def onLButtonDown(flags, x, y, view)
        @ip1.pick view, x, y
        if( @ip1.valid? )
            Sketchup::set_status_text("Point", SB_PROMPT)
            @xdown = x
            @ydown = y
        end
    # Clear any inference lock
    view.lock_inference
    self.create_geometry(@ip1.position,view)
end

def onLButtonUp(flags, x, y, view)
  if( @dragging && @ip2.valid? )
    self.reset(view)
  end
end

def onKeyDown(key, repeat, flags, view)
  if( key == CONSTRAIN_MODIFIER_KEY && repeat == 1 )
    @shift_down_time = Time.now
    # if we already have an inference lock, then unlock it
    if( view.inference_locked? )
      # calling lock_inference with no arguments actually unlocks
      view.lock_inference
    end
  end
end

def onKeyUp(key, repeat, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY &&
        view.inference_locked? &&
        (Time.now - @shift_down_time) > 0.5 )
        view.lock_inference
    end
end

def onUserText(text, view)
    begin
        value = text.to_l
    rescue
        # Error parsing the text
        UI.beep
        puts "Cannot convert #{text} to a radius"
        value = nil
        Sketchup::set_status_text "", SB_VCB_VALUE
    end
    return if !value
    if value
      $cbt_spray_radius = value.to_l
    end    
    self.reset(view)
end

def draw(view)
  if( @ip1.valid? )
    if( @ip1.display? )
      @ip1.draw(view)
      v_rad = Geom::Vector3d.new($cbt_spray_radius,0,0)
      self.draw_geometry(@ip1.position, @ip1.position.offset(v_rad), $cbt_spray_radius, view)
      @drawn = true
    end
  end
end

def onCancel(flag, view)
    self.reset(view)
    Sketchup.send_action "selectSelectionTool:"
end

def reset(view)
  @state = 0
  Sketchup::set_status_text($exStrings.GetString("Point"), SB_PROMPT)
  @ip1.clear
  if( view )
    view.tooltip = nil
    view.invalidate if @drawn
  end
  @drawn = false
  @dragging = false
end

def create_geometry(p1, view)
  model = Sketchup.active_model
  model.start_operation "Component Paintbrush (Spray)"
  p2= p1.offset(Geom::Vector3d.new($cbt_spray_radius,0,0))
  $cbt_radius = $cbt_spray_radius
  n_comps = cbt_n_points(@shape,p1, $cbt_radius, $cbt_pressure, nil)
  cbt_random_points(n_comps, @shape, p1, p2)
  model.commit_operation
end

# Draw the geometry
def draw_geometry(pt1, pt2, radius, view)
  sides = 12
  ang = Math::PI*2/sides
  pts=[]
  top = [pt1.x,pt1.y, pt1.z+$cbt_spray_radius]
  
  0.upto(sides-1) do |i|
    pts.push(pt2.transform(Geom::Transformation.rotation(pt1,[0,0,1],i*ang)))
  end
  
  0.upto(sides-1) do |i|
    pts.push(pt2.transform(Geom::Transformation.rotation(pt1,[0,0,1],i*ang)))
    view.draw GL_LINES, [top, pt2.transform(Geom::Transformation.rotation(pt1,[0,0,1],i*ang))]
  end
  pts.push pts.first
  view.draw_polyline pts
  
end

end # class ComponentSprayBrushTool
#----------------------------------------------------------------------------------------------------------
#----------------------------------------------------------------------------------------------------------

class ComponentRectBrushTool

def initialize
    @ip = Sketchup::InputPoint.new
    @ip1 = Sketchup::InputPoint.new
    @shape = $cbt_shape
    reset
end

def reset
    @pts = []
    @state = 0
    @ip1.clear
    @drawn = false
    Sketchup::set_status_text "", SB_VCB_LABEL
    Sketchup::set_status_text "", SB_VCB_VALUE
    Sketchup::set_status_text "Start point"
    @shift_down_time = Time.now
end

def activate
    self.reset
end

def deactivate(view)
    view.invalidate if @drawn
end

def onMouseLeave(view)
  Sketchup.send_action "selectSelectionTool:"
end

def set_current_point(x, y, view)
    if( !@ip.pick(view, x, y, @ip1) )
        return false
    end
    need_draw = true
    
    # Set the tooltip that will be displayed
    view.tooltip = @ip.tooltip
        
    # Compute points
    case @state
    when 0
        @pts[0] = @ip.position
        @pts[4] = @pts[0]
        need_draw = @ip.display? || @drawn
    when 1
        @pts[1] = @ip.position
        @width = @pts[0].distance @pts[1]
        Sketchup::set_status_text @width.to_s, SB_VCB_VALUE
    when 2
        pt1 = @ip.position
        pt2 = pt1.project_to_line @pts
        vec = pt1 - pt2
        @height = vec.length
        if( @height > 0 )
            # test for a square
            square_point = pt2.offset(vec, @width)
            if( view.pick_helper.test_point(square_point, x, y) )
                @height = @width
                @pts[2] = @pts[1].offset(vec, @height)
                @pts[3] = @pts[0].offset(vec, @height)
                view.tooltip = "Square"
            else
                @pts[2] = @pts[1].offset(vec)
                @pts[3] = @pts[0].offset(vec)
            end
        else
            @pts[2] = @pts[1]
            @pts[3] = @pts[0]
        end
        Sketchup::set_status_text @height.to_s, SB_VCB_VALUE
    end

    view.invalidate if need_draw
end

def onMouseMove(flags, x, y, view)
    self.set_current_point(x, y, view)
end

def create_geometry
  model = Sketchup.active_model
  model.start_operation "Component Paintbrush"
  
  $cbt_radius = (@pts[0].distance @pts[2]) / 2.0
  n_comps = cbt_n_points(@shape, @pts[0], @pts[2], $cbt_pressure, @pts)
  diag = @pts[0].vector_to @pts[2]
  diag.length = (@pts[0].distance @pts[2])/2.0
  rect_center = @pts[0].offset(diag)
  @pts.pop
  cbt_random_points(n_comps, @shape, rect_center, @pts)

  model.commit_operation
  self.reset
end

def increment_state
    @state += 1
    case @state
    when 1
        @ip1.copy! @ip
        Sketchup::set_status_text "Second point"
        Sketchup::set_status_text "Length", SB_VCB_LABEL
        Sketchup::set_status_text "", SB_VCB_VALUE
    when 2
        @ip1.clear
        Sketchup::set_status_text "Third point"
        Sketchup::set_status_text "Width", SB_VCB_LABEL
        Sketchup::set_status_text "", SB_VCB_VALUE
    when 3
        self.create_geometry
    end
end

def onLButtonDown(flags, x, y, view)
    self.set_current_point(x, y, view)
    self.increment_state
    view.lock_inference
end

def onCancel(flag, view)
    view.invalidate if @drawn
    self.reset
end

# This is called when the user types a value into the VCB
def onUserText(text, view)
    begin
        value = text.to_l
    rescue
        # Error parsing the text
        UI.beep
        value = nil
        Sketchup::set_status_text "", SB_VCB_VALUE
    end
    return if !value
    
    case @state
    when 1
        # update the width
        vec = @pts[1] - @pts[0]
        if( vec.length > 0.0 )
            vec.length = value
            @pts[1] = @pts[0].offset(vec)
            view.invalidate
            self.increment_state
        end
    when 2
        # update the height
        vec = @pts[3] - @pts[0]
        if( vec.length > 0.0 )
            vec.length = value
            @pts[2] = @pts[1].offset(vec)
            @pts[3] = @pts[0].offset(vec)
            self.increment_state
        end
    end
end

def getExtents
    bb = Geom::BoundingBox.new
    case @state
    when 0
        # We are getting the first point
        if( @ip.valid? && @ip.display? )
            bb.add @ip.position
        end
    when 1
        bb.add @pts[0]
        bb.add @pts[1]
    when 2
        bb.add @pts
    end
    bb
end

def draw(view)
    @drawn = false
    
    # Show the current input point
    if( @ip.valid? && @ip.display? )
        @ip.draw(view)
        @drawn = true
    end

    # show the rectangle
    if( @state == 1 )
        # just draw a line from the start to the end point
        view.set_color_from_line(@ip1, @ip)
        inference_locked = view.inference_locked?
        view.line_width = 3 if inference_locked
        view.draw(GL_LINE_STRIP, @pts[0], @pts[1])
        view.line_width = 1 if inference_locked
        @drawn = true
    elsif( @state > 1 )
        # draw the curve
        view.drawing_color = "red"
        view.draw(GL_LINE_STRIP, @pts)
        @drawn = true
    end
end

def onKeyDown(key, rpt, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY && rpt == 1 )
        @shift_down_time = Time.now
        
        # if we already have an inference lock, then unlock it
        if( view.inference_locked? )
            view.lock_inference
        elsif( @state == 0 )
            view.lock_inference @ip
        elsif( @state == 1 )
            view.lock_inference @ip, @ip1
        end
    end
end

def onKeyUp(key, rpt, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY &&
        view.inference_locked? &&
        (Time.now - @shift_down_time) > 0.5 )
        view.lock_inference
    end
end

end # of class RectangleTool



#----------------------------------------------------------------------------------------------------------
#----------------------------------------------------------------------------------------------------------

def facesbrushtool(shape)
model = Sketchup.active_model
sel = model.selection
if not sel.empty?
  clean_sel
  else
  UI.messagebox("No selection.")
  return nil
end
model.start_operation "Component Paintbrush (Faces set)"
bb = cbt_faces_bounds(cbt_sort_faces(model.selection))
corners = [(bb.corner 0),(bb.corner 1),(bb.corner 3),(bb.corner 2)]
# Virtual polygon of BB from which to cast rays
pts = map_z(corners, cbt_model_top+1000)
$cbt_radius = (pts[0].distance pts[2]) / 2.0
n_comps = cbt_n_points(shape, pts[0], pts[2], $cbt_pressure, pts)
diag = pts[0].vector_to pts[2]
diag.length = (pts[0].distance pts[2])/2.0
center = pts[0].offset(diag)
cbt_random_points(n_comps, shape, center, pts)

model.commit_operation
end

def componentbrushtool
  # Exit current command, if any
  Sketchup.send_action "selectSelectionTool:"
  # Dialog box brush style
  cbt_dialog_main
  # Switch
  case $cbt_shape
    when "Point"
      Sketchup.active_model.select_tool ComponentPointBrushTool.new
    when "Line"
      Sketchup.active_model.select_tool ComponentLineBrushTool.new
    when "Rectangle"
      Sketchup.active_model.select_tool ComponentRectBrushTool.new
    when "Selection"
      facesbrushtool("Selection")
    when "Spray"
      Sketchup.active_model.select_tool ComponentSprayBrushTool.new
    else
      Sketchup.active_model.select_tool ComponentNgonBrushTool.new
    end

  # Restore dialog setings
  cbt_restore_dialog_settings  
end


#----------------------------------------------------------------------------------------------------------
#----------------------------------------------------------------------------------------------------------

# initialize basic settings
cbt_set_basic

if (not file_loaded?("compo_spray.rb"))
#  component_paintbrush_menu = UI.menu("Draw").add_submenu("Component Spray")
#  component_paintbrush_menu.add_item("Component Spray Tool") { componentbrushtool }
#  component_paintbrush_menu.add_item("Set default settings") { cbt_dialog_set_defaults }
end
file_loaded("compo_spray.rb")
