#        Name : projection.rb
# Description : 
#      Author : Didier Bur
#       Usage : 
#        Date : 11.2005
#        Type : geometric tools set
#              : 10.11.07 updated
# more user-friendly commands (click points instead of selecting objects for plane projections and vectors
# project faces along Z and along vector gathered in one command.
# ExtrudelineTool included
# lines2cyl and credits suppressed
#Cline at intersection of faces enhanced
# new icons

require 'sketchup.rb'

def create_faces_dialog
$proj_create_faces = "No" if not $proj_create_faces
create_faces = ["Yes","No"]
enums = [create_faces.join("|")]
prompts = ["Create face(s): "]
values = [$proj_create_faces]
results = inputbox prompts, values, enums, "Projection Options"
return if not results
$proj_create_faces = results[0]
end

def create_lines_dialog
$proj_create_lines = "No" if not $proj_create_lines
create_lines = ["Yes","No"]
enums = [create_lines.join("|")]
prompts = ["Create line(s): "]
values = [$proj_create_lines]
results = inputbox prompts, values, enums, "Projection Options"
return if not results
$proj_create_lines = results[0]
end

# Removes from selection all objects of types contained in object_types array
def sel_remove(sel, object_types)
  sel.each do |obj|
    if object_types.include? obj.typename
      sel.remove(obj)
    end
  end
return sel
end

# Removes from selection all objects of types NOT contained in object_types array
def sel_keep(sel, object_types)
  sel.each do |obj|
    if (not object_types.include? obj.typename)
      sel.remove(obj)
    end
  end
return sel
end

#-----------------------------------------------------------------------------------------------------
#                                        PROJECTION FUNCTIONS
#-----------------------------------------------------------------------------------------------------

# --------------------------------------------------- projects the selected point perpendicular to the selected line
def project_point_line

model = Sketchup.active_model
model.start_operation "Project point on line"
entities = model.active_entities
ss = model.selection

case ss.length
  when 0
    UI.messagebox("No selection.\nPlease select a construction point and a line.")
  return
  when 1
    UI.messagebox("You have selected only one " + ss[0].typename + ".\nPlease select a construction point and a line.")
  return
  when 2
    # Who's the edge and who's the point ?
    ent1 = ss[0]
    ent2 = ss[1]
    if( ent1.typename == "ConstructionPoint")
      pt = ent1
      ln = ent2
    else
      ln = ent1
      pt = ent2
    end
    #Dialog
    (create_lines_dialog)
  else
    UI.messagebox("You have selected more than 2 objects.\nPlease select a construction point and a line.")
    return
end

# Projection 
Sketchup.active_model.active_entities.add_cpoint (pt.position.project_to_line([ln.vertices[0], ln.vertices[1]]))
if $proj_create_lines == "Yes"
  Sketchup.active_model.active_entities.add_line(pt.position, pt.position.project_to_line([ln.vertices[0], ln.vertices[1]]))
end
end

# --------------------------------------------------- projects the selected point perpendicular to the selected plan
def project_point_plane

model = Sketchup.active_model
model.start_operation "Project point on plane"
entities = model.active_entities
ss = model.selection

case ss.length
  when 0
    UI.messagebox("No selection.\nPlease select a construction point and a face.")
  return
  when 1
    UI.messagebox("You have selected only one " + ss[0].typename + ".\nPlease select a construction point and a face.")
  return
  when 2
    # Who's the edge and who's the face ?
    ent1 = ss[0]
    ent2 = ss[1]
    if( ent1.typename == "ConstructionPoint")
      pt = ent1
      fc = ent2
    else
      fc = ent1
      pt = ent2
    end
    #Dialog
    (create_lines_dialog)
  else
    UI.messagebox("You have selected more than 2 objects.\nPlease select a construction point and a face.")
    return
end
vec = fc.normal.normalize!

# Projection
plan = fc.plane
Sketchup.active_model.active_entities.add_cpoint pt.position.project_to_plane(plan[0], plan[1], plan[2], plan[3])
if $proj_create_lines == "Yes"
  Sketchup.active_model.active_entities.add_line(pt.position, pt.position.project_to_plane(plan[0], plan[1], plan[2], plan[3]))
end
end

#  --------------------------------------------------- projects the selected line perpendicular to the selected plane
def project_lines_plane

model = Sketchup.active_model
model.start_operation "Project lines on plane"
entities = model.active_entities
ss = model.selection
# Dialog box
(create_faces_dialog)

case ss.length
  when 0
    UI.messagebox("No selection.\nPlease select line(s) and a face.")
  return
  when 1
    UI.messagebox("You have selected only one " + ss[0].typename + ".\nPlease select line(s) and a face.")
  return
  when 2
    # Edge and face ?
    ent1 = ss[0]
    ent2 = ss[1]
    if( ent1.typename == "Edge") and ( ent2.typename == "Face")
      e = ent1
      fc = ent2
    else
      fc = ent1
      e = ent2
    end
    # Projection
    plan = fc.plane
    p1 = e.start.position.project_to_plane(plan[0], plan[1], plan[2], plan[3])
    p2 = e.end.position.project_to_plane(plan[0], plan[1], plan[2], plan[3])
    Sketchup.active_model.active_entities.add_line( p1, p2 )
    if(  $proj_create_faces == "Yes" )
      Sketchup.active_model.active_entities.add_face( e.vertices[0].position, e.vertices[1].position, p2, p1 )
    end
    else
    #Set of lines to be projected
    count_faces = 0
    ss.each do |e|
      if e.typename == "Face"
        fc = e
        count_faces = count_faces + 1
      end
    end
    case count_faces 
      when 0
        UI.messagebox("No face selected.\nPlease try again.")
        return
      when 1
        ss=sel_keep(ss,["Edge"])
        plan = fc.plane
        ss.remove(fc)
        # Projections
        ss.each do |e|
          p1 = e.vertices[0].position.project_to_plane(plan[0], plan[1], plan[2], plan[3])
          p2 = e.vertices[1].position.project_to_plane(plan[0], plan[1], plan[2], plan[3])
          Sketchup.active_model.active_entities.add_line( p1, p2 )
          if(  $proj_create_faces == "Yes" )
            Sketchup.active_model.active_entities.add_face( e.vertices[0].position, e.vertices[1].position, p2, p1 )
          end
      end
      else  
      UI.messagebox("More than one face selected.\nPlease try again.")
      return
    end
end  
end

#  --------------------------------------------------- projects the selected line perpendicular to the selected set of faces
def project_line_planes

model = Sketchup.active_model
model.start_operation "Project line on faces"
entities = model.active_entities
ss = model.selection

# Dialog box
(create_faces_dialog)
case ss.length
  when 0
    UI.messagebox("No selection.\nPlease select a line and face(s).")
  return
  when 1
    UI.messagebox("You have selected only one " + ss[0].typename + ".\nPlease select a line and face(s).")
  return
  when 2
  # Edge and face ?
    ent1 = ss[0]
    ent2 = ss[1]
    if( ent1.typename == "Edge") and ( ent2.typename == "Face")
      e = ent1
      fc = ent2
    elsif ( ent2.typename == "Edge") and ( ent1.typename == "Face")
      fc = ent1
      e = ent2
    else
      UI.messagebox("Incorrect selection.\nPlease try again.")
      return
    end
    # Projection
    plan = fc.plane
    p1 = e.start.position.project_to_plane(plan[0], plan[1], plan[2], plan[3])
    p2 = e.end.position.project_to_plane(plan[0], plan[1], plan[2], plan[3])
    Sketchup.active_model.active_entities.add_line( p1, p2 )
    if(  $proj_create_faces == "Yes" )
      Sketchup.active_model.active_entities.add_face( e.vertices[0].position, e.vertices[1].position, p2, p1 )
    end
    
    else
    
    # Set of faces to project on
    ss.each do |obj|
      if( obj.typename == "Edge")
        e = obj
        ss.remove(obj)
      end
    end
    ss=sel_keep(ss,["Face"])
    ss.each do |f|
      plan = f.plane
      p1 = e.start.position.project_to_plane(plan[0], plan[1], plan[2], plan[3])
      p2 = e.end.position.project_to_plane(plan[0], plan[1], plan[2], plan[3])
      Sketchup.active_model.active_entities.add_line( p1, p2 )
      if(  $proj_create_faces == "Yes" )
        Sketchup.active_model.active_entities.add_face( e.start.position, e.end.position, p2, p1 )
      end
    end
end

end

# --------------------------------------------------- projects the selected faces perpendicular to the selected plane
def project_faces_plane

model = Sketchup.active_model
model.start_operation "Project faces on face"
entities = model.active_entities
ss = model.selection

case ss.length
  when 0
    UI.messagebox("No selection.\nPlease select faces.")
  return
  when 1
    UI.messagebox("You have selected only one " + ss[0].typename + ".\nPlease select faces.")
  return
end

# Who are the faces and who's the face to project on ?
i = 0
plane_face = []
project_face = []
ss.each do |obj|
  if( obj.typename == "Edge")
    plane_face.push obj.faces
    ss.remove(obj)
  end
end
plane_face.flatten!

# Keep only plane_face that is selected
plane_face.each do |f|
  if ss.contains? f
    project_face.push(f)
  end
end
project_face.uniq!

case project_face.length
  when 0
    UI.messagebox("Face on which to project isn't selected.")
    return
  when 1
    ss.remove(project_face[0])
    plan = project_face[0].plane
    # Dialog box
    (create_faces_dialog)
    # Projections
    ss.each do |f|
      edges = f.edges
      edges.each do |e|
        points = e.vertices
        p1 = points[0].position.project_to_plane(plan[0], plan[1], plan[2], plan[3])
        p2 = points[1].position.project_to_plane(plan[0], plan[1], plan[2], plan[3])
        Sketchup.active_model.active_entities.add_line( p1, p2 )
        if(  $proj_create_faces == "Yes" )
          Sketchup.active_model.active_entities.add_face( points[0].position, points[1].position, p2, p1 )
        end
      end
    end
  else
    UI.messagebox("Too much faces on which to project selected.\nPlease try again.")
    return
end

end

# --------------------------------------------------- Projects the selected faces perpendicular to the selected plane

class ProjectFacesPerpendicularTool
def initialize
    $ss = Sketchup.active_model.selection
    $points=[]
    @ip1 = nil
    @ip2 = nil
    @ip3 = nil
    @xdown = 0
    @ydown = 0
    @state = 0
end

def activate
    Sketchup.active_model.start_operation "Project faces perpendicular to plane"
    @ip1 = Sketchup::InputPoint.new
    @ip2 = Sketchup::InputPoint.new
    @ip3 = Sketchup::InputPoint.new
    @ip = Sketchup::InputPoint.new
    @drawn = false
    self.reset(nil)
end

def deactivate(view)
    view.invalidate if @drawn
end

def onMouseMove(flags, x, y, view)
@ip.pick view, x, y
  if( @ip != @ip1 )
    view.invalidate if( @ip.display? or @ip1.display? )
    @ip1.copy! @ip
  end

case @state
    when 0
      view.tooltip = "First point of plane"
      Sketchup::set_status_text("First point of plane", SB_PROMPT)
    when 1
      view.tooltip = "Second point of plane"
      Sketchup::set_status_text("Second point of plane", SB_PROMPT)
    when 2
      view.tooltip = "Third point of plane"
      Sketchup::set_status_text("Third point of plane", SB_PROMPT)
  end
end

def onLButtonDown(flags, x, y, view)
  case @state
    when 0
      @xdown = x
      @ydown = y
      @ip1.pick view, x, y
      view.tooltip = @ip1.tooltip if( @ip1.valid? )
      $points.push @ip1.position
      @state = 1
      view.invalidate
    when 1
      @xdown = x
      @ydown = y
      @ip2.pick view, x, y, @ip1
      view.tooltip = @ip2.tooltip if( @ip2.valid? )
      $points.push @ip2.position
      @state = 2
      view.invalidate 
    when 2
      @xdown = x
      @ydown = y
      @ip3.pick view, x, y, @ip1
      view.tooltip = @ip3.tooltip if( @ip3.valid? )
      $points.push @ip3.position
      @state = 3
      view.invalidate
    # Clear any inference lock
    view.lock_inference
  end #case
end

# The onLButtonUp method is called when the user releases the left mouse button.
def onLButtonUp(flags, x, y, view)
self.reset(view)
self.create_geometry(view) if @state == 3
end

def onKeyDown(key, repeat, flags, view)
if( key == CONSTRAIN_MODIFIER_KEY && repeat == 1 )
        @shift_down_time = Time.now
        # if we already have an inference lock, then unlock it
        if( view.inference_locked? )
        # calling lock_inference with no arguments actually unlocks
            view.lock_inference
        elsif( @state == 0 && @ip1.valid? )
            view.lock_inference @ip1
        elsif( @state == 1 && @ip2.valid? )
            view.lock_inference @ip2
        elsif( @state == 2 && @ip3.valid? )
            view.lock_inference @ip3
        end
    end
end

def onKeyUp(key, repeat, flags, view)
return if not @state == 3
    if( key == CONSTRAIN_MODIFIER_KEY &&
        view.inference_locked? &&
        (Time.now - @shift_down_time) > 0.5 )
        view.lock_inference
    end
end


def onUserText(text, view)

    return if not @state == 3
    return if not @ip5.valid?
    
    begin
        @value = text.to_l
    rescue
        # Error parsing the text
        UI.messagebox("Cannot convert #{text} to a length.")
        @value = nil
        Sketchup::set_status_text "", SB_VCB_VALUE
    end
    return if !@value
    self.reset(view)
end

def draw(view)
    if( @ip1.valid? )
        if( @ip1.display? )
            @ip1.draw(view)
            @drawn = true
        end
    end
    
  if $points.length == 1
    view.draw_points $points[0], 6, 1, "yellow"
  end
  
  if $points.length == 2
    view.line_width = 2
    view.draw_points $points[0], 6, 2, "yellow"
    view.draw_points $points[1], 6, 1, "red"
    # Draw temp axis system
    view.drawing_color = "red"
    view.draw_line($points[0],$points[1])
  end
  
  if $points.length == 3
    view.line_width = 2
    view.draw_points $points[0], 6, 2, "yellow"
    view.draw_points $points[1], 6, 2, "red"
    view.draw_points $points[2], 6, 2, "green"
    # Draw temp axis system
    view.drawing_color = "red"
    view.draw_line($points[0],$points[1])
    view.drawing_color = "green"
    view.draw_line($points[0],$points[2])
    vz=($points[0] - $points[1]).cross($points[0] - $points[2])
    vz.length=(($points[0] - $points[1]).length / 2.0)
    view.drawing_color = "blue"
    view.draw_points $points[0].offset(vz), 6, 2, "blue"
    view.draw_line($points[0],$points[0].offset(vz))
  end
  view.lock_inference
end

# onCancel is called when the user hits the escape key
def onCancel(flag, view)
self.reset(view)
    self.draw_geometry(view) if @state < 3
    if $points.length < 3
      # We got less than 5 points
      UI.messagebox($points.length.to_s + " points are not enough to define a plane !")
      self.reset(view)
    end
end

# Reset the tool back to its initial state
def reset(view)
    #@state = 0
    Sketchup::set_status_text("First point of plane", SB_PROMPT)
    @ip1.clear
    @ip2.clear
    @ip3.clear
    if( view )
        view.tooltip = nil
        view.invalidate if @drawn
    end
    @drawn = false
    @dragging = false
end

# Create new geometry when the user has picked three points.
def create_geometry(view)
model = Sketchup.active_model
model.start_operation "Project faces perpendicular to plane"
entities = model.active_entities
ss = sel_keep(model.selection, ["Face"])
ss = sel_keep(ss,["Face"])

# Plane's equation and vector
vz=($points[0] - $points[1]).cross($points[0] - $points[2])
a = vz.x*$points[1].x
b = vz.y*$points[1].y
c = vz.z*$points[1].z
d = a + b + c
d = 0.0 - d
plan = [vz.x,vz.y,vz.z,d]
vect = ($points[0] - $points[2])

# Dialog
(create_faces_dialog)
 
# Projections
i = 0
trans = Geom::Transformation.translation(vect)

ss.each do |f|
  edges = f.edges
  edges.each do |e|
    points = e.vertices
    p1 = points[0].position.project_to_plane(plan[0], plan[1], plan[2], plan[3])
    p2 = points[1].position.project_to_plane(plan[0], plan[1], plan[2], plan[3])
    Sketchup.active_model.active_entities.add_line( p1, p2 )
    if(  $proj_create_faces == "Yes" )
      Sketchup.active_model.active_entities.add_face( points[0].position, points[1].position, p2, p1 )
    end
  end
end
model.commit_operation
Sketchup.send_action "selectSelectionTool:"

end

# Draw the geometry (temp)
def draw_geometry(pt1, pt2, view)
    view.draw_line(pt1, pt2)
end

end # class ProjectFacesPerpendicularTool


# --------------------------------------------------- Projects the selected faces to the selected plane, along a vector

class ProjectFacesVectorTool
def initialize
    $ss = Sketchup.active_model.selection
    $points=[]
    $ip1 = nil
    $ip2 = nil
    $ip3 = nil
    $ip4 = nil
    $ip5 = nil
    @xdown = 0
    @ydown = 0
    @state = 0
end

def activate
    Sketchup.active_model.start_operation "Project faces on plane"
    $ip1 = Sketchup::InputPoint.new
    $ip2 = Sketchup::InputPoint.new
    $ip3 = Sketchup::InputPoint.new
    $ip4 = Sketchup::InputPoint.new
    $ip5 = Sketchup::InputPoint.new
    @ip = Sketchup::InputPoint.new
    @drawn = false
    self.reset(nil)
end

def deactivate(view)
    view.invalidate if @drawn
end

def onMouseMove(flags, x, y, view)
@ip.pick view, x, y
  if( @ip != $ip1 )
    view.invalidate if( @ip.display? or $ip1.display? )
    $ip1.copy! @ip
  end

case @state
    when 0
      view.tooltip = "First point of plane"
      Sketchup::set_status_text("First point of plane", SB_PROMPT)
    when 1
      view.tooltip = "Second point of plane"
      Sketchup::set_status_text("Second point of plane", SB_PROMPT)
    when 2
      view.tooltip = "Third point of plane"
      Sketchup::set_status_text("Third point of plane", SB_PROMPT)
    when 3
      view.tooltip = "First point of vector"
      Sketchup::set_status_text("First point of vector", SB_PROMPT)
    when 4
      view.tooltip = "Second point of vector"
      Sketchup::set_status_text("Second point of vector", SB_PROMPT)
      if( @ip.valid? )
        length = $ip4.position.distance(@ip.position)
        Sketchup::set_status_text length.to_s, SB_VCB_VALUE
        if( (x-@xdown).abs > 10 || (y-@ydown).abs > 10 )
            @dragging = true
        end
      end
  end
end

def onLButtonDown(flags, x, y, view)
  case @state
    when 0
      @xdown = x
      @ydown = y
      $ip1.pick view, x, y
      view.tooltip = $ip1.tooltip if( $ip1.valid? )
      $points.push $ip1.position
      @state = 1
      view.invalidate
    when 1
      @xdown = x
      @ydown = y
      $ip2.pick view, x, y, $ip1
      view.tooltip = $ip2.tooltip if( $ip2.valid? )
      $points.push $ip2.position
      @state = 2
      view.invalidate 
    when 2
      @xdown = x
      @ydown = y
      $ip3.pick view, x, y, $ip1
      view.tooltip = $ip3.tooltip if( $ip3.valid? )
      $points.push $ip3.position
      @state = 3
      view.invalidate
    when 3
      @xdown = x
      @ydown = y
      $ip4.pick view, x, y
      view.tooltip = $ip4.tooltip if( $ip4.valid? )
      $points.push $ip4.position
      @state = 4
      view.invalidate
    when 4
      @xdown = x
      @ydown = y
      $ip5.pick view, x, y, $ip4
      view.tooltip = $ip5.tooltip if( $ip5.valid? )
      $points.push $ip5.position
      @state = 5
      view.invalidate
  end #case
# Clear any inference lock
view.lock_inference  
end

# The onLButtonUp method is called when the user releases the left mouse button.
def onLButtonUp(flags, x, y, view)
  self.create_geometry(view) if @state == 5
end

def onKeyDown(key, repeat, flags, view)
if( key == CONSTRAIN_MODIFIER_KEY && repeat == 1 )
        @shift_down_time = Time.now
        # if we already have an inference lock, then unlock it
        if( view.inference_locked? )
        # calling lock_inference with no arguments actually unlocks
            view.lock_inference
        elsif( @state == 0 && $ip1.valid? )
            view.lock_inference $ip1
        elsif( @state == 1 && $ip2.valid? )
            view.lock_inference $ip1
        elsif( @state == 2 && $ip3.valid? )
            view.lock_inference $ip1
        elsif( @state == 3 && $ip4.valid? )
            view.lock_inference $ip4
        elsif( @state == 4 && $ip5.valid? )
            view.lock_inference $ip5
        end
    end
end

def onKeyUp(key, repeat, flags, view)
#return if not @state == 5
    if( key == CONSTRAIN_MODIFIER_KEY &&
        view.inference_locked? &&
        (Time.now - @shift_down_time) > 0.5 )
        view.lock_inference
    end
end


def onUserText(text, view)

    return if @state != 4
    #return if not $ip5.valid?
    
    begin
        @value = text.to_l
    rescue
        # Error parsing the text
        UI.messagebox("Cannot convert #{text} to a length.")
        @value = nil
        Sketchup::set_status_text "", SB_VCB_VALUE
    end
    return if !@value

    @pt1 = $ip4.position
    @vec = $ip5.position - @pt1
    if( @vec.length == 0.0 )
        return
    end

    @vec.length = @value
    @pt2=@pt1.offset @vec
    
    #self.create_geometry(@pt1, @pt2, view)
    $points[4] = @pt2
    @state = 5
    self.create_geometry(view)
    #self.reset(view)
end

def draw(view)
    if( $ip1.valid? )
        if( $ip1.display? )
            $ip1.draw(view)
            @drawn = true
        end
    end
    
  if $points.length == 1
    view.draw_points $points[0], 6, 1, "yellow"
  end
  
  if $points.length == 2
    view.line_width = 2
    view.draw_points $points[0], 6, 2, "yellow"
    view.draw_points $points[1], 6, 1, "red"
    # Draw temp axis system
    view.drawing_color = "red"
    view.draw_line($points[0],$points[1])
  end
  
  if $points.length == 3
    view.line_width = 2
    view.draw_points $points[0], 6, 2, "yellow"
    view.draw_points $points[1], 6, 2, "red"
    view.draw_points $points[2], 6, 2, "green"
    # Draw temp axis system
    view.drawing_color = "red"
    view.draw_line($points[0],$points[1])
    view.drawing_color = "green"
    view.draw_line($points[0],$points[2])
    vz=($points[0] - $points[1]).cross($points[0] - $points[2])
    vz.length=(($points[0] - $points[1]).length / 2.0)
    view.drawing_color = "blue"
    view.draw_points $points[0].offset(vz), 6, 2, "blue"
    view.draw_line($points[0],$points[0].offset(vz))
  end
  
  if $points.length == 4
    view.draw_points $points[3], 8, 1, "black"
    self.draw_geometry($points[3],@ip.position,view)
  end
  
  if $points.length == 5
    view.draw_points $points[3], 8, 1, "black"
    view.draw_points $points[4], 8, 1, "black"
    
  end
  view.lock_inference
end

# onCancel is called when the user hits the escape key
def onCancel(flag, view)
self.reset(view)
    #self.draw_geometry(view) if @state < 5
    if $points.length < 5
      # We got less than 5 points
      UI.messagebox($points.length.to_s + " points are not enough for this projection !")
      self.reset(view)
      Sketchup.active_model.commit_operation
      Sketchup.send_action "selectSelectionTool:"
    end
end


# Reset the tool back to its initial state
def reset(view)
    #@state = 0
    Sketchup::set_status_text("First point of plane", SB_PROMPT)
    $ip1.clear
    $ip2.clear
    $ip3.clear
    $ip4.clear
    $ip5.clear
    if( view )
        view.tooltip = nil
        view.invalidate if @drawn
    end
    @drawn = false
    @dragging = false
end

# Create new geometry when the user has picked five points.
def create_geometry(view)
model = Sketchup.active_model
model.start_operation "Project faces on plane along vector"
entities = model.active_entities
ss = sel_keep(model.selection, ["Face"])

# Plane's equation and vector
#f = Sketchup.active_model.entities.add_face($points[0].transform(t_temp),$points[1].transform(t_temp), $points[2].transform(t_temp))
#plan = f.plane
vz=($points[0] - $points[1]).cross($points[0] - $points[2])
a = vz.x*$points[1].x
b = vz.y*$points[1].y
c = vz.z*$points[1].z
d = a + b + c
d = 0.0 - d
plan = [vz.x,vz.y,vz.z,d]
vect = Geom::Vector3d.new($points[4] - $points[3])
 
# Dialog
(create_faces_dialog)
 
# Projections
i = 0
trans = Geom::Transformation.translation(vect)

ss.each do |current_face|
  if( current_face.typename != "Edge")
    current_edges = current_face.edges
    current_edges.each do |current_edge|
      points = current_edge.vertices
      new_pt1 = points[0].position.transform(trans)
      new_line = [points[0].position, new_pt1]
      new_pt1bis = Geom.intersect_line_plane(new_line, plan)
      new_pt2 = points[1].position.transform(trans)
      new_line = [points[1].position, new_pt2]
      new_pt2bis = Geom.intersect_line_plane(new_line, plan)
      Sketchup.active_model.active_entities.add_line( new_pt1bis, new_pt2bis )
      if(  $proj_create_faces == "Yes" )
        Sketchup.active_model.active_entities.add_face( points[0].position, points[1].position, new_pt2bis, new_pt1bis )
      end
    end
  end
end
model.commit_operation
Sketchup.send_action "selectSelectionTool:"

end

# Draw the geometry (temp)
def draw_geometry(pt1, pt2, view)
    view.draw_line(pt1, pt2)
end

end # class ProjectFacesVectorTool


# --------------------------------------------------- Push-pulls selected faces along a vector

class PushPullFacesVectorTool
def initialize
    $points=[]
    @ip1 = nil
    @ip2 = nil
    @xdown = 0
    @ydown = 0
    @state = 0
    @model = Sketchup.active_model
    @model.start_operation "Push-Pull faces along vector"
    @entities = @model.active_entities
    @ss = @model.selection
end

def activate
    Sketchup.active_model.start_operation "Push-Pull faces along vector"
    @ip1 = Sketchup::InputPoint.new
    @ip2 = Sketchup::InputPoint.new
    @ip = Sketchup::InputPoint.new
    @drawn = false
    self.reset(nil)
    
    case @ss.length
      when 0
        UI.messagebox("No selection.\nPlease select faces.")
        Sketchup.active_model.commit_operation
        Sketchup.send_action "selectSelectionTool:"
        return
      when 1
        if @ss[0].typename != "Face"
          UI.messagebox("You have selected only one " + @ss[0].typename + ".\nPlease select faces.")
          @ss.clear
          Sketchup.active_model.commit_operation
          Sketchup.send_action "selectSelectionTool:"
        end
        return  
    end
end

def deactivate(view)
    view.invalidate if @drawn
end

def onMouseMove(flags, x, y, view)
@ip.pick view, x, y
  if( @ip != @ip1 )
    view.invalidate if( @ip.display? or @ip1.display? )
    @ip1.copy! @ip
  end
        
case @state
    when 0
      view.tooltip = "First point of vector"
      Sketchup::set_status_text("First point of vector", SB_PROMPT)
    when 1
      view.tooltip = "Second point of vector"
      Sketchup::set_status_text("Second point of vector", SB_PROMPT)
    when 2
      view.tooltip = ""
      Sketchup::set_status_text("", SB_PROMPT)
  end
end

def onLButtonDown(flags, x, y, view)
  case @state
    when 0
      @xdown = x
      @ydown = y
      @ip1.pick view, x, y
      view.tooltip = @ip1.tooltip if( @ip1.valid? )
      $points.push @ip1.position
      @state = 1
      view.invalidate
    when 1
      @xdown = x
      @ydown = y
      @ip2.pick view, x, y, @ip1
      view.tooltip = @ip2.tooltip if( @ip2.valid? )
      $points.push @ip2.position
      @state = 2
      view.invalidate 
    when 2
      # Clear any inference lock
      view.lock_inference
  end #case
end

# The onLButtonUp method is called when the user releases the left mouse button.
def onLButtonUp(flags, x, y, view)
self.reset(view)
self.create_geometry(view) if @state == 2
end

def onKeyDown(key, repeat, flags, view)
if( key == CONSTRAIN_MODIFIER_KEY && repeat == 1 )
        @shift_down_time = Time.now
        # if we already have an inference lock, then unlock it
        if( view.inference_locked? )
        # calling lock_inference with no arguments actually unlocks
            view.lock_inference
        elsif( @state == 0 && @ip1.valid? )
            view.lock_inference @ip1
        elsif( @state == 1 && @ip2.valid? )
            view.lock_inference @ip2
        end
    end
end

def onKeyUp(key, repeat, flags, view)
return if not @state == 3
    if( key == CONSTRAIN_MODIFIER_KEY &&
        view.inference_locked? &&
        (Time.now - @shift_down_time) > 0.5 )
        view.lock_inference
    end
end

def onUserText(text, view)

    return if not @state == 1
    #return if not @ip2.valid?
    
    begin
        @value = text.to_l
    rescue
        # Error parsing the text
        UI.messagebox("Cannot convert #{text} to a length.")
        @value = nil
        Sketchup::set_status_text "", SB_VCB_VALUE
    end
    return if !@value

    @pt1 = @ip1.position
    @vec = @ip2.position - @pt1
    if( @vec.length == 0.0 )
        return
    end

    @vec.length = @value
    @pt2=@pt1.offset @vec
    #puts @vec
    #puts @vec.length.to_s
    #puts @pt1
    #puts @pt2
    
    $points.push(@pt2)
    self.create_geometry(view)
    self.reset(view)
end

def draw(view)
    if( @ip1.valid? )
        if( @ip1.display? )
            @ip1.draw(view)
            @drawn = true
        end
    end
    
  if $points.length == 1
    view.draw_points $points[0], 8, 1, "black"
  end
  
  if $points.length == 2
    view.line_width = 2
    view.draw_points $points[0], 8, 1, "black"
    view.draw_points $points[1], 8, 1, "black"
  end
  view.lock_inference
end

# onCancel is called when the user hits the escape key
def onCancel(flag, view)
self.reset(view)
    self.draw_geometry(view) if @state < 2
    if $points.length < 2
      # We got less than 5 points
      UI.messagebox($points.length.to_s + " points are not enough for this projection !")
      self.reset(view)
    end
end


# Reset the tool back to its initial state
def reset(view)
    #@state = 0
    Sketchup::set_status_text("First point of vector", SB_PROMPT)
    @ip1.clear
    @ip2.clear
    if( view )
        view.tooltip = nil
        view.invalidate if @drawn
    end
    @drawn = false
    @dragging = false
end

# Create new geometry when the user has picked five points.
def create_geometry(view)
#model = Sketchup.active_model
#model.start_operation "Push-Pull faces along vector"
#entities = model.active_entities
ss = sel_keep(@model.selection, ["Face"])
vect = Geom::Vector3d.new($points[1] - $points[0])
t = Geom::Transformation.translation(vect)

# Push-Pull, process each face
current_face = 0

pt_vector_1 = $points[0]
pt_vector_2 = $points[1]

delta_x = pt_vector_2[0] - pt_vector_1[0]
delta_y = pt_vector_2[1] - pt_vector_1[1]
delta_z = pt_vector_2[2] - pt_vector_1[2]

ss.each do |face| 
  pts = face.vertices
  length_pts = pts.length

  # copy the selected face
  pts2 = []
  pts.each do |pt|
    pts2.push(pt.position.transform(t))
  end
  face_copy = @entities.add_face pts2

  pts_copy = face_copy.vertices

  # create faces between pairs of vertices of each face
  i = 0
  j = 0

  0.upto(pts.length - 2) do |new_face| 
    new_face = @entities.add_face(pts[i], pts_copy[i], pts_copy[i + 1], pts[i + 1])
    i = i + 1
  end    # of upto
  # close the shape
  new_face = @entities.add_face(pts[pts.length - 1], pts_copy[pts.length - 1], pts_copy[0], pts[0])

end

@model.commit_operation
Sketchup.send_action "selectSelectionTool:"

end

# Draw the geometry (temp)
def draw_geometry(pt1, pt2, view)
    view.draw_line(pt1, pt2)
end

end # class PushPullFacesVectorTool


#-----------------------------------------------------------------------------------------------------
#                                        TOOLS
#-----------------------------------------------------------------------------------------------------

class CLineFaceIntersect

   def CLineFaceIntersect::check_selection
      # check selection
      ss = sel_keep(Sketchup.active_model.selection, ["Face"])
   end
   
   def CLineFaceIntersect::createCLineAtFaceIntersection
      Sketchup.active_model.start_operation "Construction lines at faces intersections"
      selec = sel_keep(Sketchup.active_model.selection, ["Face"])
      couples=[]
      selec.each do |current_f|
        selec.each do |f|
          if current_f != f
            #don"t skip
            line = Geom.intersect_plane_plane(current_f.plane, f.plane)
            if (line) and not couples.include?([current_f,f]) and not couples.include?([f,current_f])
              Sketchup.active_model.entities.add_cline line[0], line[1]
              couples.push([current_f,f])
            end
          end
        end
      end
   end
Sketchup.active_model.commit_operation
end # CLineFaceIntersect


#-----------------------------------------
#-----------------------------------------

class AddCpointTool
def initialize
    $ip1 = nil
    @xdown = 0
    @ydown = 0
    @state = 0
end

def activate
    $ip1 = Sketchup::InputPoint.new
    @ip = Sketchup::InputPoint.new
    @drawn = false
    self.reset(nil)
end

def deactivate(view)
    view.invalidate if @drawn
end

def onMouseMove(flags, x, y, view)
    if( @state == 0 )
       
        @ip.pick view, x, y
        if( @ip != $ip1 )
            view.invalidate if( @ip.display? or $ip1.display? )
            $ip1.copy! @ip
            view.tooltip = $ip1.tooltip
        end
    else
        #if( (x-@xdown).abs > 10 || (y-@ydown).abs > 10 )
            #@dragging = true
        #end
    end
end

def onLButtonDown(flags, x, y, view)
    if( @state == 0 )
        $ip1.pick view, x, y
        if( $ip1.valid? )
            @state = 1
            Sketchup::set_status_text "", SB_PROMPT
            @xdown = x
            @ydown = y
            self.create_geometry($ip1.position,view)
            self.reset(view)
        end
    end
    #end
    view.lock_inference
end

# The onLButtonUp method is called when the user releases the left mouse button.
def onLButtonUp(flags, x, y, view)
end

def onKeyDown(key, repeat, flags, view)
end

def onKeyUp(key, repeat, flags, view)
end

def onUserText(text, view)
end

def draw(view)
    if( $ip1.valid? )
        if( $ip1.display? )
            $ip1.draw(view)
            @drawn = true
            #view.draw_points $ip1.position, 8, 1, "black"
        end
    end
end

# onCancel is called when the user hits the escape key
def onCancel(flag, view)
    self.reset(view)
end

# Reset the tool back to its initial state
def reset(view)
    #@state = 0
    Sketchup::set_status_text("Construction point", SB_PROMPT)
    $ip1.clear
    if( view )
        view.tooltip = nil
        view.invalidate if @drawn
    end
    @drawn = false
    @dragging = false
end

# Create new geometry when the user has selected a point.
def create_geometry(p1, view)
Sketchup.active_model.start_operation "Insert construction point"
Sketchup.active_model.active_entities.add_cpoint p1
Sketchup.active_model.commit_operation
end

end # class AddCpointTool

#-----------------------------------------------------------------------------------------------------
#                                        PUSH-PULL FUNCTIONS
#-----------------------------------------------------------------------------------------------------


# --------------------------------------------------- Push-pull the selected line faces along Z
def push_pull_z_set

model = Sketchup.active_model 
model.start_operation "Push-Pull along Z"
entities = model.active_entities
ss = model.selection

if ss.empty? 
  UI.messagebox("No selection. Select face(s) please.")
  return nil
end 

  a_face_is_selected = false ;
  ss.each {|e| 
    a_face_is_selected = true if (e.is_a? Sketchup::Face)
  } 

# If the user did not select any faces, present an error message and exit 
  if (not a_face_is_selected) then 
    UI.messagebox("No faces selected.\nPlease re-try.")
    return nil
    end
 
# Get the push_pull distance 

  prompts = ["Push-pull height "]
  value = [100.cm]
  results = inputbox prompts, value, "Push-Pull options"

  return if not results
  height = results[0] ;
  pts2 = []
  newlines = []
  adjuster = [0,0,height] ;

  ss.each {|f| 
    next if not (f.is_a? Sketchup::Face)
    next if f.normal.z == 0
    pts2.clear 
    newlines.clear
    
  # copy the selected face
    f.vertices.each {|p| 
      pts2 << p.position + adjuster
      newlines << entities.add_line([p.position,pts2.last])
    }
    face_copy = entities.add_face pts2

    newlines.each {|nl|
      if nl.valid? then nl.find_faces end ; 
      } 
    }
model.commit_operation
end    # of def

# --------------------------------------------------- Extrudes a (set of) face(s) along a vector, not along the face normal
def push_pull_vector

model = Sketchup.active_model
model.start_operation "Push-Pull along vector"
entities = model.active_entities
ss = model.selection

if ss.empty? 
  UI.messagebox("No selection.")
  return nil
end
if (ss.length <= 1)
  UI.messagebox("Select vector and face(s).")
  return nil
end

# Who is the vector and who are the faces ?
i = 0
indice_edge = 0
count_edge = 0
face_selected = 0
0.upto( ss.length - 1) do |something|
  element = ss[i]
  if( element.typename == "Edge")
    indice_edge = i
    count_edge = count_edge + 1
  end
  if( element.typename == "Face")
    face_selected = 1
  end
  i = i + 1
end # of upto

if( face_selected == 0 )
  UI.messagebox("Select also a face please.")
  return nil
end

if( count_edge > 1 )
  UI.messagebox("More than a vector has been selected.")
  return nil
end

vect = ss[indice_edge]

# Remove vect from selection
ss.remove(vect) 

# process each face
current_face = 0
pts_vector = vect.vertices

pt_vector_1 = pts_vector[0].position
pt_vector_2 = pts_vector[1].position

delta_x = pt_vector_2[0] - pt_vector_1[0]
delta_y = pt_vector_2[1] - pt_vector_1[1]
delta_z = pt_vector_2[2] - pt_vector_1[2]


0.upto(ss.length - 1) do |pts| 
  pts = ss[current_face].vertices
  length_pts = pts.length

  # copy the selected face
  i = 0
  pts2 = []

  pts.each do |pt|
    pts2[i] = pt.position + [delta_x , delta_y , delta_z]
    i = i +1
  end
  face_copy = entities.add_face pts2

  pts_copy = face_copy.vertices

  # create faces between pairs of vertices of each face
  i = 0
  j = 0

  0.upto(pts.length - 2) do |new_face| 
    new_face = entities.add_face(pts[i], pts_copy[i], pts_copy[i + 1], pts[i + 1])
    i = i + 1
  end    # of upto
  # close the shape
  new_face = entities.add_face(pts[pts.length - 1], pts_copy[pts.length - 1], pts_copy[0], pts[0])

current_face = current_face + 1
end     #end of upto faces

#That's all folks
model.commit_operation
end    # of def

# --------------------------------------------------- Extrudes a (set of) face(s) along their normals
def extrude_faces_along_normal
model=Sketchup.active_model
model.start_operation "Push-Pull along normal"
ss=model.selection
$pp_dist = 1 if not $pp_dist
prompts = ["Push-Pull distance: "]
value = [$pp_dist]
result = inputbox prompts, value, "Push-Pull options"
return if not result
$pp_dist = result[0]
ss=sel_keep(ss,["Face"])
#Sketchup.active_model.selection.each{|e| if e.typename == "Face" then
ss.each do |e|
  #e.pushpull Float( dist[0])
  e.pushpull($pp_dist.to_l)
end

model.commit_operation
end
# --------------------------------------------------- Random pushpull within interval
def randomPushPull
	model=Sketchup.active_model
	model.start_operation "Random Push-Pull within interval"
	ss=model.selection
	if ss.empty? 
  		UI.messagebox("No Selection.\nPlease select faces.")
  		return nil
	end
  ss = sel_keep(ss,["Face"])

	if not $ppmax
		$ppmax=2.0 ### numbers in current units currently assumes mm - see below***
		$ppmin=1.0
	end
	prompts = ["Height (max) :  ", "Height (min) :  "]
	value =   [$ppmax, $ppmin]
	results = inputbox prompts, value, "Random Push-Pull Range (mm)" ###***
	return if not results
	$ppmax=results[0]
	$ppmin=results[1]
	ss.each do |obj|
		if obj.typename == "Face"
			ht=(($ppmax - $ppmin) * rand)
			obj.pushpull -ht #.mm ### into mm ***
		end
	end
	model.commit_operation
end

# --------------------------------------------------- Extrude Line Tool
class ExtrudeLineTool
def initialize
    $ss = Sketchup.active_model.selection
    $ip1 = nil
    $ip2 = nil
    @xdown = 0
    @ydown = 0
end

def activate
    $ip1 = Sketchup::InputPoint.new
    $ip2 = Sketchup::InputPoint.new
    @ip = Sketchup::InputPoint.new
    @drawn = false
    
    self.reset(nil)
end

def deactivate(view)
    view.invalidate if @drawn
end

def onMouseMove(flags, x, y, view)
    if( @state == 0 )
       
        @ip.pick view, x, y
        if( @ip != $ip1 )
            view.invalidate if( @ip.display? or $ip1.display? )
            $ip1.copy! @ip
            view.tooltip = $ip1.tooltip
        end
    else
        $ip2.pick view, x, y, $ip1
        view.tooltip = $ip2.tooltip if( $ip2.valid? )
        view.invalidate
        
        if( $ip2.valid? )
            length = $ip1.position.distance($ip2.position)
            Sketchup::set_status_text length.to_s, SB_VCB_VALUE
        end
        
        if( (x-@xdown).abs > 10 || (y-@ydown).abs > 10 )
            @dragging = true
        end
    end
end

def onLButtonDown(flags, x, y, view)
    if( @state == 0 )
        $ip1.pick view, x, y
        if( $ip1.valid? )
            @state = 1
            Sketchup::set_status_text "Second point", SB_PROMPT
            @xdown = x
            @ydown = y
        end
    else
        if( $ip2.valid? )
            self.create_geometry($ip1.position, $ip2.position,view)
            self.reset(view)
        end
    end
    view.lock_inference
end

# The onLButtonUp method is called when the user releases the left mouse button.
def onLButtonUp(flags, x, y, view)
    if( @dragging && $ip2.valid? )
        self.create_geometry($ip1.position, $ip2.position,view)
        self.reset(view)
    end
end

def onKeyDown(key, repeat, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY && repeat == 1 )
        @shift_down_time = Time.now
        if( view.inference_locked? )
            view.lock_inference
        elsif( @state == 0 && $ip1.valid? )
            view.lock_inference $ip1
        elsif( @state == 1 && $ip2.valid? )
            view.lock_inference $ip2, $ip1
        end
    end
end

def onKeyUp(key, repeat, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY &&
        view.inference_locked? &&
        (Time.now - @shift_down_time) > 0.5 )
        view.lock_inference
    end
end


def onUserText(text, view)

    return if not @state == 1
    return if not $ip2.valid?
    
    begin
        value = text.to_l
    rescue
        # Error parsing the text
        puts "Cannot convert #{text} to a length."
        value = nil
        Sketchup::set_status_text "", SB_VCB_VALUE
    end
    return if !value

    pt1 = $ip1.position
    vec = $ip2.position - pt1
    if( vec.length == 0.0 )
        return
    end

    vec.length = value
    #pt2 = pt1 + vec
    pt2=pt1.offset vec
    #puts (pt1.distance pt2).to_s
    
    self.create_geometry(pt1, pt2, view)
    self.reset(view)
end


def draw(view)
    if( $ip1.valid? )
        if( $ip1.display? )
            $ip1.draw(view)
            @drawn = true
            view.draw_points $ip1.position, 8, 1, "black"
        end
        
        if( $ip2.valid? )
            $ip2.draw(view) if( $ip2.display? )
            
            view.set_color_from_line($ip1, $ip2)
            self.draw_geometry($ip1.position, $ip2.position, view)
            @drawn = true
        end
    end
    view.draw_points $ip2.position, 8, 1, "black"
end

# onCancel is called when the user hits the escape key
def onCancel(flag, view)
    self.reset(view)
end


# Reset the tool back to its initial state
def reset(view)
    @state = 0
    
    Sketchup::set_status_text("First point", SB_PROMPT)
    
    $ip1.clear
    $ip2.clear
    
    if( view )
        view.tooltip = nil
        view.invalidate if @drawn
    end
    
    @drawn = false
    @dragging = false
end

# Create new geometry when the user has selected two points.
def create_geometry(p1, p2, view)

model = Sketchup.active_model
model.start_operation "Extrude lines"
entities = model.active_entities
ss = model.selection
indice_line = 0

if ss.empty? 
  UI.messagebox("No selection.")
  return nil
end

# Selection error checking
others = 0
i = 0
0.upto(ss.length - 1) do |something|
  element = ss[i]
  if( element.typename != "Edge")
    others = others + 1
  end
  i = i + 1
end  #of upto

if( others != 0 )
  UI.messagebox( others.to_s + " selected objects are not lines and will be ignored.")
end

dx = p2.x - p1.x
dy = p2.y - p1.y
dz = p2.z - p1.z

i = 0
0.upto(ss.length - 1) do |something|
  if( ss[i].typename == "Edge" )
    pts = ss[i].vertices
    v1 = pts[0].position
    v2 = pts[1].position
    v1_top = Geom::Point3d.new(v1.x+dx, v1.y+dy, v1.z+dz)
    v2_top = Geom::Point3d.new(v2.x+dx, v2.y+dy, v2.z+dz)
    new_face = Sketchup.active_model.active_entities.add_face(v1, v1_top, v2_top, v2)
    if ss[i].soft?
      new_face.edges.each do |e|
        e.smooth = true
        e.soft = true
      end
    end
  end
  #Next line
    i = i + 1
end # of upto

end

# Draw the geometry
def draw_geometry(pt1, pt2, view)
    view.draw_line(pt1, pt2)
end

end # class ExtrudeLineTool

def project_credits
UI.messagebox("Line at intersection of 2 faces: J. Gall\nRandom Push-Pull: TIG\nAll other commands: D. Bur", MB_MULTILINE, "Credits")
end