# Name :        quick_selection.rb
# Description : Mimicks the Quick Selection of AutoCad
# Author :      D. Bur
# Usage :       1.Context Menu - Quick Selection
#		2.Fill the dialog box with filters
#               3. It loops until you hit the Cancel button
#               Number of selected objects is displayed in the status bar
# Date :        16.Jan.2oo6
# Type :        tool
# History:      beta 16.Jan.2oo6  

require 'sketchup'

class Selection

attr_reader :entities
attr_writer :entities

def initialize
	@entities=[]
end

def memorize
	puts "Memorizing selection"
	self.entities=[]
	Sketchup.active_model.selection.each {|e| entities.push(e)}
	self.entities
end

def add
	puts "Adding to selection"
	Sketchup.active_model.selection.each {|e| entities.push(e)}
	self.entities.uniq!
	self.entities
end

def recall
	puts "Recalling previous selection"
	temp=[]
	self.entities.each do |ent|
		if ent.valid?
			Sketchup.active_model.selection.add(ent)
		else
			temp.push(ent)
		end
	end
	self.entities-=temp
	self.entities.uniq!
	self.entities
end

end #class Selection



def nothing_to_do
end

def build_visible_entities
puts "Parsing searchable entities"
ents = Sketchup.active_model.entities
$set_visible_entities = Set.new

for e in ents
  case $through_what
    when "Whole model"
      $set_visible_entities.insert e
    when "All visible objects"
      if e.layer.visible? and not e.hidden?
        $set_visible_entities.insert e
      end
    when "Visible layers"
      if e.layer.visible?
        $set_visible_entities.insert e
      end
    when "Visible objects"
      if not e.hidden?
        $set_visible_entities.insert e
      end
    when "Hidden Layers"
      if not e.layer.visible?
        $set_visible_entities.insert e
      end
    when "Hidden objects"
      if e.hidden?
        $set_visible_entities.insert e
      end
  end
end
end

def build_hidden_entities
ents = Sketchup.active_model.entities
$set_hidden_entities = Set.new
for e in ents
  if e.hidden?
    $set_hidden_entities.insert e
  end
end
end

def select_all_active
puts "Selecting all"
model = Sketchup.active_model
for e in model.active_entities
  $ss.add(e) if $set_visible_entities.contains? e
end
Sketchup.set_status_text("All active objects selected: " + $ss.length.to_s)
end

def select_me
puts "Selecting with single filter: " + $type
model = Sketchup.active_model
for e in model.active_entities
  if( yield(e) )
    case $how
      when "Start new"
        $ss.add(e) if $set_visible_entities.contains? e
      when "Add to current"
        $ss.add(e) if $set_visible_entities.contains? e
      when "Remove from current"
        $ss.remove(e) if $set_visible_entities.contains? e
    end
  end
end
Sketchup.set_status_text("Single filter: " + $ss.length.to_s + " objects selected.")
end

def invert_selection
puts "Inverting selection"
ents = Sketchup.active_model.entities
$set_invert = Set.new
for e in ents
  if not $ss.contains? e
    $set_invert.insert e if $set_visible_entities.contains? e
  end
end
$ss.clear
$ss.add($set_invert.to_a)
end

def put_in_set
puts "Putting in temp set"
ents = Sketchup.active_model.entities
case $set
  when "ty"
    s = $set_type
  when "gn"
    s = $set_group_name
  when "cn"
    s = $set_component_name
  when "ln"
    s = $set_layer
  when "mn"
    s = $set_material
  when "bn"
    s = $set_bmaterial
end
for e in ents
  if yield(e)
    s.insert e if $set_visible_entities.contains? e
  end
end
end

def put_in_set_material( side, m )
puts "Putting in temp set " + side + " material: " + m
ents = Sketchup.active_model.entities
case $set
  when "ty"
    s = $set_type
  when "gn"
    s = $set_group_name
  when "cn"
    s = $set_component_name
  when "ln"
    s = $set_layer
  when "mn"
    s = $set_material
  when "bn"
    s = $set_bmaterial
end

ents.each do |e|
  case side
  when "Front"
    if e.typename == "Face" and e.material and e.material.name == m
      s.insert(e) if $set_visible_entities.contains? e
    end
  when "Back"
    if e.typename == "Face" and e.back_material and e.back_material.name == m
      s.insert(e) if $set_visible_entities.contains? e
    end
  end
end
end

def combine_sets
puts "Combining sets"
a_type = $set_type.to_a
a_group = $set_group_name.to_a
a_comp = $set_component_name.to_a
a_lay = $set_layer.to_a
a_mat = $set_material.to_a
a_bmat = $set_bmaterial.to_a

result = []
if not a_type.empty? and not a_group.empty?
  result = a_type & a_group
  else
  result = a_type if not a_type.empty?
  result = a_group if not a_group.empty?
  result = [] if a_type.empty? and a_group.empty?
end
if not result.empty? and not a_comp.empty?
  result = result & a_comp
  else
  result = a_comp if result.empty? and not a_comp.empty? 
end
if not result.empty? and not a_lay.empty?
  result = result & a_lay
  else
  result = a_lay if result.empty? and not a_lay.empty?
end
if not result.empty? and not a_mat.empty?
  result = result & a_mat
  else
  result = a_mat if result.empty? and not a_mat.empty?
end
if not result.empty? and not a_bmat.empty?
  result = result & a_bmat
  else
  result = a_bmat if result.empty? and not a_bmat.empty?
end
return result
end

#------------------------------- Mixes $ss and elligible set together to get final selection
def update_selection( el )
puts "Updating selection"
el.each do |entity|
  if( $ss.include?( entity ) )
    case $how
      when "Start New"
        (nothing_to_do)
      when "Add to current"
        $ss.add(entity)
      when "Remove from current"
        $ss.remove(entity)
    end
  else
    case $how
      when "Start new"
        $ss.add(entity)
      when "Add to current"
        $ss.add(entity)
      when "Remove from current"
        $ss.remove(entity)
    end
  end
end
end

def select_in_set
puts "Parsing current set"
for e in $ss
  if( yield(e) )
    case $how
      when "Start new"
        $ss.add(e) if $set_visible_entities.contains? e
      when "Add to current"
        $ss.add(e) if $set_visible_entities.contains? e
      when "Remove from current"
        $ss.remove(e) if $set_visible_entities.contains? e
    end
  else
  case $how
      when "Start new"
        $ss.remove(e) if $set_visible_entities.contains? e
      when "Add to current"
        $ss.remove(e) if $set_visible_entities.contains? e
    end
  end
end
end

def collect_groups_names
g_list = []
Sketchup.active_model.entities.each do |g|
if g.typename == "Group" and g.name != ""
  g_list.push g.name
end
end
return g_list
end

# No need to have unused components in the list
def collect_components_names
model = Sketchup.active_model
components = model.definitions
list = []
components.each do |c|
  if c.count_instances != 0 and not c.name["Group"] and not c.name["Image"]
    list.push(c.name)
  end
end
list
end

def sel_curves
puts "Selecting curves"
ents = Sketchup.active_model.entities
ents.each do |e|
  if e.kind_of?(Sketchup::Edge) and e.curve
    case $how
      when "Start new"
        $ss.add(e.curve.edges)
      when "Add to current"
        $ss.add(e.curve.edges)
      when "Remove from current"
        $ss.remove(e.curve.edges)
    end
  end
end
end

def sel_curves_in_set
puts "Selecting curves in temp set"
ents = Sketchup.active_model.entities
ents.each do |e|
  if e.kind_of?(Sketchup::Edge) and e.curve
    case $how
      when "Start new"
        $set_type.insert(e) if $set_visible_entities.contains? e
      when "Add to current"
        $set_type.insert(e) if $set_visible_entities.contains? e
      when "Remove from current"
        $set_type.delete(e) if $set_visible_entities.contains? e
    end
  end
end
end


def select_component_by_name n
puts "Selecting components by name: " + n
ents = Sketchup.active_model.entities
ents.each do |e|
  if e.kind_of?(Sketchup::ComponentInstance) and e.definition.name == n
    case $how
      when "Start new"
        $ss.add(e) if $set_visible_entities.contains? e
      when "Add to current"
        $ss.add(e) if $set_visible_entities.contains? e
      when "Remove from current"
        $ss.remove(e) if $set_visible_entities.contains? e
    end
  end
end
end

def select_group_by_name n
puts "Selecting groups by name: " + n
ents = Sketchup.active_model.entities
ents.each do |e|
  if e.typename == "Group" and e.name == n
    case $how
      when "Start new"
        $ss.add(e) if $set_visible_entities.contains? e
      when "Add to current"
        $ss.add(e) if $set_visible_entities.contains? e
      when "Remove from current"
        $ss.remove(e) if $set_visible_entities.contains? e
    end
  end
end
end

def select_material( side, m )
puts "Selecting " + side + " material: " + m
  ents = Sketchup.active_model.entities
  ents.each do |e|
  case side
  when "Front"
    if e.typename == "Face" and e.material and e.material.name == m
    case $how
        when "Start new"
          $ss.add(e) if $set_visible_entities.contains? e
        when "Add to current"
          $ss.add(e) if $set_visible_entities.contains? e
        when "Remove from current"
          $ss.remove(e) if $set_visible_entities.contains? e
      end
    end
  when "Back"
    if e.typename == "Face" and e.back_material and e.back_material.name == m
      case $how
        when "Start new"
          $ss.add(e) if $set_visible_entities.contains? e
        when "Add to current"
          $ss.add(e) if $set_visible_entities.contains? e
        when "Remove from current"
          $ss.remove(e) if $set_visible_entities.contains? e
      end
    end
  end
  end
end

def select_material_default( side )
puts "Selecting " + side + " default "
  ents = Sketchup.active_model.entities
  ents.each do |e|
  case side
  when "Front"
    if e.typename == "Face" and not e.material
      case $how
        when "Start new"
          $ss.add(e) if $set_visible_entities.contains? e
        when "Add to current"
          $ss.add(e) if $set_visible_entities.contains? e
        when "Remove from current"
          $ss.remove(e) if $set_visible_entities.contains? e
      end
    end
  when "Back"
    if e.typename == "Face" and not e.back_material
      case $how
        when "Start new"
          $ss.add(e) if $set_visible_entities.contains? e
        when "Add to current"
          $ss.add(e) if $set_visible_entities.contains? e
        when "Remove from current"
          $ss.remove(e) if $set_visible_entities.contains? e
      end
    end
  end
  end
end

def quick_selection
model = Sketchup.active_model
model.start_operation "Quick selection"
$stop_it = false
entities = model.active_entities
# Current selection set
$ss = model.selection

# Components, Layers, Materials names
layers = model.layers
materials = model.materials
components = model.definitions
layers_names = layers.collect {|l| l.name}
materials_names = materials.collect {|m| m.name}
materials_names.unshift("<Default>")
materials_names.unshift("All")
materials_display_names = materials.collect {|m| m.display_name}
materials_display_names.unshift("Default")

groups_names = (collect_groups_names)
components_names = (collect_components_names)

#defaults values
$how = "Start new" if not $how
$through_what = "Whole model" if not $through_what
$type = "All" if not $type
$group = "All" if not $group
$component = "All" if not $component
$layer = "All" if not $layer
$material = "All" if not $material
$bmaterial = "All" if not $bmaterial

# Dialog box
what_list = ["Start new", "Add to current", "Remove from current", "Invert", "Memorize", "Recall"].join("|")
through_list = ["Whole model", "All visible objects", "Visible layers", "Visible objects", "Hidden layers", "Hidden objects"].join("|")
types_list = ["All", "Construction point", "Construction line", "Edge", "Curve", "Face", "Group", "Component", "Dimension", "Text", "Image", "Section Plane"].join("|")
groups_list = groups_names.unshift("All").join("|")
components_list = components_names.unshift("All").join("|")
layers_list = layers_names.unshift("All").join("|")
materials_list = materials_display_names.unshift("All").join("|")

dropdowns = [what_list, through_list, types_list, groups_list, components_list, layers_list, materials_list,materials_list]

prompts = ["Selection set:  ", "Search through:  ", "Objects:  ", "Group name:  ", "Component name:  ", "On layer:  ", "Front material:  ", "Back material:  "]
values = [$how,$through_what,$type,$group,$component,$layer,$material, $bmaterial]
results = inputbox prompts, values, dropdowns, "Quick selection filter"
$stop_it = true if not results
return if not results
$how = results[0]
$through_what = results[1]
$type = results[2]
$group = results[3]
$component = results[4]
$layer = results[5]
$material = results[6]
$bmaterial = results[7]

# Entities init
#Visible entities
(build_visible_entities)

# Selection init
case $how
  when "Start new"
    $ss.clear
  when "Remove from current"
    entities = $ss
end

# Invert, Memorize or Recall
case $how
  when "Invert"
    (invert_selection)
  when "Memorize"
    $selectionSet.memorize
  when "Recall"
    $selectionSet.recall
end

# Check filters
nb_filters = (results.uniq.length - 3)

# No, single or multiple filter
case nb_filters
when 0
$ss = (select_all_active)
when 1
# Object type filter
if ( $type!= "All" )
  case $type
    when "Construction point"
      select_me {|e| e.kind_of?(Sketchup::ConstructionPoint)}
    when "Construction line"
      select_me {|e| e.kind_of?(Sketchup::ConstructionLine)}
    when "Edge"
      select_me {|e| e.kind_of?(Sketchup::Edge)}
    when "Curve"
      sel_curves
    when "Face"
      select_me {|e| e.kind_of?(Sketchup::Face)}
    when "Group"
      select_me {|e| e.typename == "Group"}
    when "Component"
      select_me {|e| e.kind_of?(Sketchup::ComponentInstance)}
    when "Dimension"
      select_me {|e| (e.typename == "DimensionLinear")}
      select_me {|e| (e.typename == "DimensionRadial")}
    when "Text"
      select_me {|e| e.kind_of?(Sketchup::Text)}
    when "Image"
      select_me {|e| e.kind_of?(Sketchup::Image)}
    when "Section Plane"
      select_me {|e| e.kind_of?(Sketchup::SectionPlane)}
  end
end

# Group name filter
if ( $group != "All" )
  select_group_by_name $group
end

# Component name filter
if ( $component != "All" )
  select_component_by_name $component
end
# Layer name filter
if ( $layer != "All" )
  select_me {|e| e.layer.name == $layer}
end
# Material filter
case $material
  when "All"
    (nothing_to_do)
  when "Default"
    (select_material_default "Front")
  else
  index = materials_display_names.index($material) 
  materialname = index ? materials_names[index] : "Default"
  (select_material "Front", materialname)
end

# Back material filter
case $bmaterial
  when "All"
    (nothing_to_do)
  when "Default"
    (select_material_default "Back")
  else
  index = materials_display_names.index($bmaterial) 
  bmaterialname = index ? materials_names[index] : "Default"
  (select_material "Back", bmaterialname)
end

Sketchup.set_status_text("Single filter: " + $ss.length.to_s + " objects selected.")

else

# Combine filters
# Object type filter
# Init sets
$set_type = Set.new
$set_group_name = Set.new
$set_component_name = Set.new
$set_layer = Set.new
$set_material = Set.new
$set_bmaterial = Set.new
combined_set = []

if ( $type!= "All" )
  $set = "ty"
  case $type
    when "Construction point"
      put_in_set {|e| e.kind_of?(Sketchup::ConstructionPoint)}
    when "Construction line"
      put_in_set {|e| e.kind_of?(Sketchup::ConstructionLine)}
    when "Edge"
      put_in_set {|e| e.kind_of?(Sketchup::Edge)}
    when "Curve"
      sel_curves_in_set
    when "Face"
      put_in_set {|e| e.kind_of?(Sketchup::Face)}
    when "Group"
      put_in_set {|e| e.typename == "Group"}
    when "Component"
      put_in_set {|e| e.kind_of?(Sketchup::ComponentInstance)}
    when "Dimension"
      put_in_set {|e| (e.typename == "DimensionLinear")}
      put_in_set {|e| (e.typename == "DimensionRadial")}
      put_in_set {|e| e.kind_of?(Sketchup::Text)}
    when "Image"
      put_in_set {|e| e.kind_of?(Sketchup::Image)}
    when "Section Plane"
      put_in_set {|e| e.kind_of?(Sketchup::SectionPlane)}
  end
end

# Group name filter
if ( $group != "All" )
  $set = "gn"
  select_group_by_name $group
end

# Component name filter
if ( $component != "All" )
  $set = "cn"
  select_component_by_name $component
end
# Layer name filter
if ( $layer != "All" )
  $set = "ln"
  put_in_set {|e| e.layer.name == $layer }
end
# Material filter
case $material
  when "All"
    (nothing_to_do)
  when "Default"
    (select_material_default "Front")
  else
  $set = "mn"
  index = materials_display_names.index($material)
  materialname = index ? materials_names[index] : "Default"
  #(select_material "Front", materialname)
  (put_in_set_material "Front", materialname)
end
# Back material filter
case $bmaterial
  when "All"
    (nothing_to_do)
  when "Default"
    (select_material_default "Back")
  else
  $set = "bn"
  index = materials_display_names.index($bmaterial) 
  bmaterialname = index ? materials_names[index] : "Default"
  #(select_material "Back", bmaterialname)
  (put_in_set_material "Back", bmaterialname)
end


combined_set = (combine_sets)
(update_selection combined_set)

Sketchup.set_status_text("Multiple filter: " + $ss.length.to_s + " objects selected.")
end
model.commit_operation
end

def qss
puts "Entering Quick selection..."
model = Sketchup.active_model
#model.start_operation "Quick selection"
#$stop_it = true
#while not $stop_it
  (quick_selection)
#  end
Sketchup.set_status_text("Total of " + $ss.length.to_s + " objects selected.")
#model.commit_operation
puts "Exiting Quick selection..."
end


$selectionSet=Selection.new

# Add some menu items
#if not file_loaded?("quick_selections.rb")
  ####### Add a context menu handler that will add a menu choice to a context menu
#  UI.add_context_menu_handler do |menu|
#    menu.add_item("Quick selection...") { qss }
#    submenu=menu.add_submenu("Selection Memory")
#		submenu.add_item("Memorize") { $selectionSet.memorize }
#		submenu.add_item("Add") { $selectionSet.add }
#		submenu.add_item("Recall") { $selectionSet.recall }
#  end
#
#  UI.menu("Plugins").add_separator
#  UI.menu("Plugins").add_item("Quick selection...") { qss }
#end
file_loaded("quick_selections.rb")
