require 'sketchup.rb'
class MirrorTool

    def initialize
        @MIRROR_AT_POINT = 1
        @MIRROR_AT_LINE = 2
        @MIRROR_AT_PLANE = 3
        @numPrecision = 0.00001
    end
    def reset
        @pts = []
        @state = 0
        @transformationType = nil
        @trans = nil
        @point = nil
        @line = nil
        @linev = nil
        @plane = nil
        @planeNormal = nil
        @pointOnPlane = nil
        @curvesDone = Hash.new()
        @cloneGroup = nil
        @ip = Sketchup::InputPoint.new
        @ip1 = Sketchup::InputPoint.new
        @ip2 = Sketchup::InputPoint.new
        @ip3 = Sketchup::InputPoint.new
        @ip = Sketchup::InputPoint.new
        Sketchup::set_status_text "Mirror:  Pick First Point"###
        @drawn = false
    end
    def activate
        ###Sketchup.send_action("showRubyPanel:")
        self.reset
        ss=Sketchup.active_model.selection
        if ss.empty?
            Sketchup::set_status_text "Mirror: NO Selection !"###
    		UI.messagebox("ǰѡ   ")###v2.8
            Sketchup.active_model.select_tool nil
        end
    end
    def deactivate(view)
        view.invalidate if @drawn
        @ip1 = nil
        @ip2 = nil
        @ip3 = nil
    end
    def onMouseMove(flags, x, y, view)
        case @state
        when 0 # getting the first end point
            @ip.pick view, x, y
            if( @ip.valid? && @ip != @ip1 )
                @ip1.copy! @ip
            end
            view.invalidate
            view.tooltip = @ip.tooltip if @ip.valid?
        when 1 # getting the second end point
            @ip.pick view, x, y, @ip1
            if( @ip.valid? && @ip != @ip2 )
                @ip2.copy! @ip
                @pts[1] = @ip2.position
            end
            view.invalidate
            view.tooltip = @ip.tooltip if @ip.valid?
        when 2 # getting the third end point
            @ip.pick view, x, y, @ip2
            if( @ip.valid? && @ip != @ip3 )
                @ip3.copy! @ip
                @pts[1] = @ip3.position
            end
            view.invalidate
            view.tooltip = @ip.tooltip if @ip.valid?
        end
    end
    def onLButtonDown(flags, x, y, view)
        @ip.pick view, x, y
        if( @ip.valid? )
            case @state
            when 0
                if( @ip.valid? )
                    @pts[0] = @ip.position
                    Sketchup::set_status_text "Mirror:  Hit RETURN to Mirror at Point or Pick Point 2"
                    @state = 1
                end
            when 1
                if( @ip.valid? && @ip != @ip1 )
                    @pts[1] = @ip.position
                    @state = 2
                    Sketchup::set_status_text "Mirror:  Hit RETURN to Mirror at Line or Pick Point 3"
                end
            when 2
                if( @ip.valid? && @ip != @ip1 && @ip != @ip2 )
                    @pts[2] = @ip.position
                    @state = 3
                    mirror()
                end
            end
        end
    end
    def onCancel(flag, view)
        view.invalidate if @drawn
        reset
    end
    def draw(view)
        if( @ip.valid? && @ip.display? )
            @ip.draw(view)
            @drawn = true
        end
        if( @state == 1 )
            view.set_color_from_line(@ip1, @ip)
            view.draw(GL_LINE_STRIP, @ip1.position, @ip.position)
            @drawn = true
        elsif( @state == 2 )
            view.drawing_color = "gray"
            view.draw(GL_LINE_STRIP, @ip1.position, @ip2.position)
            view.set_color_from_line(@ip2, @ip)
            view.draw(GL_LINE_STRIP, @ip2.position, @ip.position)
            @drawn = true
        end
    end
    def onKeyDown(key, repeat, flags, view)
        if( key == CONSTRAIN_MODIFIER_KEY && repeat == 1 )
            @shift_down_time = Time.now
            if( view.inference_locked? )
                view.lock_inference
            elsif( @ip.valid? )
                view.lock_inference @ip
            end
        end
    end
    def onKeyUp(key, repeat, flags, view)
        if( key == CONSTRAIN_MODIFIER_KEY && view.inference_locked? && (Time.now - @shift_down_time) > 0.5 )
            view.lock_inference
        end
        if( key == 13)
            mirror()
            self.reset
        end
    end
    
    def mirror()
        case @state
        when 1
            @transformationType = @MIRROR_AT_POINT
            @point = @ip1.position.clone
            @trans = Geom::Transformation.scaling(@point, -1.0)
            Sketchup.active_model.start_operation "Mirror at Point"
        when 2
            @transformationType = @MIRROR_AT_LINE
            @line = [@ip1.position, @ip2.position] 
            @linev = Geom::Vector3d.new(@ip2.position.x-@ip1.position.x, @ip2.position.y-@ip1.position.y, @ip2.position.z-@ip1.position.z)
            Sketchup.active_model.start_operation "Mirror at Line"
        when 3
            @transformationType = @MIRROR_AT_PLANE
            @plane = Geom.fit_plane_to_points(@ip1.position, @ip2.position, @ip3.position) 
            @planeNormal = Geom::Vector3d.new(@plane[0], @plane[1], @plane[2])
            @planeNormal.normalize!
            @pointOnPlane = @ip1.position
            Sketchup.active_model.start_operation "Mirror at Plane"
        end
        ###
        ss=Sketchup.active_model.selection###
    	sents=[]
    	ss.each{|e|sents.push(e)}
    	copy_group=Sketchup.active_model.active_entities.add_group(sents)
        new_group=copy_group.copy
        nents=[]
        new_group.entities.each{|e|nents.push(e)}
        new_center = new_group.bounds.center.clone
        mirror_point(new_center)
        t=nil
        t = Geom::Transformation.scaling(new_center,-1,-1,-1)if @state==1
        t = Geom::Transformation.rotation(new_center,@linev,Math::PI)if @state==2
        tt=nil
        if @state==3
           xxx=@planeNormal.normalize.to_a[0].abs*-1
           yyy=@planeNormal.normalize.to_a[1].abs*-1
           zzz=@planeNormal.normalize.to_a[2].abs*-1
           xxx=1 if xxx>0
           yyy=1 if yyy>0
           zzz=1 if zzz>0
           xxx=-1 if xxx<=0 
           yyy=-1 if yyy<=0
           zzz=-1 if zzz<=0
           t = Geom::Transformation.scaling(new_center,xxx,yyy,zzz)
           tt= Geom::Transformation.rotation(new_center,@planeNormal,Math::PI)
        end#if
        new_group.transform!(t)if t###mirror
        new_group.transform!(tt)if tt###
        transVec = Geom::Vector3d.new(new_center.x-new_group.bounds.center.x, new_center.y-new_group.bounds.center.y, new_center.z-new_group.bounds.center.z) 
        t = Geom::Transformation.translation(transVec)
        new_group.transform!(t)###then move
        ### ending dialog...
    	Sketchup::set_status_text "Mirror: Erase Original Selection ?"###
    	if UI.messagebox("ǷɾԴѡ   ? ",MB_YESNO,"")==6  
    	   ### 6=YES 7=NO
    	   Sketchup.active_model.commit_operation###v2.9
      	   Sketchup.active_model.start_operation "Mirror: Erase Original Selection"
    	    Sketchup.active_model.selection.clear
      	    copy_group.erase!
    	    new_group.explode
      	    nents.each{|e|Sketchup.active_model.selection.add(e)if e.valid?}###v3.0
    	   Sketchup.active_model.commit_operation
      	else
           Sketchup.active_model.selection.clear
           new_group.explode
           copy_group.explode
           sents.each{|e|Sketchup.active_model.selection.add(e)if e.valid?}###v3.0
           Sketchup.active_model.commit_operation###v2.9
    	end#if
    	###
        self.reset
        Sketchup.send_action "selectSelectionTool:"

    end#def
    
    def mirror_point(new_p)
        if (@transformationType == @MIRROR_AT_PLANE)
            if (!new_p.on_plane?(@plane))
                mirrorp = new_p.project_to_plane(@plane)
                @trans = Geom::Transformation.scaling(mirrorp, -1.0)
                new_p.transform!(@trans)
            end
        elsif (@transformationType == @MIRROR_AT_LINE)
            if (!new_p.on_line?(@line))
                mirrorp = new_p.project_to_line(@line)
                @trans = Geom::Transformation.scaling(mirrorp, -1.0)
                new_p.transform!(@trans)
            end
        elsif (@transformationType == @MIRROR_AT_POINT)
            if (!(new_p == @point))
                new_p.transform!(@trans)
            end
        end
    end#def
    
end # class MirrorTool

### menus ##############################################################
if( not file_loaded?("mirror3.0.rb") )
###if( not file_loaded?("59-mirror.rb") )###for TIG menus only
   UI.menu("Plugins").add_item("ѡ   ") { Sketchup.active_model.select_tool MirrorTool.new }
   ###$submenu5.add_item("Mirror [Selection]") { Sketchup.active_model.select_tool MirrorTool.new }###for TIGmenus only
   ### Context menu
   UI.add_context_menu_handler do | menu |### v2.6 TIG
      if Sketchup.active_model.selection
         menu.add_separator
         menu.add_item("Mirror Selection") { Sketchup.active_model.select_tool MirrorTool.new }
      end#if
   end#do menu###
end
###file_loaded("59-mirror.rb")###for TIG menus only
file_loaded("mirror3.0.rb")

